package es.verdnatura.presentation.view.feature.ajustes.fragment

import android.app.AlertDialog
import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Context.CLIPBOARD_SERVICE
import android.content.SharedPreferences
import android.os.Bundle
//import android.preference.PreferenceManager
import androidx.preference.PreferenceManager
import android.provider.Settings
import android.view.KeyEvent
import android.view.View
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentAjustesBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnAjustesItemClickListener
import es.verdnatura.presentation.common.hideKeyboard
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.adapter.AjustesAdapter
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_ajustes.*
import kotlinx.android.synthetic.main.fragment_ajustes.splash_progress
import kotlinx.android.synthetic.main.fragment_login.*
import timber.log.Timber

class AjustesFragment : BaseFragment<FragmentAjustesBinding,AjustesViewModel>(AjustesViewModel::class) {

    private var user:String? = ""
    private var userFk:String?=""
    private var password:String? = ""
    private lateinit var customDialog: CustomDialog
    private var sectorListVO:List<SectorItemVO> = listOf()
    private var vozList:List<String> = listOf()
    private var prefs: SharedPreferences? = null
    private var ajustesAdapter: AjustesAdapter? = null
    //añadido
    private var wagonList:List<String> = listOf()
    private var tagsList:List<String> = listOf()

    companion object {
        fun newInstance() = AjustesFragment()
    }
    override fun getLayoutId(): Int = R.layout.fragment_ajustes

    override fun onCreate(savedInstanceState: Bundle?) {
        prefs = requireActivity().getSharedPreferences(PREFS_USER,0)
        customDialog = CustomDialog(requireContext())

//modificat el InitilializeAjusts
        viewModel.inititializeDefaultAjusts(prefs!!.getString(SECTORDESCRIP,getString(R.string.Sinsector)).toString(),prefs!!.getInt(SECTORFK,0),prefs!!.getInt(WAREHOUSEFK,0),prefs!!.getString(VOZ,"NO").toString(),prefs!!.getInt(WAGON,2).toString(),prefs!!.getString(TAGSTYPE,"Stickers").toString())
        super.onCreate(savedInstanceState)


    }


    override fun init() {

        requireActivity().main_bottom_navigation.visibility = View.VISIBLE
        user_text.setText(prefs!!.getString(USERFK,""))
        val versionName = requireActivity().packageManager.getPackageInfo(requireActivity().packageName,0).versionName!!
        item_version.setText(versionName)
        user = prefs!!.getString(USER,"")
        userFk =prefs!!.getString(USERFK,"")
        password = prefs!!.getString(PASSWORD,"")
        androidid_text.setText(Settings.Secure.getString(context?.contentResolver, Settings.Secure.ANDROID_ID))


        txtserver.setText(this.getDefaults("base_url",this.requireContext()))
        txtserver.setOnKeyListener(View.OnKeyListener { v, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_ENTER && event.action == KeyEvent.ACTION_UP) {
                this.setDefaults("base_url",txtserver.text.toString(),this.requireContext())
                // sergio: se comenta línea de abajo porque da fallo en la app//
                //   this.setDefaults("base_url", edittext_server.text.toString(), this.requireContext())
             this.hideKeyboard()
                //sergio:creada restartapp en BaseFragment por si es necesario utilizarla de nuevo.
                // se hace un restart por si se cambia el servidor para que tome nuevos valores

             restartapp(this)

                return@OnKeyListener false
            }

            false
        })

        super.init()

        setEvents()
    }

    fun setEvents() {
        androidid_text.setOnClickListener { copyanddrag(androidid_text.text.toString()) }
        androidid_text.setOnLongClickListener { copyanddrag(androidid_text.text.toString()) }
    }


    override fun observeViewModel() {
        with(viewModel){
            ajustesAdapter = AjustesAdapter(ajustesitem,object: OnAjustesItemClickListener{
                override fun onAjustesItemClickListener(item: AjustesItemVO) {
                    if (item.id == 0){
                        splash_progress.visibility = View.VISIBLE
                        viewModel.getSectors(user!!,password!!)
                    } else if (item.id == 5){
                        requireActivity().onBackPressed()
                    }else if (item.id == 1){
                       // //////Log.i("VERDNATURA:","Carros pulsados")
                        val listCarros : ArrayList<String> = ArrayList()
                        listCarros.add("1")
                        listCarros.add("2")
                        listCarros.add("3")
                        listCarros.add("4")
                        listCarros.add("5")
                        listCarros.add("6")
                        wagonList = listCarros

                        val array= arrayOfNulls<String>(listCarros.size)

                        showDialogCarros(array=listCarros.toArray(array))


                    }else if (item.id == 2){
                        val listVoz : ArrayList<String> = ArrayList()
                        listVoz.add("NO")
                        listVoz.add("YES")
                        vozList = listVoz
                        val array = arrayOfNulls<String>(listVoz.size)
                        showDialogVoz(array = listVoz.toArray(array))

                    }else if (item.id == 3){
                        val listTagsType : ArrayList<String> = ArrayList()
                        listTagsType.add("Stickers")
                        listTagsType.add("SmartTags")
                        tagsList = listTagsType
                        val array = arrayOfNulls<String>(listTagsType.size)
                        showItemPickerType(array = listTagsType.toArray(array))
                    }else if (item.id == 4){
                        (activity as MainActivity).onPasillerosItemClickListener( PasillerosItemVO(title = "Control de usuario de vehículo"),userFk!!)

                    }
                }
            })
            ajustess_items.adapter = ajustesAdapter
            ajustess_items.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

            loadSectorList.observe(viewLifecycleOwner, { event ->
                event.getContentIfNotHandled().notNull {
                    splash_progress.visibility = View.INVISIBLE
                    if (it.list.isNotEmpty() && it.list.get(0).isError){
                        customDialog.setTitle("Error").setDescription(it.list.get(0).errorMessage).setOkButton(getString(R.string.Close)){
                            customDialog.dismiss()
                        }.show()
                    }else{

                        if (it.list != null && it.list.size > 0){
                            val listSectores : ArrayList<String> = ArrayList()
                            it.list.forEach {
                                listSectores.add(it.description)
                            }
                            val array = arrayOfNulls<String>(listSectores.size)
                            sectorListVO = it.list
                            showDialogSector(listSectores.toArray(array))

                        }else{
                            customDialog.setTitle("Sectores").setDescription(getString(R.string.Noexistessectores)).setOkButton(getString(R.string.Close)){
                                customDialog.dismiss()
                            }.show()
                        }

                    }
                }
            })
        }
        super.observeViewModel()
    }

    private fun showDialogSector(array:Array<String>){
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.Seleccionaunsector))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            sectorListVO.forEach {
                if (it.description.equals(selected)){
                    val editor = prefs!!.edit()
                    editor.putString(SECTORDESCRIP,it.description)
                    editor.putInt(SECTORFK,it.id)
                    it.warehouseFk?.let { it1 -> editor.putInt(WAREHOUSEFK, it1) }
                    editor.apply()

                    viewModel.ajustesitem.get(0).sectorFk = it.id
                    viewModel.ajustesitem.get(0).warehouse = it.warehouseFk
                    viewModel.ajustesitem.get(0).selected = it.description
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }

        }
        val dialog = builder.create()
        dialog.show()
    }

    private fun showDialogVoz(array:Array<String>){
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.Seleccionaunsector))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            vozList.forEach {
                if (it.equals(selected)){
                    val editor = prefs!!.edit()
                    editor.putString(VOZ,it)
                    editor.apply()
                    viewModel.ajustesitem.get(2).selected = it
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }


    //sergio : guardar datos
    private fun showDialogCarros(array:Array<String>){
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.Seleccionauncarro))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            wagonList.forEach {
                if (it.equals(selected)){
                    val editor = prefs!!.edit()
                    editor.putInt(WAGON,it.toInt())
                    editor.apply()

                    viewModel.ajustesitem.get(1).selected = it
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }

    private fun showItemPickerType(array:Array<String>){
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.typeTags))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            tagsList.forEach {
                if (it.equals(selected)){
                    val editor = prefs!!.edit()
                    editor.putString(TAGSTYPE,it)
                    editor.apply()

                    viewModel.ajustesitem.get(3).selected = it
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }



    fun setDefaults(key: String?, value: String?, context: Context?) {
        val preferences: SharedPreferences = PreferenceManager.getDefaultSharedPreferences(context)
        val editor = preferences.edit()
        editor.putString(key, value)
        editor.commit()
    }

    fun getDefaults(key: String?, context: Context?): String? {
        val preferences = PreferenceManager.getDefaultSharedPreferences(context)
        return preferences.getString(key, null)
    }


    fun copyanddrag(string:String): Boolean {
        val clipboard = getActivity()?.getSystemService(CLIPBOARD_SERVICE) as ClipboardManager
        val clip: ClipData = ClipData.newPlainText("Android_id", string)
        clipboard.setPrimaryClip(clip)
        //androidid_text.performLongClick()
        //Log.i(TD,"se copia $string")
        //getString(R.string.copied).toast(requireContext())
        //setTooltipText(androidid_text, getString(R.string.copied))
        return false

    }

}
