package es.verdnatura.presentation.view.feature.articulo.fragment



import android.app.AlertDialog
import android.content.Intent
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentItemCardBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnBarcodeRowClickListener
import es.verdnatura.presentation.common.OnItemCardRowClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.loadUrl
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogTwoButtons
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.adapter.ItemCardAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemCardRowVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemCardVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemPackingType
import es.verdnatura.presentation.view.feature.calidad.model.BuyerListVO
import es.verdnatura.presentation.view.feature.imageview.activity.ImageViewActivity
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.buyers_fragment.*
import kotlinx.android.synthetic.main.fragment_item_card.*
import kotlinx.android.synthetic.main.fragment_item_card.splash_progress
import kotlinx.android.synthetic.main.toolbar.*
import timber.log.Timber

class ItemCardFragment(
    var itemFk:String = ""
) : BaseFragment<FragmentItemCardBinding,ItemCardViewModel>(ItemCardViewModel::class) {

    private var urlLarge:String = ""
    private var titleImage:String = ""
    private var user = ""
    private var password = ""
    private var warehouseFk = ""
    private var adapter : ItemCardAdapter? = null
    private var barcodeAdapter : BarcodeAdapter? = null
    private var listItemsRow:ArrayList<ItemCardRowVO> = ArrayList()
    private var listBarcodes:ArrayList<BarcodeVO> = ArrayList()
    private var itemsPackingType:ArrayList<ItemPackingType> = ArrayList()



    private var itemInfoG:ItemCardVO? = null

    companion object {
        fun newInstance(entryPoint:String) = ItemCardFragment(entryPoint)
    }
    private lateinit var customDialog: CustomDialog
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogList: CustomDialogList
    private lateinit var customDialogTwo: CustomDialogTwoButtons

    /*private val _buyersList by lazy { MutableLiveData<BuyerListVO>() }
    val buyersList: LiveData<BuyerListVO>
        get() = _buyersList*/


    override fun getLayoutId(): Int = R.layout.fragment_item_card

    override fun init() {
        itemcard_layout.visibility = View.GONE
        requireActivity().main_bottom_navigation.visibility = View.GONE
        setEvents()
        toolbar_title.text = getString(R.string.ConsultarArticulo)
        customDialog = CustomDialog(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogList = CustomDialogList(requireContext())
        customDialogTwo = CustomDialogTwoButtons(requireContext())


        if (!itemFk.isNullOrEmpty()){
            getItemCard(itemFk)
        }
        setToolBar()
        super.init()


    }



    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        val iconHistory : Drawable = resources.getDrawable(R.drawable.ic_history_black_24dp,resources.newTheme())
        listIcons.add(iconReload)
        listIcons.add(iconHistory)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    getItemCard(itemFk)
                }
                if (item == iconHistory){
                    if (itemFk.isNullOrEmpty()){
                        "Escanea un item".toast(requireActivity())
                    }else{
                            // //Log.i("VERDNATURA:","el item es ${itemInfoG!!.id}")
                        //(activity as MainActivity).onPasillerosItemClickListener( PasillerosItemVO(title = "Historico"),itemFk)
                        (activity as MainActivity).onPasillerosItemClickListener( PasillerosItemVO(title = "Historico"),itemInfoG!!.id)

                    }

                }
            }

        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents(){
        edit_itemFk.requestFocus()
        edit_itemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_itemFk.text.toString().isNullOrEmpty())
                    getItemCard(edit_itemFk.text.toString())
                edit_itemFk.setText("")
                (activity as MainActivity).hideKeyboard(edit_itemFk)
                return@setOnEditorActionListener true
            }
            false
        }


        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        itemcard_image.setOnClickListener {
            val i:Intent = Intent(activity,ImageViewActivity::class.java)
            i.putExtra("url",urlLarge)
            i.putExtra("title",titleImage)
            startActivity(i)
        }
    }

    private fun getItemCard(itemFk:String){
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        this.itemFk = itemFk
        splash_progress.visibility = View.VISIBLE
        viewModel.getItemCard(itemFk,warehouseFk,user,password,"item_card")
        viewModel.getItemPackingType(user,password,"get_ItemPackingType")
    }

    override fun observeViewModel() {
        with(viewModel){
            itemcard.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError){
                    itemcard_layout.visibility = View.GONE
                    toolbar_title.text = "ItemCard"
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    if (it.id != "0"){
                        itemcard_layout.visibility = View.VISIBLE
                        setItemCard(it)
                    }else{
                        itemcard_layout.visibility = View.GONE
                        toolbar_title.text = "ItemCard"
                        customDialog.setTitle("Sin resultados").setDescription("No hemos podido encontrar el articulo. Revisa el sector.").setOkButton("Cerrar"){
                            customDialog.dismiss()
                        }.show()
                    }

                }
            })

            itemspackinglist.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                // //Log.i("VERDNATURA:","el code es $it.code")
               // //Log.i("VERDNATURA:","la descrip es $it.description")
                createItemTypeTypeList(it.list)
            })

            response.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    getItemCard(itemFk)
                }

            })
        }
    }

    private fun createItemTypeTypeList(list: List<ItemPackingType>) {
        itemsPackingType.clear()
        list.forEach { item ->
            if (!item.isError){
                try {
                   itemsPackingType.add(ItemPackingType(item.code,item.description))

                }catch (e:Exception){}
            }
        }}




    private fun setItemCard(itemInfo:ItemCardVO){
        itemInfoG = itemInfo
        toolbar_title.text = itemInfo.id+"-"+itemInfo.longName
        titleImage = itemInfo.id+"-"+itemInfo.longName
        itemcard_image.setImageDrawable(resources.getDrawable(R.drawable.loadphoto,resources.newTheme()))
        itemcard_image.loadUrl(itemInfo.urlImage200)
        urlLarge = itemInfo.urlImage
        itemcard_tag1.text = itemInfo.size
        itemcard_tag2.text = itemInfo.value5
        itemcard_tag3.text = itemInfo.value6
        itemcard_tag4.text = itemInfo.value7 + " " + itemInfo.origin
        listItemsRow = ArrayList()
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.total),value = itemInfo.total,isEditable = false))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.Disponible),value = itemInfo.available,isEditable = false))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.Ubicado),value = itemInfo.enAltillo,isEditable = false))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.SINUBICAR),value = itemInfo.enNicho,isEditable = false, action = "itemStockUpdate"))

        //EDITABLES

        listItemsRow.add(ItemCardRowVO(title = "Artículos similares",isEditable = true, action = "itemProposal"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.DARALTA),value = "",isEditable = true, action = "itemStockUpdateAdd"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.DARBAJA),value = "",isEditable = true, action = "itemStockUpdateRemove"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.BuscarItem),value = "",isEditable = true, action = "buscarItem"))
        //listItemsRow.add(ItemCardRowVO(title = "NICHO",value = itemInfo.nicho,isEditable = true, action = "itemPlacementSave"))
       // listItemsRow.add(ItemCardRowVO(title = "RESERVA",value = itemInfo.reserva,isEditable = true, action = "itemPlacementSave"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.Grouping),value = itemInfo.grouping,isEditable = true, action = "updateGrouping"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.Packing),value = itemInfo.packing,isEditable = true, action = "updatePacking"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.MINIMO),value = itemInfo.min,isEditable = true, action = "itemSaveMin"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.Referencia),value = itemInfo.referencia,isEditable = true, action = "item_saveReference"))
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.Barcode),barcodes = itemInfo.barcodes,isEditable = true, action = "toBarcode"))

        //sergio para itemTypePacking
        listItemsRow.add(ItemCardRowVO(title = getString(R.string.tipodeencajado),value = itemInfo.itemPackingTypeFk,isEditable = true, action = "updateSector"))
       // listItemsRow.add(ItemCardRowVO(title = "Artículos similares",value = itemInfo.itemPackingTypeFk,isEditable = true, action = "itemProposal"))


        listBarcodes = itemInfo.barcodes as ArrayList<BarcodeVO>

        adapter = ItemCardAdapter(listItemsRow,object: OnItemCardRowClickListener{
            override fun onItemCardRowClickListener(item: ItemCardRowVO) {

              if (item.action == "itemProposal"){

                  (activity as MainActivity).onPasillerosItemClickListener(
                      PasillerosItemVO(
                          title = "Buscar artículos similares"
                      ), itemFk
                  )

              }
                  else{
                if (item.action == "updateSector") {
                    showDialogSelectItemTypePacking()
                } else {
                    if (item.action != "toBarcode" && item.action != "buscarItem" && item.action != "itemStockUpdateAdd" && item.action != "itemStockUpdateRemove") {
                        customDialogInput.setTitle(item.title!!)
                            .setDescription("Valor actual: " + item.value!!)
                            .setOkButton("Guardar") {
                                (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                editItemCardRow(item, customDialogInput.getValue())
                                customDialogInput.dismiss()

                            }.setKoButton("Cancelar") {
                            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                            customDialogInput.dismiss()
                        }.setValue("").show()
                        customDialogInput.getEditText().requestFocus()
                        customDialogInput.getEditText()
                            .setOnEditorActionListener { v, actionId, event ->
                                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                                    if (!customDialogInput.getValue().isNullOrEmpty())
                                        editItemCardRow(item, customDialogInput.getValue())
                                    customDialogInput.setValue("")
                                    customDialogInput.dismiss()
                                    (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                    return@setOnEditorActionListener true
                                }
                                false
                            }
                    } else if (item.action == "toBarcode") {
                        //BARCODE ROW
                        prepareBarcodeDialog(item)
                    } else if (item.action == "buscarItem") {
                        (activity as MainActivity).onPasillerosItemClickListener(
                            PasillerosItemVO(
                                title = "Buscar item"
                            ), itemFk
                        )
                    } else if (item.action == "itemStockUpdateAdd") {
                        customDialogInput.setTitle(item.title!!)
                            .setDescription("Indica la cantidad a dar de alta")
                            .setOkButton("Guardar") {
                                (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                editItemCardRow(item, customDialogInput.getValue())
                                customDialogInput.dismiss()

                            }.setKoButton("Cancelar") {
                            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                            customDialogInput.dismiss()
                        }.setValue("").show()
                        customDialogInput.getEditText().requestFocus()
                        customDialogInput.getEditText()
                            .setOnEditorActionListener { v, actionId, event ->
                                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                                    if (!customDialogInput.getValue().isNullOrEmpty())
                                        editItemCardRow(item, customDialogInput.getValue())
                                    customDialogInput.setValue("")
                                    customDialogInput.dismiss()
                                    (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                    return@setOnEditorActionListener true
                                }
                                false
                            }
                    } else if (item.action == "itemStockUpdateRemove") {
                        customDialogInput.setTitle(item.title!!)
                            .setDescription("Indica la cantidad a dar de baja ")
                            .setOkButton("Guardar") {
                                (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                editItemCardRow(item, customDialogInput.getValue())
                                customDialogInput.dismiss()

                            }.setKoButton("Cancelar") {
                            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                            customDialogInput.dismiss()
                        }.setValue("").show()
                        customDialogInput.getEditText().requestFocus()
                        customDialogInput.getEditText()
                            .setOnEditorActionListener { v, actionId, event ->
                                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                                    if (!customDialogInput.getValue().isNullOrEmpty())
                                        editItemCardRow(item, customDialogInput.getValue())
                                    customDialogInput.setValue("")
                                    customDialogInput.dismiss()
                                    (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                    return@setOnEditorActionListener true
                                }
                                false
                            }
                    }
                }
            }
            }
        })
        itemcard_recyclerview.adapter = adapter
        itemcard_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

        scroll_view.post({
            val position = IntArray(2)
            if (itemcard_image != null){
                itemcard_image.getLocationInWindow(position)
                scroll_view.scrollTo(0, position[1])
            }
        })

    }

    private fun editItemCardRow(item:ItemCardRowVO,value:String){

        when (item.action){
           // cau:sergio itemTrash x itemStock Update . No se utiliza esta funicón
               // "itemStockUpdate" -> prepareItemStockUpdate(item,value)
            "itemStockUpdateAdd" -> prepareItemStockUpdate2(item,value)
            "itemStockUpdateRemove" -> prepareItemStockUpdate2(item,value)
            "itemPlacementSave" -> viewModel.itemPlacementSave(itemFk = itemFk, warehouseFk = warehouseFk, user = user, password = password, value = value,"itemPlacementSave")
            "updateGrouping" -> viewModel.updateGrouping(itemFk = itemFk, user = user, password = password, value = value,warehouseFk = warehouseFk,"buy_updateGrouping")
            "updatePacking" -> viewModel.updatePacking(itemFk = itemFk, user = user, password = password, value = value,warehouseFk = warehouseFk,"buy_updateGrouping")
            "itemSaveMin" -> viewModel.itemSaveMin(itemFk = itemFk, user = user, password = password, value = value,"itemSaveMin")
            "item_saveReference"->viewModel.item_saveReference(itemFk = itemFk, user = user, password = password, value = value,"item_saveReference")


        }
        if(item.action != "itemStockUpdate")
           // changeOfflineValue(item,value, listBarcodes)
               showProgress()
    }

    //sergio: changeOffLineValue solo hace visible el progress

    private fun showProgress(){
        splash_progress.visibility = View.VISIBLE
    }

   /* private fun changeOfflineValue(item:ItemCardRowVO,newValue:String, barcodes:List<BarcodeVO>){

        splash_progress.visibility = View.VISIBLE

       //sergio: esta parte ya estaba comentada
         /*var i = 0
        var position = 0
        listItemsRow.forEach {
            if (it.title == item.title) {
                position = i
            }
            i = i.plus(1)
        }
        if (listItemsRow.get(position).title != "BARCODE") listItemsRow.get(position).value = newValue
        else listItemsRow.get(position).barcodes = barcodes
        adapter?.notifyDataSetChanged()*/
    }*/

   /* private fun prepareItemStockUpdate(itemB:ItemCardRowVO,value:String){

        //sergio: cau: no se llama a esta función itemTrash itemStockUpdate
        try{

            if (itemB.value!!.toInt() > value.toInt()) {
                customDialogTwo.setTitle(itemB.title!!).setDescription("Indica la causa de eliminar stock:").setOkButton("Falta"){
                    viewModel.itemStockUpdate(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "0")
                    changeOfflineValue(itemB,value, listBarcodes)
                    customDialogTwo.dismiss()

                }.setOkButtonTwo("Basura"){
                    viewModel.itemStockUpdate(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "1")
                    changeOfflineValue(itemB,value, listBarcodes)
                    customDialogTwo.dismiss()

                }.setKoButton("Cancelar"){

                    customDialogTwo.dismiss()
                }.show()
            }else{
                changeOfflineValue(itemB,value, listBarcodes)
                viewModel.itemStockUpdate(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "0")
            }
        }catch (e:Exception){
            itemcard_layout.visibility = View.GONE
            customDialog.setTitle("Error").setDescription("El elemento introducido no es un número").setKoButton("Cancelar"){
                customDialog.dismiss()
            }.show()
        }

    }*/

    private fun prepareItemStockUpdate2(itemB:ItemCardRowVO,value:String){
        try{

            if (itemB.action == "itemStockUpdateRemove") {
                customDialogTwo.setTitle(itemB.title!!).setDescription("Indica la causa de eliminar stock:").setOkButton("Falta"){
                    viewModel.itemTrash(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "0","itemTrash")
                    //changeOfflineValue(itemB,value, listBarcodes)
                    showProgress()
                    customDialogTwo.dismiss()

                }.setOkButtonTwo("Basura"){
                    viewModel.itemTrash(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "1","itemTrash")
                    //changeOfflineValue(itemB,value, listBarcodes)
                    showProgress()
                    customDialogTwo.dismiss()

                }.setKoButton("Cancelar"){

                    customDialogTwo.dismiss()
                }.show()
            }else{
                //changeOfflineValue(itemB,value, listBarcodes)
                    showProgress()
                viewModel.itemTrash(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = (value.toInt() * -1).toString(),isTrash = "0","itemTrash")
            }
        }catch (e:Exception){
            itemcard_layout.visibility = View.GONE
            customDialog.setTitle("Error").setDescription("El elemento introducido no es un número").setKoButton("Cancelar"){
                customDialog.dismiss()
            }.show()
        }
    }

    private fun prepareBarcodeDialog(itemB:ItemCardRowVO){

        customDialogList.setTitle("Barcodes").setOkButton("Guardar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
            if (!customDialogList.getValue().isNullOrEmpty())
                updateBarcode(customDialogList.getValue(),"0",itemB)
            customDialogList.dismiss()

        }.setKoButton("Cerrar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
            customDialogList.dismiss()
        }.setValue("").show()


        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
        customDialogList.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogList.getValue().isNullOrEmpty())
                    updateBarcode(customDialogList.getValue(),"0",itemB)
                customDialogList.setValue("")
                customDialogList.dismiss()
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        barcodeAdapter = BarcodeAdapter(listBarcodes,object: OnBarcodeRowClickListener{
            override fun onBarcodeRowClickListener(item: BarcodeVO) {
                updateBarcode(item.code!!,"1",itemB)
            }
        })
        customDialogList.getRecyclerView().adapter = barcodeAdapter

        customDialogList.getRecyclerView().layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

    private fun updateBarcode(code:String,delete:String,item:ItemCardRowVO){
        if (delete == "1"){
            var i = 0
            var posDelete = 0
            listBarcodes.forEach {
                if (it.code == code)
                    posDelete = i
                i+=1
            }
            listBarcodes.removeAt(posDelete)
        }else{
            listBarcodes.add(BarcodeVO(code))
        }
        viewModel.barcodesEdit(itemFk = itemInfoG!!.id, user = user, password = password, value = code, delete = delete,"itemBarcode_update")
       // viewModel.barcodesEdit(itemFk = itemFk, user = user, password = password, value = code, delete = delete,"itemBarcode_update")
        barcodeAdapter!!.notifyDataSetChanged()
        //changeOfflineValue(item,"", listBarcodes)
        showProgress()
    }

    private fun showDialogSelectItemTypePacking(){

       var array= arrayOfNulls<String>(itemsPackingType.size)
        for (indice in array.indices){
            array[indice]=itemsPackingType[indice].code + " : "+itemsPackingType[indice].description
        }
        val builder = AlertDialog.Builder(this.context)
        builder.setTitle(getString(R.string.mensajeseleccionencajado))
            .setItems(array
            ) { _, position ->
                updateItemPackingType(itemsPackingType[position].code.toString())
            }

        builder.create().show()
        }

    private fun updateItemPackingType(itemPackingType:String) {
        splash_progress.visibility = View.VISIBLE
        viewModel.updatePackingType(this.itemFk,user,password,itemPackingType,"item_updatePackingType")
    }

}


