package es.verdnatura.presentation.view.feature.articulo.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.Gravity
import android.view.View
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.appcompat.widget.TooltipCompat
import androidx.core.content.ContextCompat
import androidx.core.content.res.ResourcesCompat.getColor
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentInventaryBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnItemProposalClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.feature.articulo.adapter.ItemProposalAdapter
import es.verdnatura.presentation.view.feature.articulo.model.ItemProposal
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.inventario.fragment.SearchBuyerModel
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_buscar_item.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*
import kotlin.collections.ArrayList

class ItemProposalFragment(
    var itemFk: String = ""
) : BaseFragment<FragmentInventaryBinding, ItemCardViewModel>(
    ItemCardViewModel::class
) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter: ItemProposalAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private var listInvetory: ArrayList<ItemProposal> = ArrayList()
    private var listInvetoryAux: ArrayList<ItemProposal> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var reload = false
    private var hideLoad = true
    private var layoutManager: LinearLayoutManager? = null
    private var firstVisiblePosition = 0
    private var filter = "TRUE"
    private var prefs: SharedPreferences? = null

    companion object {
        fun newInstance(entryPoint: String) = ItemProposalFragment(entryPoint)
    }

    //override fun getLayoutId(): Int = R.layout.fragment_inventary
    override fun getLayoutId(): Int = R.layout.fragment_itemproposal

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        prefs = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs!!.getString(USER, "").toString()
        password = prefs!!.getString(PASSWORD, "").toString()
        warehouseFk = prefs!!.getInt(WAREHOUSEFK, 60).toString()

        viewModel.item_getSimilar(
            itemFk,
            warehouseFk,
            vShipped = currentDay(),
            vIsShowedByType = "TRUE",
            user = user,
            password = password,"item_getSimilar"
        )
        super.onCreate(savedInstanceState)
    }

    private fun currentDay(): String {

        val calendar = Calendar.getInstance()
        val format = SimpleDateFormat("yyyy-MM-dd")
        System.out.println(format.format(calendar.time))

        /* val current = LocalDateTime.now()
         val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
         val currentDay = current.format(formatter)*/


        return format.format(calendar.time)
    }

    override fun init() {

        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        splash_progress.visibility = View.VISIBLE
        toolbar_title.text = getString(R.string.ArticleSimilar) + ":" + itemFk
        setToolBar()
        setEvents()

        super.init()
    }

    private fun setToolBar() {
        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconReload: Drawable =
            resources.getDrawable(R.drawable.ic_autorenew_black_24dp, resources.newTheme())
        val iconFilter: Drawable =
            resources.getDrawable(R.drawable.filter_outline, resources.newTheme())
        /*  listIcons.add(iconReload)*/
        listIcons.add(iconFilter)


        toolbar_icons.adapter = ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload) {
                    splash_progress.visibility = View.VISIBLE
                    firstVisiblePosition = layoutManager?.findFirstVisibleItemPosition() ?: 0
                    viewModel.item_getSimilar(
                        itemFk,
                        warehouseFk,
                        vIsShowedByType = filter,
                        vShipped = currentDay(),
                        user = user,
                        password = password
                    ,callFunction = "item_getSimilar")

                }
                if (item == iconFilter) {

                    splash_progress.visibility = View.VISIBLE
                    firstVisiblePosition = layoutManager?.findFirstVisibleItemPosition() ?: 0


                    if (filter.equals("TRUE")) {
                        showMessage("Sin Filtrar por tipo de flor")
                        filter = "FALSE"
                        iconFilter.setTint(
                            ContextCompat.getColor(
                                context!!,
                                R.color.verdnatura_white
                            )
                        )

                    } else {
                        filter = "TRUE"
                        showMessage("Filtrado por tipo de flor")
                        iconFilter.setTint(
                            ContextCompat.getColor(
                                context!!,
                                R.color.verdnatura_pumpkin_orange
                            )
                        )
                    }
                    viewModel.item_getSimilar(
                        itemFk,
                        warehouseFk,
                        vIsShowedByType = filter,
                        vShipped = currentDay(),
                        user = user,
                        password = password ,
                        callFunction = "item_getSimilar")
                }

            }

        })
        toolbar_icons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    private fun showMessage(message:String){
        val mytoast = Toast.makeText(
            requireContext(),
            message,
            Toast.LENGTH_SHORT
        )
        mytoast.setGravity(Gravity.TOP or Gravity.RIGHT, 0, -100)
        mytoast.show()
    }
    private fun setEvents() {
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

    }

    override fun onPause() {
        super.onPause()
        firstVisiblePosition = layoutManager?.findFirstVisibleItemPosition() ?: 0
    }

    override fun onResume() {
        super.onResume()
        if (location_recyclerview.layoutManager != null) {
            if (firstVisiblePosition <= listInvetory.size) {
                location_recyclerview.layoutManager!!.scrollToPosition(firstVisiblePosition)
            }
        }
    }

    override fun observeViewModel() {
        with(viewModel) {

            itemProposallist.observe(viewLifecycleOwner, Observer {
                if (hideLoad) {
                    splash_progress.visibility = View.GONE
                } else {
                    hideLoad = true
                }

                listInvetory = ArrayList()
                listInvetoryAux = ArrayList()
                it.list.forEach {
                    if (it.longName != "0") {
                        listInvetory.add(it)
                        listInvetoryAux.add(it)
                    }
                }

                adapter = ItemProposalAdapter(listInvetory, object : OnItemProposalClickListener {
                    override fun onItemProposalClickListener(item: ItemProposal) {

                        customDialog.setTitle(item.itemFk + "\n" + item.longName + " " + item.subName)
                            .setDescription("Cantidad real(" + item.available + ")")
                            .setOkButton("Prueba") {

                                customDialog.dismiss()
                            }.setKoButton("Cancelar") {
                                // customDialogInput.dismiss()
                                customDialog.dismiss()
                            }.show()

                    }
                }, onPasillerosItemClickListener = pasillerosItemClickListener!!)

                location_recyclerview.adapter = adapter
                layoutManager =
                    LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
                location_recyclerview.layoutManager = layoutManager

                if (firstVisiblePosition <= listInvetory.size) {
                    location_recyclerview.layoutManager!!.scrollToPosition(firstVisiblePosition)
                }

            })

        }

    }
}


