package es.verdnatura.presentation.view.feature.buffer.fragment

import android.content.Context
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import es.verdnatura.R
import es.verdnatura.databinding.FragmentBufferBinding
import es.verdnatura.domain.ConstAndValues
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnCollectionSelectedListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import kotlinx.android.synthetic.main.fragment_controlador.*
import kotlinx.android.synthetic.main.toolbar.*

class BufferFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentBufferBinding, BufferFragmentViewModel>(BufferFragmentViewModel::class) {


    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var goBack: Boolean = false
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    override fun getLayoutId(): Int = R.layout.fragment_buffer
    private lateinit var customDialog: CustomDialog
    private var type = ""

    companion object {
        fun newInstance(entryPoint: String) = BufferFragment(entryPoint)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }


    override fun init() {


       // Log.i("VERDNATURA:", "entrypoint es : ${entryPoint}")

        splash_progress.visibility = View.GONE
        customDialog = CustomDialog(requireContext())
        //toolbar_title.text = getString(R.string.verticket)
        setToolbar()
        setEvents()
        super.init()
    }

    private fun setToolbar() {

        toolbar_title.text = entryPoint
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        type = ConstAndValues.PRECHECKER
        super.onCreate(savedInstanceState)
    }

    private fun setEvents() {

        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                goBack = false
                if (!scan_input.text.isNullOrEmpty()) {
                    splash_progress.visibility = View.VISIBLE

                    if (entryPoint.equals("Vaciado buffer")){
                    viewModel.buffer_setTypeByName(user,password,scan_input.text.toString(),"UNLOADING","buffer_setTypeByName")
                    }
                    if (entryPoint.equals("Llenado buffer")){
                        viewModel.buffer_setTypeByName(user,password,scan_input.text.toString(),"LOADING","buffer_setTypeByName")
                    }
                    if (entryPoint.equals("Acumulación buffer")){
                        viewModel.buffer_setTypeByName(user,password,scan_input.text.toString(),"ACCUMULATION","buffer_setTypeByName")
                    }
                    if (entryPoint.equals("Desactivado buffer")){
                        viewModel.buffer_setTypeByName(user,password,scan_input.text.toString(),"DISABLED","buffer_setTypeByName")
                    }

                }
                scan_input.setText("")
                (activity as MainActivity).hideKeyboard(scan_input)
                return@setOnEditorActionListener true
            }
            false
        }
    }

    override fun observeViewModel() {

        with(viewModel) {
            itembuffertype.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError) {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                   // customDialog.setTitle("Mensaje").setDescription(it.errorMessage)

                    customDialog.setTitle("Mensaje").setDescription("Operación "+entryPoint.lowercase() + " realizada correctamente")
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                            backButton.performClick()
                        }.show()

                }

            })
        }

    }


}

