package es.verdnatura.presentation.view.feature.buscaritem.fragment

import android.content.Context
import android.content.SharedPreferences
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentBuscarItemAllBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.buscaritemall.adapter.LocationAdapterAll
import es.verdnatura.presentation.view.feature.buscaritemall.fragment.BuscarItemAllViewModel
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_buscar_item.*
import kotlinx.android.synthetic.main.fragment_buscar_item.edit_itemFk
import kotlinx.android.synthetic.main.fragment_buscar_item.location_recyclerview
import kotlinx.android.synthetic.main.fragment_buscar_item.splash_progress
import kotlinx.android.synthetic.main.fragment_buscar_item_all.*
import kotlinx.android.synthetic.main.fragment_inventary.*
import kotlinx.android.synthetic.main.toolbar.*

class BuscarItemAllFragment(
    var itemFk:String = ""
) : BaseFragment<FragmentBuscarItemAllBinding, BuscarItemAllViewModel>(BuscarItemAllViewModel::class) {

    private var user = ""
    private var password = ""
    private var warehouseFk = ""
    private var adapter : LocationAdapterAll? = null
    private lateinit var customDialog: CustomDialog
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null

    companion object {
        fun newInstance(entryPoint:String) = BuscarItemAllFragment(entryPoint)
    }

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }
    override fun getLayoutId(): Int = R.layout.fragment_buscar_item_all

    override fun init() {
        requireActivity().main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = getString(R.string.getubicaition)
        customDialog = CustomDialog(requireContext())
        setEvents()
        if (!itemFk.isNullOrEmpty()){
            getLocations(itemFk)
        }
        super.init()
    }


    private fun setEvents(){
        edit_itemFk.requestFocus()
        edit_itemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_itemFk.text.toString().isNullOrEmpty())
                    getLocations(edit_itemFk.text.toString())
                edit_itemFk.setText("")
                (activity as MainActivity).hideKeyboard(edit_itemFk)
                return@setOnEditorActionListener true
            }
            false
        }

        edit_itemFk.setOnKeyListener { v, keyCode, event ->
            //Log.i("VERDNATURA:","text")
            return@setOnKeyListener false}
           /*if (filter_itemFk.text.toString().isNullOrEmpty()){
                if (listInvetory.size != listInvetoryAux.size){
                    listInvetory.removeAll(listInvetoryAux)
                    listInvetoryAux.forEach {
                        listInvetory.add(it)
                    }

                }

            }else{
                listInvetory.removeAll(listInvetoryAux)
                listInvetoryAux.forEach {
                    if ((it.itemFk.contains(filter_itemFk.text.toString(),true)) || (it.producer?.contains(filter_itemFk.text.toString(),true))){
                        listInvetory.add(it)
                    }
                }


            }
            LocationAdapterAll!!.notifyDataSetChanged()
            return@setOnKeyListener false
        }*/




        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

    }

    private fun getLocations(itemFk:String){
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        this.itemFk = itemFk
        splash_progress.visibility = View.VISIBLE
        viewModel.getLocationsAll(user,password,itemFk,"getItemUbication")
    }

    override fun observeViewModel() {
        with(viewModel){
            loadLocationListAll.observe(viewLifecycleOwner, Observer { event ->

                event.getContentIfNotHandled().notNull {
                    splash_progress.visibility = View.GONE
                    adapter = LocationAdapterAll(it.list,pasillerosItemClickListener!!)

                    location_all_recyclerview.adapter = adapter
                    location_all_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

                    var totalVisible:Int = 0
                    it.list.forEach {
                        if (!it.visible.isNullOrEmpty())
                            totalVisible += it.visible.toInt()
                    }
                    toolbar_title.text = "Item: "+itemFk+ " Total visible: "+totalVisible


                }
            })
        }
    }

}
