package es.verdnatura.presentation.view.feature.buscaritemall.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetBuscarItemAllUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageError
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.view.feature.buscaritemall.model.ItemLocationAll
import es.verdnatura.presentation.view.feature.buscaritemall.model.LocationListAll

import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class BuscarItemAllViewModel(context: Context) : BaseViewModel() {
    private val getBuscarItemAllUserCase: GetBuscarItemAllUserCase = GetBuscarItemAllUserCase(context)


    private val _locationListAll by lazy { MutableLiveData<LocationListAll>() }
    val locationListAll: LiveData<LocationListAll>
        get() = _locationListAll


    val loadLocationListAll = Transformations.map(_locationListAll) { Event(it) }


    fun getLocationsAll(user:String,password:String,itemFk:String,callFunction:String){
        getBuscarItemAllUserCase.searchItemsUbicadorAll(user,password,itemFk).enqueue(object : Callback<List<ItemLocationAll>> {
            override fun onFailure(call: Call<List<ItemLocationAll>>, t: Throwable) {
                val listError:ArrayList<ItemLocationAll> = ArrayList()
                listError.add(ItemLocationAll(isError = true,errorMessage =getMessageFromAllResponse(callFunction,t.message!!)))
                _locationListAll.value = LocationListAll(listError)
            }

            override fun onResponse(
                call: Call<List<ItemLocationAll>>,
                response: Response<List<ItemLocationAll>>
            ) {
                if (response.body() != null){
                    _locationListAll.value = response.body()?.let { LocationListAll(it) }
                }else{
                    val listError:ArrayList<ItemLocationAll> = ArrayList()
                    listError.add(ItemLocationAll(isError = true,errorMessage = getMessageFromAllResponse(callFunction,response.message())))
                    _locationListAll.value = LocationListAll(listError)
                }
            }

        })
    }

}
