package es.verdnatura.presentation.view.feature.calidad.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.View
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.BuyersFragmentBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnAjustesItemClickListener
import es.verdnatura.presentation.common.OnBuyerSelectedListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.adapter.AjustesAdapter
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.calidad.model.BuyerVO
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.buyers_fragment.*
import kotlinx.android.synthetic.main.buyers_fragment.splash_progress
import kotlinx.android.synthetic.main.fragment_ajustes.*
import kotlinx.android.synthetic.main.toolbar.*

class BuyersFragment : BaseFragment<BuyersFragmentBinding,BuyersViewModel>(BuyersViewModel::class) {


    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private lateinit var customDialog: CustomDialog
    private var onBuyerSelectedListener : OnBuyerSelectedListener? = null
    private var buyersAdapter: AjustesAdapter? = null

    companion object {
        fun newInstance() = BuyersFragment()
    }

    override fun getLayoutId(): Int = R.layout.buyers_fragment

    override fun onAttach(context: Context) {
        if (context is OnBuyerSelectedListener) onBuyerSelectedListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        viewModel.itemShelvingBuyerGet(user = user,password = password,"itemShelving_BuyerGet")
        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialog = CustomDialog(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        splash_progress.visibility = View.VISIBLE
        toolbar_title.text = getString(R.string.quality)
        setToolBar()
        setEvents()
        super.init()
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        listIcons.add(iconReload)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.itemShelvingBuyerGet(user = user,password = password,"itemShelving_BuyerGet")

                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }
    }

    override fun observeViewModel() {
        with(viewModel) {
            buyersList.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                createBuyerList(it.list)
            })
        }
    }

    private fun createBuyerList(list:List<BuyerVO>){
        var buyers = ArrayList<AjustesItemVO>()
        list.forEach { buyer ->
            if (!buyer.isError){
                try {
                    buyers.add(
                        AjustesItemVO(
                            id = buyer.userFk.toInt(),
                            title = buyer.nickname,
                            sectorFk = 0,
                            warehouse = 0
                        )
                    )
                }catch (e:Exception){}
            }
        }
        buyersAdapter = AjustesAdapter(buyers,object: OnAjustesItemClickListener {
            override fun onAjustesItemClickListener(item: AjustesItemVO) {
                onBuyerSelectedListener?.onBuyerSelected(item.id.toString())
            }
        })
        buyers_items.adapter = buyersAdapter
        buyers_items.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

}