package es.verdnatura.presentation.view.feature.calidad.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.QaualityFragmentBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.calidad.adapter.ItemBuyersAdapter
import es.verdnatura.presentation.view.feature.calidad.model.ItemBuyerVO
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_collection.*
import kotlinx.android.synthetic.main.qauality_fragment.*
import kotlinx.android.synthetic.main.qauality_fragment.scan_input
import kotlinx.android.synthetic.main.qauality_fragment.splash_progress
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class QaualityFragment(
    var userFk : String = "0"
) : BaseFragment<QaualityFragmentBinding,QaualityViewModel>(QaualityViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private lateinit var customDialog: CustomDialog
    private var buyersAdapter: ItemBuyersAdapter? = null
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var onStarSelectedListener : OnStarSelectedListener? = null
    private var listItems:List<ItemBuyerVO> = listOf()
    private var lm : LinearLayoutManager? = null
    private var storedPosition: Int = 0
    private var goBack:Boolean = false


    companion object {
        fun newInstance(userFk:String) = QaualityFragment(userFk)
    }

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.qauality_fragment

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        viewModel.itemShelvingBuyerTask(user = user,password = password,userFk = userFk,"itemShelving_BuyerTask")
        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialog = CustomDialog(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        splash_progress.visibility = View.VISIBLE
        toolbar_title.text = "itemShelving_BuyerTask"
        setToolBar()
        setEvents()
        super.init()
    }

    override fun onPause() {
        super.onPause()
        goBack = true
        storedPosition = lm?.findFirstVisibleItemPosition() ?: 0
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        listIcons.add(iconReload)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.itemShelvingBuyerTask(user = user,password = password,userFk = userFk,"itemShelving_BuyerTask")
                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        //ESCANER =========
        hideKeyboards()
        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!scan_input.text.toString().isNullOrEmpty()){
                    findSale(scan_input.text.toString())
                }
                scan_input.setText("")
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            true
        }
    }

    private fun hideKeyboards(){
        try{
            requireActivity().hideKeyboard()
        }catch (e:Exception){}
    }

    override fun observeViewModel() {
        with(viewModel) {
            buyersList.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                createBuyerList(it.list)
            })

            response.observe(viewLifecycleOwner, Observer {
                viewModel.itemShelvingBuyerTask(user = user,password = password,userFk = userFk,"itemShelving_BuyerTask")
            })
        }
    }

    private fun createBuyerList(list:List<ItemBuyerVO>){
        list.forEach { item ->
            if (!goBack){
                if (item.created != null && item.created != ""){
                    item.created = getCalendarFromDate(item.created).convertToDateString()
                }
            }

            if (item.visible.contains("Visible:",ignoreCase = true)){
                item.visible = item.visible
            }else{
                item.visible = "Visible: "+item.visible
            }

            item.iconResourceOk = R.drawable.ic_start_ok
            item.iconResourceKo = R.drawable.ic_star_ko
        }
        goBack = false
        listItems = list
        buyersAdapter = ItemBuyersAdapter(listItems,pasillerosItemClickListener!!,object: OnStarSelectedListener{
            override fun onStarSelected(vId: String, vStars: String) {
                splash_progress.visibility = View.VISIBLE
                viewModel.itemShelvingStarsUpdate(user,password,vId,vStars)
            }

        })
        shelving_buyers_items.adapter = buyersAdapter
        lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        shelving_buyers_items.layoutManager = lm

        if (storedPosition < listItems.size)
            goToPosition(storedPosition)
    }

    private fun getCalendarFromDate(date:String): Calendar {
        val sdf = SimpleDateFormat("yyyy-MM-dd")
        val cal = Calendar.getInstance()
        cal.time = sdf.parse(date)
        return cal
    }

    private fun findSale(matricula:String){
        var isSearch = false
        var index = 0
        var position = 0
        listItems.forEach { item ->
            if (item.shelvingFk.uppercase() == matricula.uppercase()){
                item.showStars = true
                if (!isSearch){
                    position = index
                    isSearch = true
                }
            }else{
                item.showStars = false
            }
            index += 1
        }

        storedPosition = position
        goToPosition(position)

        buyersAdapter?.notifyDataSetChanged()
    }

    private fun goToPosition(position:Int){
        if (shelving_buyers_items != null){
            shelving_buyers_items.addViewObserver {
                lm!!.scrollToPositionWithOffset(position,0)
            }
        }
    }

}