package es.verdnatura.presentation.view.feature.collection.adapter

import android.content.Context
import android.graphics.Color
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.ItemArticleRowBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.common.OnMistakeClickListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.OnQuantityClickListener
import es.verdnatura.presentation.common.OnSaleClickListener
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.model.SaleVO

class SaleAdapter (
    private val items: List<SaleVO>,
    private val onPasillerosItemClickListener: OnPasillerosItemClickListener,
    private val onQuantityClick: OnQuantityClickListener,
    private val onSaleClickListener: OnSaleClickListener,
    private val onMistakeClickListener: OnMistakeClickListener
): RecyclerView.Adapter<SaleAdapter.AjustesItemHolder> () {
    var context:Context? = null
    var position:Int = 0
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AjustesItemHolder {
        this.context = parent.context
        return AjustesItemHolder(
            ItemArticleRowBinding.inflate(LayoutInflater.from(parent.context),parent,false)
        )
    }

    override fun getItemCount() =items.size

    override fun onBindViewHolder(holder: AjustesItemHolder, position: Int) {
        this.position = position
        holder.bind(items[position])
    }

   inner class AjustesItemHolder(
        val binding: ItemArticleRowBinding
    ) : RecyclerView.ViewHolder(binding.root){
        fun bind(sale: SaleVO) {
            binding.apply {
                if (sale.pickedQuantity.isNullOrEmpty())
                    sale.pickedQuantity = "0"

                val childLayoutManager = LinearLayoutManager(context!!, RecyclerView.HORIZONTAL, false)

                itemArticlePlacements.apply {
                    layoutManager = childLayoutManager
                    adapter = PlacementAdapter(sale.placements,onPasillerosItemClickListener)
                }

                //CLICK EVENTS
                contentLayout.setOnClickListener {
                    onSaleClickListener.onSaleClick(sale)
                }

                itemArticleItemFk.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(PasillerosItemVO(title = "Consultar artículo"),sale.itemFk)
                }


                itemArticleQuantity.setOnClickListener {
                    onQuantityClick.onQuantityClick(sale)
                }

                itemArticleQuantityPicked.setOnClickListener {
                    onQuantityClick.onQuantityClick(sale)
                }

                txtde.setOnClickListener {
                    onQuantityClick.onQuantityClick(sale)
                }

                contentLayout.setOnLongClickListener {
                    "Nueva funcionalidad en el icono de la derecha".toast(context!!)
                    //onMistakeClickListener.onMistakeClickListener(sale)
                    false
                }

                /*itemRowLayout.setOnLongClickListener{
                    onMistakeClickListener.onMistakeClickListener(sale)
                    false
                }*/
                imageErrorMessage.setOnClickListener{
                onMistakeClickListener.onMistakeClickListener(sale)
                false
            }


                //ERROR
                if (sale.originalQuantity != sale.quantity){
                    layoutError.visibility = View.VISIBLE
                    txtError.text = "Cantidad original: "+sale.originalQuantity
                    if (sale.isPrepared == "1" || sale.isControlled == "1")
                        sale.pickedQuantity = sale.quantity
                }

                if (sale.isNew){
                    layoutError.visibility = View.VISIBLE
                    txtError.text = "Artículo nuevo"

                }

                /*if (sale.level.equals("AMARILLO")){
                level.setTextColor(Color.YELLOW)
                }else{
                    level.setTextColor(Color.GREEN)
                }*/


                if (!sale.isNew && sale.originalQuantity == sale.quantity){
                    layoutError.visibility = View.GONE
                }


                //SEMAFORO
                if (sale.isPreviousPrepared == "1"){
                    itemArticleRowSemaforoPre.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_dark_sky_blue))
                }else{
                    itemArticleRowSemaforoPre.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_warm_grey))
                }

                if (sale.isPrepared == "1"){
                    itemArticleRowSemaforoSac.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_pumpkin_orange))
                }else{
                    itemArticleRowSemaforoSac.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_warm_grey))
                }

                if (sale.isControlled == "1"){
                    itemArticleRowSemaforoCon.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_dark_mint))
                }else{
                    itemArticleRowSemaforoCon.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_warm_grey))
                }

                if (sale.isControlled == "1"){
                    contentLayout.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_dark_mint))
                }else  if (sale.isPrepared == "1"){
                    contentLayout.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_pumpkin_orange))
                }else  if (sale.isPreviousPrepared == "1"){
                    contentLayout.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_dark_sky_blue))
                }else{
                    contentLayout.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_black))
                }

                //sergio: para de momento que pinten las lineas sin actualizar ni nada
                if (sale.isControlled == "2"){
                    itemArticleRowSemaforoCon.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_white))
                    contentLayout.setBackgroundColor(ContextCompat.getColor(context!!, R.color.verdnatura_dark_mint_light))
                }


                //ASIGNAMOS VALOR A LA VSITA
                this.sale = sale

            }
        }
    }
}