package es.verdnatura.presentation.view.feature.collection.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Bundle
import android.speech.SpeechRecognizer
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.FragmentCollectionBinding
import es.verdnatura.domain.ConstAndValues.CONTROLADOR
import es.verdnatura.domain.ConstAndValues.OK
import es.verdnatura.domain.ConstAndValues.ON_CHECKING
import es.verdnatura.domain.ConstAndValues.ON_PREPARATION
import es.verdnatura.domain.ConstAndValues.PRECHECKER
import es.verdnatura.domain.ConstAndValues.SACADOR
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.collection.ItemVO
import es.verdnatura.presentation.view.feature.collection.adapter.SaleAdapter
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.model.*
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_collection.*
import kotlinx.android.synthetic.main.toolbar.*
import timber.log.Timber

class CollectionFragment(
    var collection: CollectionVO = CollectionVO(),
    var type: String = SACADOR
) : BaseFragment<FragmentCollectionBinding, CollectionViewModel>(CollectionViewModel::class) {
    private lateinit var dataMessageSalix:DataMessageSalix
    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var SectorDescription = ""
    private var token = ""
    private var voz = ""
    private var sales: List<SaleVO> = listOf()
    private var saleAdapter: SaleAdapter? = null
    private var lm: LinearLayoutManager? = null
    private var storedPosition: Int = 0
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private lateinit var customDialogList: CustomDialogList
    private var placementSupplyAdapter: BarcodeAdapter? = null
    private var listPlacementSupply: ArrayList<BarcodeVO> = ArrayList()
    private var storedShelvingPosition: Int = 0
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons
    private var ticketSelected: String = ""
    private lateinit var customDialog: CustomDialog
    private var goBack: Boolean = false
    private var goBack2: Boolean = false
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null
    private var itemShelvingFkStored: String = ""
    private var storedBackPosition: Int = 0
    private var tickets: ArrayList<String> = ArrayList()
    private var mistakeSale: SaleVO? = null

    //private var myKM: KeyguardManager? = null

    private var state = 0

    private var mIsListening = false
    private var placementPicked: PlacementVO? = null

    private var lastPlacementPicked: PlacementVO? = null

    private var originalItemScan: String = ""
    private var workerFkFromTicket = ""

    companion object {
        fun newInstance(collection: CollectionVO, type: String) =
            CollectionFragment(collection, type)
    }

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_collection

    override fun onCreate(savedInstanceState: Bundle?) {

        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        token = prefs.getString(TOKEN, "").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        voz = prefs.getString(VOZ, "NO").toString()
        mperror = MediaPlayer.create((activity as MainActivity), R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity), R.raw.ok)
        SectorDescription=prefs.getString(SECTORDESCRIP,"Sin Sector").toString()




        if ((collection.collectionFk != null) && type.equals(CONTROLADOR)) {
            //sergio: comprueba si viene de la pantalla de controlador
            viewModel.ticketOrCollection_checkFullyControlled(user, password, collection.collectionFk,"ticketOrCollection_checkFullyControlled")
            workerFkFromTicket=collection.tickets.get(0).sales.get(0).workerFk
        }

        if (collection.tickets.isEmpty()) {
            viewModel.collectionTicketGet(user, password, collection.collectionFk, sectorFk, type,"collection_getTickets")
        }


        super.onCreate(savedInstanceState)
    }

    override fun onResume() {
        super.onResume()
        scanRequest()
    }

    override fun init() {
        customDialogList = CustomDialogList(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        if (splash_progress != null) splash_progress.visibility = View.VISIBLE




        setEvents()

        if (type != PRECHECKER) {
            setToolBar()
        } else {
            toolbar_subtitle.visibility = View.VISIBLE
            scan_input.visibility = View.GONE
        }


        if (type == SACADOR && voz != "NO") {
            setSpeak()
        }

        if (collection.tickets.isNotEmpty()) {
            createCollectionList()
        }

        super.init()
    }

    override fun onPause() {
        super.onPause()
        goBack = true
        goBack2 = true
    }

    private fun setToolBar() {
        toolbar_subtitle.visibility = View.VISIBLE

        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconPrint: Drawable =
            resources.getDrawable(R.drawable.ic_print_black_24dp, resources.newTheme())
        val iconSearch: Drawable =
            resources.getDrawable(R.drawable.ic_visibility, resources.newTheme())
        val iconAdd: Drawable =
            resources.getDrawable(R.drawable.ic_playlist_add_black_24dp, resources.newTheme())
        val iconFace: Drawable =
            resources.getDrawable(R.drawable.face_man_outline, resources.newTheme())

        listIcons.add(iconFace)
        listIcons.add(iconPrint)
        // listIcons.add(iconSearch)

        listIcons.add(iconAdd)


        toolbar_icons.adapter = ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconPrint) {
                    print()
                }/* else if (item == iconSearch) {
                    showDisponibility()
                } */else if (item == iconAdd) {
                    addItem()
                } else if (item == iconFace) {
                    showUser()
                }
            }
        })
        toolbar_icons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun scanRequest() {
        if (scan_input != null) {
            scan_input.requestFocus()
        }
        hideKeyboards()
    }

    private fun showUser() {


        (activity as MainActivity).onPasillerosItemClickListener(
            PasillerosItemVO(title = "Control de usuario de vehículo"),
            workerFkFromTicket
        )


    }

    private fun setEvents() {

        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }
        //ESCANER =========
        hideKeyboards()
       /* if (type.equals(CONTROLADOR)){
            scan_input.inputType= InputType.TYPE_CLASS_NUMBER
        }*/
        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {//ID=0 ACTION_NEXT  ID=5 ACTION_UNESPECEFIED)
                if (!scan_input.text.toString().isNullOrEmpty()) {
                    //////Log.i("VERDNATURA:","Pita o no pita si encuentra. Diferencia si es matricula o no")
                    findSale(scan_input.text.toString())


                }
                scan_input.setText("")
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            true
        }

        hideKeyboards()

        //LISTA =========
        collection_swipe.setOnRefreshListener {

            collection_swipe.isRefreshing = false
            if (type != PRECHECKER) {
                if (splash_progress != null) splash_progress.visibility = View.VISIBLE
                viewModel.collectionTicketGet(
                    user,
                    password,
                    collection.collectionFk,
                    sectorFk,
                    type
                    ,"collection_getTickets")
                collection_swipe.isRefreshing = false
            }
        }
    }

    private fun hideKeyboards() {
        try {
            requireActivity().hideKeyboard()
        } catch (e: Exception) {
        }
    }

    override fun observeViewModel() {
        with(viewModel) {

            collectionTicketList.observe(viewLifecycleOwner, Observer {
                if (splash_progress != null) splash_progress.visibility = View.GONE
                if (!it.isError) {
                    if (it.tickets.isNotEmpty()) {
                        collection = it
                        createCollectionList()
                        workerFkFromTicket = it.tickets.get(0).sales.get(0).workerFk
                     //   Log.i("VERDNATURA:", "El primer ticket es : ${it.tickets.get(0).ticketFk}")
                      /*  Log.i(
                            "VERDNATURA:",
                            "El primer workerfk es :${it.tickets.get(0).sales.get(0).workerFk}"
                        )*/
                    } else {
                        toolbar_subtitle.text = "0/0"
                    }
                } else {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                            if (activity != null) requireActivity().onBackPressed()
                        }.show()
                }

            })

            placementSuppleyList.observe(viewLifecycleOwner, Observer {
                if (splash_progress != null) splash_progress.visibility = View.GONE
                if (!goBack) printShelvingResult(it)
                goBack = false
            })

            item.observe(viewLifecycleOwner, Observer {
                if (splash_progress != null) splash_progress.visibility = View.GONE
                if (!goBack) toastDisponibility(it)
                goBack = false
            })

            responseNew.observe(viewLifecycleOwner, Observer {
                if (splash_progress != null) splash_progress.visibility = View.GONE
               // if (!goBack) {
                    if (it.isError) {
                        customDialog.setTitle("Disponibilidad").setDescription(it.errorMessage)
                            .setKoButton("Cerrar") {
                                scanRequest()
                                customDialog.dismiss()
                            }.show()
                    } else {

                        if (splash_progress != null) splash_progress.visibility = View.VISIBLE

                        sendSalixMessage(
                            item=dataMessageSalix.item,
                            ticketFk=dataMessageSalix.ticket,
                            quantity=dataMessageSalix.quantity,
                            salePerson=dataMessageSalix.personSale
                                        )
                        viewModel.collectionTicketGet(
                            user,
                            password,
                            collection.collectionFk,
                            sectorFk,
                            type
                            ,"collection_getTickets")

                        //  //Log.i("VERDNATURA:","La collection es ${collection.collectionFk}")
                    }
               // }

                goBack = false

            })

            responseCode.observe(viewLifecycleOwner, Observer {
                if (splash_progress != null) splash_progress.visibility = View.GONE
                if (!goBack2) {
                    if (it.isError) {
                        customDialog.setTitle("Error").setDescription(it.errorMessage)
                            .setKoButton("Cerrar") {
                                scanRequest()
                                customDialog.dismiss()
                            }.show()
                    } else {
                        if (checkItemScan(it.response)) {
                            scanRequest()
                            customDialogList.dismiss()
                            mpok?.start()
                            onQuantityOfShelvingSelected(itemShelvingFkStored)
                        } else {
                            customDialogList.setValueTwo("")
                            //showErrorMessage("El resultado del procedimiento barcodeToItem de la etiqueta escaneada es: " +it.response)
                            showErrorMessage("La línea de compra de la etiqueta escaneada no existe.")
                            if (mperror != null) mperror?.start()
                        }
                    }
                }

                goBack2 = false

            })

            responseIncQuantity.observe(viewLifecycleOwner, Observer {
                /* splash_progress.visibility = View.VISIBLE
                 viewModel.collectionTicketGet(user,password,collection.collectionFk,sectorFk)*/
            })

            mistakeList.observe(viewLifecycleOwner, Observer {
                if (splash_progress != null) splash_progress.visibility = View.GONE
                if (!goBack) showMistakeList(it.list)
                goBack = false
            })
            //sergio: para si hay algun ticket por revisar
            responsecheckfully.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError) {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                    //sergio: devuelve false si no hay para revisar y <> false.
                    if (it.response != "false") {
                        customDialog.setTitle("Aviso").setDescription("Pendiente de revisar : "+it.response)
                            .setOkButton("Aceptar") {
                                customDialog.dismiss()

                            }
                            .setKoButton("Revisar"){
                                splash_progress.visibility=View.VISIBLE
                                viewModel.collectionTicketGet(user, password, it.response, sectorFk, type,"collection_getTickets")
                                customDialog.dismiss()
                            }
                        customDialog.show()
                    }
                    // if (!goBack)navigateToCollectionList(it)
                    //goBack = false
                }
            })

            responseSalixMessage.observe(viewLifecycleOwner, Observer{
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()
                }
            })


        }
    }

    private fun getSalePerson(ticketFk:String):String
    {

        var salePerson =""
        for (it in sales)
        {
            if (it.ticketFk.equals(ticketFk))
            {
                salePerson= it.salePersonFk
                break
            }
        }
        return salePerson
    }

    private fun sendSalixMessage(item:String,ticketFk:String,quantity:String,salePerson:String)
    {
        val ticket =
            "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
        val message =
            "Se ha añadido el articulo " + item + " con " +quantity+ " unidad/es al ticket " + ticket


    /*    d("VERDNATURA:","el comercial es " + salePerson)
        d("VERDNATURA:","el item  es " + item)
        d("VERDNATURA:","el ticket es es " + ticketFk)
        d("VERDNATURA:","la cantidad es " + quantity)*/

        viewModel.sendChekingPresence(
            token = token,
            workerId = salePerson,
            message = message,"sendChekingPresence"
                                     )
    }

    //CREATE LIST
    private fun createCollectionList() {
        state = 0
        if (type == SACADOR && voz != "NO") {
            // initialize()
            speak("Colección cargada, diga listo para empezar.")
        }

        toolbar_title.text = collection.collectionFk


        if (splash_progress != null) splash_progress.visibility = View.GONE
        var salesList: ArrayList<SaleVO> = ArrayList()
        tickets = ArrayList()
        var observations = ""
        collection.tickets.forEach { ticket ->
            ticket.sales.forEach { saleVO ->
                if (type == SACADOR && saleVO.quantity != "0") {
                    salesList.add(saleVO)
                    if (tickets.firstOrNull { it == saleVO.ticketFk }.isNullOrEmpty())
                        tickets.add(saleVO.ticketFk)
                } else if (type == CONTROLADOR || type == PRECHECKER) {
                    //sergio el type CHECKED =CONTROLADOR
                    salesList.add(saleVO)
                    if (tickets.firstOrNull { it == saleVO.ticketFk }.isNullOrEmpty())
                        tickets.add(saleVO.ticketFk)
                }
            }

            observations = observations + " " + ticket.observations
        }

       //sales = salesList.sortedWith(compareBy({it.placements.get(0).placement}))
        sales = salesList.sortedWith(compareBy({it.saleOrder}))
        //sales=salesList
      /*  for (s in sales)
        {
            Log.i("VERDNATURA:","La sale del ticket ${s.ticketFk} es ${s.saleOrder}")
            for (p in s.placements){
                Log.i("VERDNATURA:","La sale del ticket ${s.saleOrder} es ${p.placement}")
                //s.placements=s.placements.sortedBy { it.saleOrder}.sortedBy { !it.visible.equals("0") }
                s.placements=s.placements.sortedBy { !it.visible.equals("0") }
                // Log.i("VERDNATURA:","La sale del ticket ${s.saleOrder} es saleorder  ${p.saleOrder}")
            }
        }*/


        /*for (s in salesList)
        {
            Log.i("VERDNATURA:","La sale del ticket ${s.ticketFk} es ${s.saleOrder}")
            for (p in s.placements){
                Log.i("VERDNATURA:","La sale del ticket ${s.saleOrder} es ${p.placement}")
                // Log.i("VERDNATURA:","La sale del ticket ${s.saleOrder} es saleorder  ${p.saleOrder}")
            }
        }*/
        //pruebas sergio
        // sales = salesList.sortedWith(compareBy({it.placements.get(0).created}))
       // SalesSorter()



        saleAdapter =
            SaleAdapter(sales, pasillerosItemClickListener!!, object : OnQuantityClickListener {

                override fun onQuantityClick(sale: SaleVO) {

                    sales.forEachIndexed { index, saleVO ->
                        if (saleVO.saleFk == sale.saleFk) {
                            if (type != "PRECHECKER") {
                                showQuantityDialog(index)
                            }

                        }
                    }

                }
            }, object : OnSaleClickListener {
                override fun onSaleClick(sale: SaleVO) {

                    sales.forEachIndexed { index, saleVO ->
                        if (saleVO.saleFk == sale.saleFk) {

                            if (type == CONTROLADOR || type == PRECHECKER) {// sergio:cuidado , esta es de lo último para marcar colores
                                markLine(index, type)
                                //////Log.i("VERDNATURA:","marcamos linea")
                            } else if (type == SACADOR) {
                                if (sale.isPrepared == "0") {
                                    showScanner(index, sale)
                                } else {
                                    unMarkLine(index, OK)
                                }

                            }
                        }

                    }
                }

            }, object : OnMistakeClickListener {
                override fun onMistakeClickListener(sale: SaleVO) {
                    if (type != "PRECHECKER") {
                        if (splash_progress != null) splash_progress.visibility = View.VISIBLE
                        mistakeSale = sale
                        viewModel.mistakeType(usuario = user, password = password)
                    }
                }
            })

        lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        fragment_sacador_collections.adapter = saleAdapter
        fragment_sacador_collections.layoutManager = lm
        setTotalLines()

        if (storedPosition != 0)
            setListPosition(storedPosition, true)
        else if (storedBackPosition != 0)
            setListPosition(storedBackPosition, true)

        setScrollListener(lm!!)
        //sergio: se quita porque se realiza desde triggers
        // changeInitTicketState()

        printObservations(observations)
    }

    private fun setScrollListener(lm: LinearLayoutManager) {
        fragment_sacador_collections.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                storedBackPosition = lm.findFirstVisibleItemPosition()
                super.onScrolled(recyclerView, dx, dy)
            }
        })
    }

    private fun printObservations(observations: String) {
        if (observations.trim().isNotEmpty()) {
            customDialog.setTitle("Observaciones").setDescription(observations)
                .setOkButton("Aceptar") {
                    customDialog.hide()
                    customDialog.dismiss()
                }.show()
        }
    }

    //SEARCH AND MARK
    private fun findSale(txtscan: String) {
        goBack = false
        var index = 0
        var isBreak = false
        var isOk = false
        goBack = false
        goBack2 = false

        if (type == SACADOR) {
            for (saleVO in sales) {
                //sergio: pita si encuentra ticket
                //////Log.i("VERDNATURA:","SE busca en sales ${saleVO.ticketFk}")

                if (saleVO.ticketFk == txtscan) {
                    isOk = true
                    if ((isOk) && mpok != null) mpok!!.start()
                    //Log.i("VERDNTURA","ENCONTRADO")
                    break

                }

                if (saleVO.isPrepared != "1" && saleVO.isPreviousPrepared != "1") {
                    //1- Por itemFk
                    /* if (txtscan == saleVO.itemFk){
                         mpok!!.start()
                         isOk = true
                         markLine(index,type)
                         break
                     }*/
                    //2- Por carro
                    if (saleVO.placements != null) {
                        var shelvingIndex = 0
                        for (placementVO in saleVO.placements) {
                            // para comprobar si es igual a una matricula
                            if (placementVO.shelving != null && placementVO.visible != null && txtscan.uppercase() == placementVO.shelving.uppercase() && placementVO.visible != "(0)") {
                                if (mpok != null) mpok!!.start()
                                isOk = true
                                //////Log.i("VERDNATURA:","encontrado producto")
                                showShelving(index, shelvingIndex)
                                isBreak = true
                                break
                            }
                            shelvingIndex += 1
                        }
                        if (isBreak) break
                    }

                    //3- Por barcode
                    /*saleVO.Barcodes.forEach { barcode ->
                        if (txtscan == barcode){
                            mpok!!.start()
                            isOk = true
                            markLine(index,type)
                            isBreak = true
                        }
                    }
                    if (isBreak) break*/
                }
                index += 1
            }
            if (!isOk) {
                if (txtscan.contains("-")) {
                    if (tickets.size > 0) {
                        viewModel.parking(
                            usuario = user,
                            password = password,
                            ticketFk = tickets[0],
                            parking = txtscan,
                            "shelvingPark"
                        )
                    }
                    Timber.i("La coleccion es "+tickets[0])
                    if (mpok != null) mpok!!.start()
                    "Ticket aparcado".toast(requireContext())
                } else {
                    if (mperror != null) mperror!!.start()
                }

            }
        } else if (type == CONTROLADOR || type == PRECHECKER) {
            //////Log.i("VERDNATURA:","El tipo es controlador")
            for (saleVO in sales) {
                if (saleVO.isControlled == "0") {
                    //1- Por itemFk
                    if (txtscan == saleVO.itemFk) {
                        if (mpok != null) mpok!!.start()
                        isOk = true
                        markLine(index, type)
                        break
                    }
                    //2- Por barcode
                    saleVO.Barcodes.forEach { barcode ->
                        if (txtscan == barcode) {
                            if (mpok != null) mpok!!.start()
                            isOk = true
                            markLine(index, type)
                            isBreak = true
                        }
                    }
                    if (isBreak) break
                }
                index += 1
            }
            //Lo escaneado no encontrado miramos si es parking o ticket
            if (!isOk) {
                if (txtscan.contains("-")) {
                    if (tickets.size > 0) {
                        viewModel.parking(
                            usuario = user,
                            password = password,
                            ticketFk = tickets[0],
                            parking = txtscan,
                            "shelvingPark"
                        )
                    }
                    if (mpok != null) mpok!!.start()
                    getString(R.string.Ticketaparcado).toast(requireContext())
                } else {
                    if (splash_progress != null) splash_progress.visibility = View.VISIBLE
                    storedPosition = 0
                    storedBackPosition = 0
                    if (type != PRECHECKER) {

                        viewModel.collectionTicketGet(user, password, txtscan, sectorFk, type,"collection_getTickets")

                        if (type.equals(CONTROLADOR)) {
                            viewModel.ticketOrCollection_checkFullyControlled(user, password, txtscan,"ticketOrCollection_checkFullyControlled")
                        }
                    } else {
                        splash_progress.visibility = View.GONE
                    }
                    ////Log.i("VERDNATURA:","La collection es ${txtscan}")
                }
            }
        }

    }

    private fun findSale(txtscan: String, position: Int) {
        var isOk = false
        if (type == SACADOR) {
            var saleVO = sales[position]

            if (saleVO.isPrepared != "1" && saleVO.isPreviousPrepared != "1") {
                //1- Por itemFk
                /*if (txtscan == saleVO.itemFk){
                    if (mpok != null) mpok!!.start()
                    isOk = true
                    markLine(position,type)

                }*/
                if (!isOk) {
                    //2- Por carro
                    var shelvingIndex = 0
                    for (placementVO in saleVO.placements) {
                        if (txtscan.uppercase() == placementVO.shelving.uppercase()) {
                            if (mpok != null) mpok!!.start()
                            isOk = true
                            showShelving(position, shelvingIndex)
                            break
                        }
                        shelvingIndex += 1
                    }
                }
                /*
                if (!isOk){
                    //3- Por barcode
                    saleVO.Barcodes.forEach { barcode ->
                        if (txtscan == barcode){
                            if (mpok != null) mpok!!.start()
                            isOk = true
                            markLine(position,type)
                        }
                    }
                }*/
            }
            if (!isOk) {
                if (txtscan.contains("-")) {
                    tickets.forEach {
                        viewModel.parking(
                            usuario = user,
                            password = password,
                            ticketFk = it,
                            parking = txtscan,
                            "shelvingPark"
                        )
                    }
                    if (mpok != null) mpok!!.start()
                    "Ticket aparcado".toast(requireContext())

                } else {
                    if (mperror != null) mperror!!.start()
                }

            }
        }

    }



    private fun markLine(position: Int, newType: String) {
        state = 0
        //////Log.i("VERDNATURA:","Marcamos linea y enviamos saleTracking")
        if (type == SACADOR) {
            sales[position].isPrepared = if (sales[position].isPrepared == "1") "0" else "1"
            if (sales[position].isPrepared == "1") {
                sales[position].pickedQuantity = sales[position].quantity
            }
        } else if (type == CONTROLADOR)
            sales[position].isControlled = if (sales[position].isControlled == "1") "0" else "1"

        if (type == PRECHECKER) {
            sales[position].isControlled =
                if (sales[position].isControlled == "2") "0" else "2" //sergio:  para diferenciar precontrol en previa
        }

        saleAdapter!!.notifyDataSetChanged()
        setListPosition(position, false)

        if (type != PRECHECKER) {
            saleTrackingReplace(position, newType)
        }

        // sergio: a borrar. Solo para hacer pruebas
        if (type == PRECHECKER) {
            saleTrackingReplace(position, newType)
        }


        setTotalLines()
    }

    private fun setListPosition(position: Int, isFromBack: Boolean) {
        storedPosition = position
        if (type == SACADOR) {
            if (fragment_sacador_collections != null) {
                fragment_sacador_collections.addViewObserver {
                    lm!!.scrollToPositionWithOffset(position, 0)
                }
            }

        } else if (isFromBack) {
            if (fragment_sacador_collections != null) {
                fragment_sacador_collections.addViewObserver {
                    lm!!.scrollToPositionWithOffset(position, 0)
                }
            }
        }

    }

    private fun saleTrackingReplace(position: Int, type: String) {
        //////Log.i("VERDNATURA:","POSICION $position y el tipo $type")
        //////Log.i("VERDNATURA:","El buyfk que no es real es $originalItemScan")
        viewModel.saleTrackingReplace(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            vIsChecked = if (type == SACADOR) sales[position].isPrepared else if (type == CONTROLADOR) sales[position].isControlled else "1",
            vOriginalQuantity = sales[position].pickedQuantity,
            vStateFk = if (type==CONTROLADOR) "CHECKED" else type,//type.toString(),
            vBuyFk = originalItemScan,
            callFunction = "saleTrackingReplace"
        )

    }

    private fun unMarkLine(position: Int, newType: String) {
        state = 0
        if (sales[position].isPrepared == "1") {
            customDialog.setTitle("Desmarcar linea")
                .setDescription("Vas a desmarcar la linea: " + sales[position].itemFk + " ¿Estás seguro?")
                .setOkButton("Desmarcar") {
                    sales[position].isPrepared = "0"
                    sales[position].pickedQuantity = "0"
                    saleAdapter!!.notifyDataSetChanged()
                    setListPosition(position, false)
                    if (type != PRECHECKER) {
                        viewModel.saleTrackingDel(
                            usuario = user,
                            password = password,
                            saleFk = sales[position].saleFk,
                            "saleTracking_del"
                        )
                    }
                    setTotalLines()
                    scanRequest()
                    customDialog.dismiss()
                }.setKoButton("Cancelar") {
                scanRequest()
                customDialog.dismiss()
            }.show()
        }

    }

    //SHELVINGS
    private fun showShelving(position: Int, shelvingPosition: Int) {
        storedShelvingPosition = shelvingPosition
        storedPosition = position
        if (splash_progress != null) splash_progress.visibility = View.VISIBLE
        var quantityGet = "0"
        try {
            quantityGet =
                (sales[position].quantity.toInt() - sales[position].pickedQuantity.toInt()).toString()
        } catch (e: Exception) {
        }
        //////Log.i("VERDNATURA:","Aparece  para enviar")
        viewModel.itemPlacementSupplyAiming(
            usuario = user,
            password = password,
            itemFk = sales[position].itemFk,
            quantity = quantityGet,
            shelvingFk = sales[position].placements[shelvingPosition].shelving,
            callFunction = "itemPlacementSupplyAiming"
        )
    }

    private fun printShelvingResult(placementSupplyListVO: PlacementSupplyListVO) {
        //////Log.i("VERDNATURA:","Print CustomDialog")
        var shelving = ""
        var item = ""
        var longName = ""
        var total = "0"
        var itemShelvingFk = "0"
        if (!placementSupplyListVO.list.isEmpty()) {
            var placement: PlacementSupplyVO? = null
            placementSupplyListVO.list.forEach { placementVO ->
                if (placementVO.stock != "0" && placement == null)
                    placement = placementVO
            }
            if (placement != null) {
                shelving = placement!!.shelving
                item = placement!!.itemFk
                longName = placement!!.longName
                total = placement!!.total
                itemShelvingFk = placement!!.itemShelvingFk
            }

        }
        listPlacementSupply = ArrayList()
        placementSupplyListVO.list.forEach {
            if (it.stock != "0")
                listPlacementSupply.add(BarcodeVO(code = it.proposal))
            //////Log.i("VERDNATURA:","Array de placementsupply")
        }
        try {
           // Log.i("VERDNATURA:","Pulso COGER-1")
            customDialogList.setTitle("$shelving($item) $total de $longName").setOkButton("Coger") {
                if (customDialogList.getValueTwo().isNotEmpty()) {
                    if (isNumber(customDialogList.getValue()) && isNumber(total) && customDialogList.getValue()
                            .toInt() > total.toInt()
                    ) {
                        "La cantidad supera a la disponible".toast(requireContext())
                    } else if (isNumber(customDialogList.getValue())) {
                        originalItemScan = customDialogList.getValueTwo()
                        // Log.i("VERDATURA","Le pasamos el siguiente item $originalItemScan que es el escaneado")
                        if (checkItemScan(customDialogList.getValueTwo())) {
                            onQuantityOfShelvingSelected(itemShelvingFk)
                            //////Log.i("VERDNATURA:","Cantidad seleccionada")
                            mpok?.start()
                            ReviewQuantityForRefreshingAndSorting(customDialogList.getValue().toInt(),sales[storedPosition].quantity.toInt(),
                                sales[storedPosition].pickedQuantity.toInt())
                            customDialogList.dismiss()
                        } else {
                            itemShelvingFkStored = itemShelvingFk
                            if (splash_progress != null) splash_progress.visibility = View.VISIBLE
                            viewModel.getIdFromCode(
                                usuario = user,
                                password = password,
                                code = customDialogList.getValueTwo(),"barcodeToItem"
                            )
                            customDialogList.dismiss()
                        }
                        scanRequest()
                        hideKeyboards()
                    } else {
                        "cantidad introducida erronea".toast(requireContext())
                    }


                } else {
                    "Escanea item para validar".toast(requireContext())
                }

            }.setKoButton("Cerrar") {
                scanRequest()
                hideKeyboards()
                customDialogList.dismiss()
            }.setHintValue("Cantidad que coges:").setValue(total).setHintValueTwo("Escanea item")
                .setValueTwo("").show()
        } catch (e: Exception) {
            //////Log.i("VERDNATURA:","${e.message.toString()}")
        }
        try {
            customDialogList.getEditTextTwo().post(Runnable {
                customDialogList.getEditTextTwo().requestFocusFromTouch()
                if (activity != null) {
                    val lManager: InputMethodManager =
                        requireActivity().getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                    lManager.hideSoftInputFromWindow(
                        customDialogList.getEditTextTwo().windowToken,
                        InputMethodManager.SHOW_FORCED
                    )
                }

            })
        } catch (e: Exception) {
        }
        try {
            customDialogList.getEditTextTwo().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {

                   // Log.i("VERDNATURA:","Comprobando COGER-1")
                    if (customDialogList.getValueTwo().isNotEmpty()) {
                        try {
                            if (isNumber(customDialogList.getValue()) && customDialogList.getValue()
                                    .toInt() > total.toInt())
                            {
                                "La cantidad supera a la disponible".toast(requireContext())
                            }
                            else if (isNumber(customDialogList.getValue())) {
                                originalItemScan = customDialogList.getValueTwo()

                                if (checkItemScan(customDialogList.getValueTwo())) {
                                    onQuantityOfShelvingSelected(itemShelvingFk)
                                    mpok?.start()
                                    ReviewQuantityForRefreshingAndSorting(customDialogList.getValue().toInt(),sales[storedPosition].quantity.toInt(),
                                        sales[storedPosition].pickedQuantity.toInt())
                                  //  SalesSorter(sales)

                                    customDialogList.dismiss()
                                } else {
                                    itemShelvingFkStored = itemShelvingFk
                                    splash_progress.visibility = View.VISIBLE
                                    viewModel.getIdFromCode(
                                        usuario = user,
                                        password = password,
                                        code = customDialogList.getValueTwo(),"barcodeToItem"
                                    )
                                    customDialogList.dismiss()
                                    scanRequest()
                                }
                            } else {
                                "cantidad introducida erronea".toast(requireContext())
                            }

                        } catch (e: Exception) {

                        }

                    } else {
                        "Escanea item para validar".toast(requireContext())
                    }
                    scanRequest()
                    hideKeyboards()
                    return@setOnEditorActionListener true

                }
                false
            }
        } catch (e: Exception) {
        }

        placementSupplyAdapter =
            BarcodeAdapter(listPlacementSupply, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    placementSupplyListVO.list.forEach {
                        if (it.proposal == item.code) {
                            customDialogList.setValue(it.total)
                            total = it.total
                            itemShelvingFk = it.itemShelvingFk
                        }
                    }

                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    private fun ReviewQuantityForRefreshingAndSorting(
        quantityPicking: Int,
        quantityTotal: Int,
        quantityPicked: Int
    ) {

       /* Log.i("VERDNATURA::","Picking"+quantityPicking)
        Log.i("VERDNATURA::","Total"+quantityTotal)
        Log.i("VERDNATURA::","Now picked"+quantityPicked)*/

        if (quantityPicked<quantityTotal)
        {

          //  SalesSorter()
           // Log.i("VERDNATURA::","Lanzamos collection para que refresque adaptador y ordene")
            viewModel.collectionTicketGet(user, password, collection.collectionFk, sectorFk, type,"collection_getTickets")

        }

    }

  /*  private fun SalesSorter()
    {

        for (s in sales)
        {
            Log.i("VERDNATURA:","La sale del ticket ${s.ticketFk} es ${s.saleOrder}")
            for (p in s.placements){
                Log.i("VERDNATURA:","La sale del ticket ${s.saleOrder} es ${p.placement}")
                //s.placements=s.placements.sortedBy { it.saleOrder}.sortedByDescending { it.visible }

                s.placements=s.placements.sortedWith(compareBy({it.saleOrder})).sortedByDescending { it.visible }

                //sales = salesList.sortedWith(compareBy({it.saleOrder}))

               /* s.placements=s.placements.sortedWith ( compareBy(
                    {it.visible.equals("0")}
                                                                ))*/
            }
        }

        d("VERDNATURA","Ordenamos")

        fragment_sacador_collections.adapter?.notifyDataSetChanged()

    }*/

    private fun checkItemScan(valueToCheck: String): Boolean {

        if (storedPosition >= 0 && sales.size > storedPosition) {
            val saleToCheck = sales[storedPosition]

            if (saleToCheck.itemFk == valueToCheck)
                return true
            else {
                saleToCheck.Barcodes.forEach { barcode ->
                    if (barcode == valueToCheck)
                        return true
                }
            }
        }
        return false
    }

    private fun onQuantityOfShelvingSelected(itemShelvingFk: String, quantity: String = "0") {
        //1 - MODIFICAR CANTIDAD DEL CARRO
        try {
            val shelvingVisible =
                sales[storedPosition].placements[storedShelvingPosition].visible.substring(
                    1,
                    sales[storedPosition].placements[storedShelvingPosition].visible.indexOf(")")
                )

            if (quantity == "0") {
                sales[storedPosition].placements[storedShelvingPosition].visible =
                    "(" + (shelvingVisible.toInt() - customDialogList.getValue()
                        .toInt()).toString() + ")"
            } else {
                sales[storedPosition].placements[storedShelvingPosition].visible =
                    "(" + (shelvingVisible.toInt() - quantity.toInt()).toString() + ")"
            }

            viewModel.itemShelvingSaleSupplyAdd(
                usuario = user,
                password = password,
                itemShelvingFk = itemShelvingFk,
                saleFk = sales[storedPosition].saleFk,
                quantity = if (quantity != "0") quantity else customDialogList.getValue(),
                callFunction = "itemShelvingSaleSupplyAdd"
            )

        } catch (e: Exception) {
        }

        //2- MODIFICAR EL PICKED DEL SALE
        try {
            if (quantity == "0") {
                sales[storedPosition].pickedQuantity =
                    (sales[storedPosition].pickedQuantity.toInt() + customDialogList.getValue()
                        .toInt()).toString()
            } else {
                sales[storedPosition].pickedQuantity =
                    (sales[storedPosition].pickedQuantity.toInt() + quantity
                        .toInt()).toString()
            }
        } catch (e: Exception) {
        }

        //3- MARCAR LINEA
        if (sales[storedPosition].pickedQuantity != sales[storedPosition].quantity) {
            sales[storedPosition].isPrepared = "1"
            markLine(storedPosition, OK)
        } else {
            markLine(storedPosition, type)
        }

    }

    //OPTIONS
    private fun print() {
        viewModel.collectionStickerPrint(
            usuario = user,
            password = password,
            collectionFk = collection.collectionFk,
            sectorFk = sectorFk,
            "collectionStickerPrint"
        )


        (getString(R.string.Imprimiendo) + " por "+SectorDescription).toast(requireContext())
    }

    private fun addItem() {
        listPlacementSupply = ArrayList()
        collection.tickets.forEach {
            listPlacementSupply.add(BarcodeVO(code = it.ticketFk))
        }
        if (listPlacementSupply.size == 1) {
            ticketSelected = listPlacementSupply[0].code!!
        }

        customDialogList.setTitle(getString(R.string.Agregarartículoparaticket) + " " + ticketSelected)
            .setOkButton(getString(R.string.Agregar)) {
                if (ticketSelected.isNullOrEmpty()) {
                    getString(R.string.Seleccionaunticketdelaista).toast(requireContext())
                } else if (customDialogList.getValue()
                        .isNullOrEmpty() || customDialogList.getValueTwo().isNullOrEmpty()
                ) {
                    getString(R.string.Todosloscampossonobligatorios).toast(requireContext())
                } else {
                    if (splash_progress != null) splash_progress.visibility = View.VISIBLE

                    //sergio datos para mensaje salix

                    dataMessageSalix= DataMessageSalix(
                        ticket=ticketSelected,
                        item = customDialogList.getValue(),
                        quantity=customDialogList.getValueTwo(),
                        personSale=getSalePerson(ticketSelected)
                        )

                    viewModel.collectionAddItem(
                        usuario = user,
                        password = password,
                        itemFk = customDialogList.getValue(),
                        ticketFk = ticketSelected,
                        quantityFk = customDialogList.getValueTwo(),
                        warehouseFk = warehouseFk,
                        callFunction="collection_addItem"
                    )


                    scanRequest()
                    customDialogList.dismiss()
                }

            }.setKoButton("Cerrar") {
            scanRequest()
            customDialogList.dismiss()
        }.setHintValue(getString(R.string.Artículo)).setValue("")
            .setHintValueTwo(getString(R.string.Cantidad)).setValueTwo("").show()

        customDialogList.getEditText().requestFocus()
        hideKeyboards()

        customDialogList.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!customDialogList.getValue().isNullOrEmpty()) {
                    customDialogList.getEditTextTwo().requestFocus()
                }
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

        customDialogList.getEditTextTwo().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!customDialogList.getValueTwo().isNullOrEmpty()) {
                    if (ticketSelected.isNullOrEmpty()) {
                        getString(R.string.Seleccionaunticketdelaista).toast(requireContext())
                    } else if (customDialogList.getValue()
                            .isNullOrEmpty() || customDialogList.getValueTwo().isNullOrEmpty()
                    ) {
                        getString(R.string.Todosloscampossonobligatorios).toast(requireContext())
                    } else {
                        if (splash_progress != null) splash_progress.visibility = View.VISIBLE

                        dataMessageSalix= DataMessageSalix(
                            ticket=ticketSelected,
                            item = customDialogList.getValue(),
                            quantity=customDialogList.getValueTwo(),
                            personSale=getSalePerson(ticketSelected)
                                                          )

                        viewModel.collectionAddItem(
                            usuario = user,
                            password = password,
                            itemFk = customDialogList.getValue(),
                            ticketFk = ticketSelected,
                            quantityFk = customDialogList.getValueTwo(),
                            warehouseFk = warehouseFk,
                            callFunction = "collection_addItem"
                        )
                        scanRequest()
                        customDialogList.dismiss()
                    }
                }
                hideKeyboards()
                return@setOnEditorActionListener true
                scanRequest()
                customDialogList.dismiss()
            }
            false
        }

        placementSupplyAdapter =
            BarcodeAdapter(listPlacementSupply, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    customDialogInput.setTitle(getString(R.string.ArtículoparaTicket) + " " + item.code!!)
                    ticketSelected = item.code!!
                    customDialogList.setTitle(getString(R.string.Agregarartículoparaticket) + " " + ticketSelected)
                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

   /* private fun showDisponibility() {

        customDialogInput.setTitle(getString(R.string.Verdisponible))
            .setDescription(getString(R.string.Escaneaetiqueta))
            .setOkButton(getString(R.string.Buscar)) {
                try {
                    if (splash_progress != null) {
                        splash_progress.visibility = View.VISIBLE
                    }

                } catch (e: Exception) {
                }

                hideKeyboards()
                viewModel.itemGetAvailable(
                    usuario = user,
                    password = password,
                    itemFk = customDialogInput.getValue(),
                    warehouseFk = warehouseFk,
                    "item_GetVisibleAvailable"
                )
                scanRequest()
                customDialogInput.dismiss()

            }.setKoButton("Cancelar") {
            if (splash_progress != null) {
                splash_progress.visibility = View.GONE
            }
            hideKeyboards()
            scanRequest()
            customDialogInput.dismiss()
        }.setValue("").show()
        customDialogInput.getEditText().requestFocus()
        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    if (splash_progress != null) splash_progress.visibility = View.VISIBLE
                    viewModel.itemGetAvailable(
                        usuario = user,
                        password = password,
                        itemFk = customDialogInput.getValue(),
                        warehouseFk = warehouseFk
                    ,"item_GetVisibleAvailable")
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }
    }*/

    private fun toastDisponibility(item: ItemVO) {
        if (item.available.isNullOrEmpty()) {
            item.available = "0"
        }

        customDialog.setTitle("Artículo: " + item.id)
            .setDescription("Disponible: " + item.available).setOkButton("Aceptar") {
            scanRequest()
            customDialog.dismiss()
        }.show()

    }

    private fun setTotalLines() {
        var totalMark = 0
        sales.forEach {
            if (type == SACADOR) {
                if (it.isPrepared == "1" || it.isControlled == "1" || it.isPreviousPrepared == "1") {
                    totalMark += 1
                } else if (it.quantity == "0") {
                    totalMark += 1
                }
            } else if (type == CONTROLADOR || type == PRECHECKER) {
                if (it.isControlled == "1" || it.isControlled == "2")
                    totalMark += 1
            }
        }
        //toolbar_title.text = "" + totalMark + "/" + sales.size
        if (toolbar_title != null) toolbar_title.text =
            if (collection.collectionFk != null) collection.collectionFk else ""
        if (toolbar_subtitle != null && totalMark != null && sales != null) toolbar_subtitle.text =
            "" + totalMark + "/" + sales.size
        if (totalMark == sales.size) {
            getString(R.string.Coleccióncompleta).toast(this.context, Toast.LENGTH_SHORT)
            showBottomNavigation(View.VISIBLE)
            //sergio: se quita porque se realiza desde triggers
            //changeTicketState()
        }
    }

    private fun showBottomNavigation(visibilitiy:Int)
    {
        //requireActivity().main_bottom_navigation.visibility = visibilitiy
    }

    //FALTAS / BASURA / SPLIT
    private fun showQuantityDialog(position: Int) {
        // //Log.i("VERDNATURA:","Aparece CustomDialog")
        customDialogThreeButtons.setDescription(getString(R.string.txtnuevacantidad)).setValue("")
            .setOkButton(getString(R.string.Faltas)) {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                } else {
                    trash(position, customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }
            }.setOkButtonTwo(getString(R.string.BasuraRechazar)) {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                } else {
                    missing(position, customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }
            }.setOkButtonThree(getString(R.string.Reject)) {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                } else {
                    reject(position, customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }

            }.setOkButtonFour("Split") {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                } else {
                    split(position, customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }

            }.setOkButtonAdd(getString(R.string.Agregar)) {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                } else {
                    increaseQuantity(position, customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }
            }.setKoButton("Cancelar") {
                scanRequest()
                customDialogThreeButtons.dismiss()
            }.show()
    }

    private fun split(position: Int, quantity: String) {
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.saleMove(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            originalQuantity = sales[position].quantity,
            "saleMove"
        )
        //sales[position].originalQuantity = quantity
        sales[position].quantity = quantity
        //sales[position].startQuantity = quantity

        if (quantity == "0")
            markLine(position, type)
        saleAdapter!!.notifyDataSetChanged()

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Split el articulo " + sales[position].itemFk + " del ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message,"sendChekingPresence"
        )

    }

    private fun trash(position: Int, quantity: String) {
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "FALSE",
            originalQuantity = quantity,
            callFunction = "saleMistake_Add"
        )
        //sales[position].originalQuantity = quantity
        sales[position].quantity = quantity
        //sales[position].startQuantity = quantity
        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position, type)

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Faltas la cantidad de " + totalQuantity + " del articulo " + sales[position].itemFk + "  ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message,"sendChekingPresence"
        )
    }

    private fun missing(position: Int, quantity: String) {
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "TRUE",
            originalQuantity = quantity,
            callFunction = "collectionMissingTrash",
        )
        //sales[position].originalQuantity = quantity
        sales[position].quantity = quantity
        //sales[position].startQuantity = quantity

        if (quantity == "0")
            markLine(position, type)
        saleAdapter!!.notifyDataSetChanged()

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Basura " + totalQuantity + " del articulo " + sales[position].itemFk + "  ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message,"sendChekingPresence"
        )
    }

    private fun reject(position: Int, quantity: String) {
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "reject",
            originalQuantity = quantity
         ,callFunction = "collectionMissingTrash")

        //sales[position].originalQuantity = quantity
        sales[position].quantity = quantity
        //sales[position].startQuantity = quantity
        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position, type)

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha modificado la cantidad original " + sales[position].originalQuantity + " del artículo " + sales[position].itemFk + " a nueva cantidad: " + sales[position].quantity + " del ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message,"sendChekingPresence"
        )
    }

    private fun increaseQuantity(position: Int, quantity: String) {
        viewModel.collectionIncreaseQuantity(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = quantity,"collection_increaseQuantity"
        )
        try {
            //val quantityPicked = (quantity.toInt() - sales[position].quantity.toInt()).toString()
            //sales[position].originalQuantity = quantity
            sales[position].quantity = quantity
            /*if (type == CONTROLADOR){
                   showShelving(position,0,quantityPicked)
               }*/
        } catch (e: Exception) {
        }
        saleAdapter?.notifyDataSetChanged()

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha modificado la cantidad original " + sales[position].originalQuantity + " del artículo " + sales[position].itemFk + " a nueva cantidad: " + sales[position].quantity + " del ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message,"sendChekingPresence"
        )
    }

    //ESTADOS
   /* private fun changeTicketState() {
        if (type == CONTROLADOR) {
            tickets.forEach {
                viewModel.collectionUpdateState(
                    usuario = user,
                    password = password,
                    collectionFk = it,
                    state = CHECKED,"collection_updateState"
                )
            }
        } else if (type == SACADOR) {
            tickets.forEach {
                viewModel.collectionUpdateState(
                    usuario = user,
                    password = password,
                    collectionFk = it,
                    state = PREPARED,"collection_updateState"
                )
            }

        }
    }*/

    private fun changeInitTicketState() {
        if (type == CONTROLADOR) {
            tickets.forEach {
                viewModel.collectionUpdateState(
                    usuario = user,
                    password = password,
                    collectionFk = it,
                    state = ON_CHECKING,"collection_updateState"
                )
            }
        } else if (type == SACADOR) {
            tickets.forEach {
                viewModel.collectionUpdateState(
                    usuario = user,
                    password = password,
                    collectionFk = it,
                    state = ON_PREPARATION,"collection_updateState"
                )
            }
        }
    }

    //OTROS
    private fun showScanner(index: Int, sale: SaleVO) {
        customDialogInput.setTitle("" + sale.itemFk)
            .setDescription(getString(R.string.Escaneaelcarroparaelitemseleccionado))
            .setOkButton("Aceptar") {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    findSale(customDialogInput.getValue(), index)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
            }.setKoButton("Cancelar") {
            customDialogInput.dismiss()
        }.setValue("").show()
        customDialogInput.getEditText().requestFocus()
        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    findSale(customDialogInput.getValue(), index)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun showErrorMessage(text: String) {
        customDialog.setTitle("Error al marcar la linea").setDescription(text)
            .setKoButton("Cerrar") {
                customDialog.dismiss()
            }.show()
    }

    private fun showMistakeList(list: List<MistakeTypeVO>) {
        if (splash_progress != null) splash_progress.visibility = View.GONE
        listPlacementSupply = ArrayList()
        list.forEach {
            listPlacementSupply.add(BarcodeVO(code = it.description))
        }


        customDialogList.setTitle("Causa de error").setKoButton("Cerrar") {
            scanRequest()
            hideKeyboards()
            customDialogList.dismiss()
        }.hideDialog().show()


        placementSupplyAdapter =
            BarcodeAdapter(listPlacementSupply, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    list.forEach {
                        if (it.description == item.code) {
                            viewModel.saleMistakeAdd(
                                usuario = user,
                                password = password,
                                vSaleFk = mistakeSale?.saleFk!!,
                                vUserFk = mistakeSale?.workerFk!!,
                                vTypeFk = it.id,"saleMistake_Add"
                            )
                            "Causa del error registrado".toast(requireContext())
                            customDialogList.dismiss()
                        }
                    }

                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

    fun isNumber(num: String): Boolean {
        var numberInt = 0
        try {
            numberInt = num.toInt()
            return true
        } catch (e: Exception) {
            return false
        }
    }

    //VOZ
    override fun onResults(results: Bundle) {
        super.onResults(results)
        Log.i("Speech", "onResults")
        val matches = results
            .getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        if (matches != null) {
            Log.i("RESPUESTA", matches[0])
            mIsListening = false
            mSpeechRecognizer!!.cancel()
            getText(matches[0])
        }
    }

    private fun getText(text: String) {
        val pos = checkText(text)
        when (pos) {
            NEW_COLLECTION -> {

            }
            CANCEL -> {

            }
            LISTO -> {
                state += 1
                speakOrder()
            }
            VOLVER -> {

            }
            REPITE -> {
                speakOrder()
            }
            OTRO -> {
                speak("Orden no registrada")
            }
            FALTA -> {
                speak("Orden no registrada")
            }
            else -> speak("Orden no registrada")
        }
    }

    private fun speakOrder() {
        if (type == SACADOR) {
            if (state == 0) {
                speak("Colección cargada.")
            } else {
                var index = 0
                for (sale in sales) {
                    if (sale.isPrepared == "0" && sale.isPreviousPrepared == "0" && sale.isControlled == "0") {
                        if (state == 1) {
                            //cantar pasillo
                            if (sale.placements.size > 0) {
                                var placementIndex = 0
                                for (placement in sale.placements) {
                                    var visible = placement.visible
                                    visible = visible.replace("(", "")
                                    visible = visible.replace(")", "")
                                    var quantityVisible = 0
                                    try {
                                        quantityVisible = visible.toInt()
                                    } catch (e: Exception) {
                                    }

                                    if (quantityVisible > 0) {
                                        val pasillo = placement.placement.subSequence(
                                            0,
                                            sale.placements[0].placement.indexOf("-")
                                        ).toString()
                                        val sector = placement.placement.subSequence(
                                            sale.placements[0].placement.indexOf("-") + 1,
                                            sale.placements[0].placement.length
                                        ).toString()
                                        var letras: ArrayList<Char> = ArrayList()
                                        val carro = placement.shelving
                                        var carroSeparate: StringBuilder = StringBuilder("")
                                        for (l in carro) {
                                            carroSeparate.append(l)
                                            carroSeparate.append(" ")
                                        }


                                        placementPicked = placement
                                        storedShelvingPosition = placementIndex

                                        if (lastPlacementPicked == null || lastPlacementPicked!!.placement != placement.placement) {
                                            lastPlacementPicked = placement
                                            try {
                                                speak("Pasillo " + pasillo.toInt() + ", Sector " + sector.toInt() + ",  " + carroSeparate.toString())
                                            } catch (e: Exception) {
                                                speak("Pasillo " + pasillo + " Sector " + sector + " Carro " + carro)
                                            }
                                        } else if (lastPlacementPicked != null && lastPlacementPicked!!.shelving != placement.shelving) {
                                            speak(" Carro " + carro)
                                        } else {
                                            state = 2
                                            speakOrder()
                                        }

                                        break
                                    }

                                    placementIndex += 1
                                }

                            } else {
                                speak("El item " + sale.itemFk + " no se encuentra disponible. Se necesita actuación manual.")
                            }

                        } else if (state == 2) {
                            //cantar articulo
                            try {
                                val item = sale.itemFk
                                val cantidad = sale.quantity.toInt() - sale.pickedQuantity.toInt()
                                val listtext = sale.longName.split(" ")
                                var nombre: String
                                if (listtext.isNotEmpty()) {
                                    nombre = listtext.get(0) + " " + sale.color
                                } else {
                                    nombre = sale.longName + " " + sale.color
                                }

                                speak("Item " + item.toInt() + " " + nombre + " , Cantidad: " + cantidad)
                            } catch (e: Exception) {
                                speak("Se necesita actuación manual")
                            }
                        } else if (state == 3) {
                            //marcar linea
                            var visible = placementPicked!!.visible
                            visible = visible.replace("(", "")
                            visible = visible.replace(")", "")
                            var quantityVisible = 0
                            try {
                                quantityVisible = visible.toInt()
                            } catch (e: Exception) {
                            }

                            var needPicked = 0
                            try {
                                needPicked = sale.quantity.toInt() - sale.pickedQuantity.toInt()
                            } catch (e: Exception) {
                            }

                            storedPosition = index
                            if (quantityVisible < needPicked) {
                                onQuantityOfShelvingSelected(
                                    itemShelvingFk = placementPicked!!.shelving,
                                    quantity = quantityVisible.toString()
                                )
                            } else {
                                onQuantityOfShelvingSelected(
                                    itemShelvingFk = placementPicked!!.shelving,
                                    quantity = needPicked.toString()
                                )
                            }

                            state = 0
                            //speak("Acción registrada. Diga listo para continuar")
                            speak("Registrada")

                        }

                        break

                    }

                    index += 1
                }

                if (index >= sales.size) {
                    speak("Colección completada.")
                }
            }
        }
    }

    override fun onDestroy() {
        cancelSpeech()
        super.onDestroy()
    }

}
