package es.verdnatura.presentation.view.feature.collection.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetLoginUserCase
import es.verdnatura.domain.GetSacadorControladorUserCase
import es.verdnatura.domain.GetUbicadorUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageError
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.getMessageFromResponse
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.collection.ItemVO
import es.verdnatura.presentation.view.feature.collection.mapper.map
import es.verdnatura.presentation.view.feature.sacador.model.*
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class CollectionViewModel(context: Context) : BaseViewModel()
{

    val emptyMessage = "La colección no tiene tickets"

    private val getSacadorControladorUserCase: GetSacadorControladorUserCase =
        GetSacadorControladorUserCase(context)
    private val getUbicadorUserCase: GetUbicadorUserCase = GetUbicadorUserCase(context)
    private val getLoginUserCase: GetLoginUserCase = GetLoginUserCase(context)

    private val _collectionTicketList by lazy { MutableLiveData<CollectionVO>() }
    val collectionTicketList: LiveData<CollectionVO>
        get() = _collectionTicketList

    private val _placementSuppleyList by lazy { MutableLiveData<PlacementSupplyListVO>() }
    val placementSuppleyList: LiveData<PlacementSupplyListVO>
        get() = _placementSuppleyList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    private val _responsecheckfully by lazy { MutableLiveData<ResponseItemVO>() }
    val responsecheckfully: LiveData<ResponseItemVO>
        get() = _responsecheckfully

    private val _responseNew by lazy { MutableLiveData<ResponseItemVO>() }
    val responseNew: LiveData<ResponseItemVO>
        get() = _responseNew

    private val _responseCode by lazy { MutableLiveData<ResponseItemVO>() }
    val responseCode: LiveData<ResponseItemVO>
        get() = _responseCode

    private val _responseIncQuantity by lazy { MutableLiveData<ResponseItemVO>() }
    val responseIncQuantity: LiveData<ResponseItemVO>
        get() = _responseIncQuantity

    private val _mistakeList by lazy { MutableLiveData<MistakeTypeListVO>() }
    val mistakeList: LiveData<MistakeTypeListVO>
        get() = _mistakeList

    private val _responseSalixMessage by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSalixMessage: LiveData<ResponseItemVO>
        get() = _responseSalixMessage

    private val _item by lazy { MutableLiveData<ItemVO>() }
    val item: LiveData<ItemVO>
        get() = _item

    fun collectionTicketGet(
        usuario: String,
        password: String,
        collectionFk: String,
        sectorFk: String,
        type: String,
        callFunction:String){
        getSacadorControladorUserCase.collectionTicketGet(
            usuario,
            password,
            collectionFk,
            sectorFk,
            "0",
            type
                                                         ).enqueue(object :
            Callback<CollectionVO>
        {
            override fun onFailure(call: Call<CollectionVO>, t: Throwable)
            {
                _collectionTicketList.value = CollectionVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                          )
            }

            override fun onResponse(
                call: Call<CollectionVO>,
                response: Response<CollectionVO>
                                   )
            {
                if (response.body() != null)
                {
                    _collectionTicketList.value = response.body()?.let { it.map() }
                } else
                {
                    _collectionTicketList.value = CollectionVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                }
            }
        })
    }

    fun saleTrackingReplace(
        usuario: String,
        password: String,
        saleFk: String,
        vOriginalQuantity: String,
        vStateFk: String,
        vIsChecked: String,
        vBuyFk: String,callFunction:String)
    {
        getSacadorControladorUserCase.saleTrackingReplace(
            usuario,
            password,
            saleFk,
            vOriginalQuantity,
            vStateFk,
            vIsChecked,
            vBuyFk
                                                         ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun itemPlacementSupplyAiming(
        usuario: String,
        password: String,
        shelvingFk: String,
        quantity: String,
        itemFk: String,callFunction:String)
    {
        getSacadorControladorUserCase.itemPlacementSupplyAiming(
            usuario,
            password,
            shelvingFk,
            quantity,
            itemFk
                                                               ).enqueue(object :
            Callback<List<PlacementSupplyVO>>
        {
            override fun onFailure(call: Call<List<PlacementSupplyVO>>, t: Throwable)
            {
                val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                listError.add(
                    PlacementSupplyVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                     )
                             )
                _placementSuppleyList.value = PlacementSupplyListVO(listError)
            }

            override fun onResponse(
                call: Call<List<PlacementSupplyVO>>,
                response: Response<List<PlacementSupplyVO>>
                                   )
            {
                if (response.body() != null)
                {
                    _placementSuppleyList.value = response.body()?.let { PlacementSupplyListVO(it) }
                } else
                {
                    val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                    listError.add(
                        PlacementSupplyVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                         )
                                 )
                    _placementSuppleyList.value = PlacementSupplyListVO(listError)
                }
            }
        })
    }

    fun itemShelvingSaleSupplyAdd(
        usuario: String,
        password: String,
        itemShelvingFk: String,
        saleFk: String,
        quantity: String,callFunction:String
                                 )
    {
        getSacadorControladorUserCase.itemShelvingSaleSupplyAdd(
            usuario,
            password,
            itemShelvingFk,
            saleFk,
            quantity
                                                               ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun collectionStickerPrint(
        usuario: String,
        password: String,
        collectionFk: String,
        sectorFk: String,callFunction:String)

    {
        getSacadorControladorUserCase.collectionStickerPrint(
            usuario,
            password,
            collectionFk,
            sectorFk
                                                            ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(isError = true, errorMessage = getMessageFromAllResponse(callFunction,t.message!!))
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

   /* fun itemGetAvailable(usuario: String, password: String, itemFk: String, warehouseFk: String,callFunction:String)
    {
        getSacadorControladorUserCase.itemGetAvailable(usuario, password, itemFk, warehouseFk)
            .enqueue(object : Callback<ItemVO>
            {
                override fun onFailure(call: Call<ItemVO>, t: Throwable)
                {
                    _item.value = ItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                        )
                }

                override fun onResponse(call: Call<ItemVO>, response: Response<ItemVO>)
                {
                    if (response.body() == null)
                    {
                        _item.value = ItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                            )
                    } else
                    {
                        _item.value = response.body()
                    }
                }
            })
    }*/

    fun collectionAddItem(
        usuario: String,
        password: String,
        itemFk: String,
        quantityFk: String,
        ticketFk: String,
        warehouseFk: String,callFunction:String)

    {
        getSacadorControladorUserCase.collectionAddItem(
            usuario,
            password,
            itemFk,
            quantityFk,
            ticketFk,
            warehouseFk
                                                       ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responseNew.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                   )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {

              if (!response.isSuccessful){
                  _responseNew.value = ResponseItemVO(
                      isError = true,
                      errorMessage = getMessageFromAllResponse(callFunction,response.message()))
              }else{
                  _responseNew.value =
                      ResponseItemVO(isError = false, response = response.body()!!)
              }


               /* if (response.body() == null)
                {
                    _responseNew.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                       )
                } else
                {
                    _responseNew.value =
                        ResponseItemVO(isError = false, response = response.body()!!)
                }*/
            }
        })
    }

    fun saleMove(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,
        originalQuantity: String,callFunction:String)

    {
        getSacadorControladorUserCase.saleMove(
            usuario,
            password,
            saleFk,
            quantity,
            originalQuantity
                                              ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message()))

                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun collectionMissingTrash(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,
        type: String,
        warehouseFk: String,
        originalQuantity: String,callFunction:String
                              )
    {
        getSacadorControladorUserCase.collectionMissingTrash(
            usuario,
            password,
            saleFk,
            quantity,
            type,
            warehouseFk,
            originalQuantity
                                                            ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun saleTrackingDel(usuario: String, password: String, saleFk: String,callFunction:String)
    {
        getSacadorControladorUserCase.saleTrackingDel(usuario, password, saleFk)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                        )
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun parking(usuario: String, password: String, ticketFk: String, parking: String,callFunction:String)
    {
        getUbicadorUserCase.shelvingPark(usuario, password, ticketFk, parking)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {

                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                        )
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun collectionUpdateState(
        usuario: String,
        password: String,
        collectionFk: String,
        state: String,callFunction:String
                             )
    {
        getSacadorControladorUserCase.collectionUpdateState(usuario, password, collectionFk, state)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage =getMessageFromAllResponse(callFunction,t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                        )
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun getIdFromCode(usuario: String, password: String, code: String,callFunction:String)
    {
        getSacadorControladorUserCase.getIdFromCode(usuario, password, code)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _responseCode.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                        )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _responseCode.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                            )
                    } else
                    {
                        _responseCode.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun collectionIncreaseQuantity(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,callFunction:String
                                  )
    {
        getSacadorControladorUserCase.collectionIncreaseQuantity(
            usuario,
            password,
            saleFk,
            quantity,

                                                                ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responseIncQuantity.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                           )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _responseIncQuantity.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                               )
                } else
                {
                    _responseIncQuantity.value =
                        ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun sendChekingPresence(token: String, workerId: String, message: String,calledFunction:String)
    {
        try
        {
            getLoginUserCase.sendChekingPresence(token, workerId.toInt(), message)
                .enqueue(object : Callback<Boolean>
                {
                    override fun onResponse(call: Call<Boolean>, response: Response<Boolean>)
                    {

                        if (!response.isSuccessful)
                        {
                            _responseSalixMessage.value = ResponseItemVO(
                                isError = true,
                                errorMessage = "Error en la llamada $calledFunction al enviar mensaje rocket." + getMessageFromResponse(
                                    response.message()
                                                                                                                                       ))
                            //Log.i("Salix","Mensaje enviado a salix")
                        }

                    }

                    override fun onFailure(call: Call<Boolean>, t: Throwable)
                    {

                        //Log.i("Salix Error",""+t.message)
                    }
                })
        } catch (e: Exception)
        {
        }

    }

    fun saleMistakeAdd(
        usuario: String,
        password: String,
        vSaleFk: String,
        vUserFk: String,
        vTypeFk: String,callFunction:String
                      )
    {
        getSacadorControladorUserCase.saleMistakeAdd(usuario, password, vSaleFk, vUserFk, vTypeFk)
            .enqueue(object :
                Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage =getMessageFromAllResponse(callFunction,t.message!!)
                                                    )
                }

                override fun onResponse(
                    call: Call<String>,
                    response: Response<String>
                                       )
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                        )
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun mistakeType(usuario: String, password: String)
    {
        getSacadorControladorUserCase.mistakeType(usuario, password)
            .enqueue(object : Callback<List<MistakeTypeVO>>
            {
                override fun onFailure(call: Call<List<MistakeTypeVO>>, t: Throwable)
                {
                    val listError: ArrayList<MistakeTypeVO> = ArrayList()
                    listError.add(MistakeTypeVO("0", ""))
                    _mistakeList.value = MistakeTypeListVO(listError)
                }

                override fun onResponse(
                    call: Call<List<MistakeTypeVO>>,
                    response: Response<List<MistakeTypeVO>>
                                       )
                {
                    if (response.body() != null)
                    {
                        _mistakeList.value = response.body()?.let { MistakeTypeListVO(it) }
                    } else
                    {
                        val listError: ArrayList<MistakeTypeVO> = ArrayList()
                        listError.add(MistakeTypeVO("0", ""))
                        _mistakeList.value = MistakeTypeListVO(listError)
                    }
                }

            })
    }

    fun ticketOrCollection_checkFullyControlled(
        usuario: String,
        password: String,
        collectionFk: String,callFunction:String
                                               )
    {
        getSacadorControladorUserCase.ticketOrCollection_checkFullyControlled(
            usuario,
            password,
            collectionFk
                                                                             ).enqueue(object :
            Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responsecheckfully.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                          )
            }

            override fun onResponse(
                call: Call<String?>,
                response: Response<String>
                                   )
            {

                if (response.body() != "false")
                {
                    //sergio:
                    _responsecheckfully.value = ResponseItemVO(
                        isError = false,
                        response = response.body()!!,
                        errorMessage = ""
                                                              )
                }
            }
        })
    }

}
