package es.verdnatura.presentation.view.feature.collection.mapper

import es.verdnatura.presentation.common.convertToDateString
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import java.text.SimpleDateFormat
import java.util.*

fun CollectionVO.map() : CollectionVO{
    try {
        this.tickets.forEach { ticket ->
            ticket.sales.forEach {
                it.ticketFk = ticket.ticketFk
                it.level = ticket.level
                it.salePersonFk = ticket.salesPersonFk
                it.agencyName = ticket.agencyName
                it.quantity = it.quantity.substring(0, it.quantity.indexOf("."))
                it.isNew = it.isAdded == "1"
                it.originalQuantity =
                    if (it.originalQuantity != "-1") it.originalQuantity.substring(
                        0,
                        it.originalQuantity.indexOf(".")
                    ) else it.quantity
                it.pickedQuantity =
                    if (it.isPrepared == "1" || it.isPreviousPrepared == "1" || it.isControlled == "1") it.quantity else it.pickedQuantity
                if (!it.placements.isNullOrEmpty()) {
                    it.placements.forEachIndexed { index, placement ->
                        if (index == 0) {
                            try {
                                it.saleOrder = placement.saleOrder.toInt()
                            } catch (e: NumberFormatException) {
                                it.saleOrder = 9999
                            }
                        }
                        placement.created =
                            getCalendarFromDate(placement.created).convertToDateString()
                        placement.visible = "(" + (if(placement.visible.toInt() > 0) placement.visible else "0") + ")"
                    }
                } else {
                    it.placements = listOf()
                }

            }
        }
    }catch (e:Exception){
        this.isError = true
        if (this.tickets == null || this.tickets.isEmpty()){
            this.errorMessage = "La colección no tiene tickets"
        }else{
            this.errorMessage = "Error al construir la colección. Descripción del error:"+ e.message.toString()
        }

    }
    return this
}

private fun getCalendarFromDate(date:String): Calendar {
    val sdf = SimpleDateFormat("yyyy-MM-dd")
    val cal = Calendar.getInstance()
    cal.setTime(sdf.parse(date))
    return cal
}