package es.verdnatura.presentation.view.feature.controlador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import es.verdnatura.R
import es.verdnatura.databinding.FragmentControladorBinding
import es.verdnatura.domain.ConstAndValues
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnCollectionSelectedListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_controlador.*
import kotlinx.android.synthetic.main.toolbar.*

class ControladorFragment : BaseFragment<FragmentControladorBinding,ControladorViewModel>(ControladorViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var goBack:Boolean = false
    private var onCollectionSelectedListener : OnCollectionSelectedListener? = null
    override fun getLayoutId(): Int = R.layout.fragment_controlador
    private lateinit var customDialog: CustomDialog
    private var type = ""

    companion object {
        fun newInstance() = ControladorFragment()
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun init() {

        splash_progress.visibility = View.GONE
        customDialog = CustomDialog(requireContext())
        toolbar_title.text = getString(R.string.controlticket)
        setEvents()
        requireActivity().main_bottom_navigation.visibility = View.VISIBLE
        super.init()
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()

        type=ConstAndValues.CONTROLADOR

        super.onCreate(savedInstanceState)
    }

    private fun setEvents(){

        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                goBack=false
                if (!scan_input.text.isNullOrEmpty()) {
                    splash_progress.visibility = View.VISIBLE

                   viewModel.collectionTicketGet(
                        usuario = user,
                        password = password,
                        sectorFk = sectorFk,
                        collectionFk = scan_input.text.toString(),
                        type = type
                    ,callFunction = "collection_getTickets")
   // viewModel.ticket_checkControlled(user,password,scan_input.text.toString())
                                  }
                scan_input.setText("")
                (activity as MainActivity).hideKeyboard(scan_input)
                return@setOnEditorActionListener true
            }
            false
        }
    }

    override fun observeViewModel() {
        with(viewModel){
            collectionTicketList.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Aceptar"){
                        customDialog.dismiss()
                    }.show()
                }else{

                    if (!goBack)navigateToCollectionList(it)
                    goBack = false
                }

            })

        }
    }

    private fun navigateToCollectionList(collection: CollectionVO){
        ////Log.i("VERDNATURA:","vamos a oncolecctionSelected ${collection.isError}")
        onCollectionSelectedListener?.onCollectionSelected(collection, ConstAndValues.CONTROLADOR)
    }




}
