package es.verdnatura.presentation.view.feature.controlvehiculo.fragment

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Bundle
import android.util.Log
import android.util.TypedValue
import android.view.View
import android.view.View.*
import android.view.inputmethod.EditorInfo
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentVehiclecontrolBinding
import es.verdnatura.domain.ConstAndValues
import es.verdnatura.domain.ConstAndValues.MAINACTIVITY
import es.verdnatura.domain.ConstAndValues.SACADOR
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.historicovehiculo.fragment.HistoricoVehiculoFragment
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.fragment.SacadorFragment
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import kotlinx.android.synthetic.main.component_custom_three_dialog.*
import kotlinx.android.synthetic.main.fragment_collection.*
import kotlinx.android.synthetic.main.fragment_controlador.*
import kotlinx.android.synthetic.main.fragment_controlador.scan_input
import kotlinx.android.synthetic.main.fragment_controlador.splash_progress
import kotlinx.android.synthetic.main.fragment_login.*
import kotlinx.android.synthetic.main.fragment_sacador.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class ControlVehiculoFragment(
    var tagName: String = ""
) : BaseFragment<FragmentVehiclecontrolBinding, ControlVehiculoViewModel>(
    ControlVehiculoViewModel::class
) {

    private var user = ""
    private var userFk = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var goBack: Boolean = false
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    override fun getLayoutId(): Int = R.layout.fragment_vehiclecontrol
    private lateinit var customDialog: CustomDialog
    private var isWorkedTimed = "UNREGISTERED"

    private lateinit var customDialogList: CustomDialogList
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons

    companion object {
        fun newInstance(tagName: String) = ControlVehiculoFragment(tagName)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        goBack = false

        splash_progress.visibility = GONE

        /*  if (getWagons() < 2 && !tagName.equals(SacadorFragment.TAG))
          {
                  (activity as MainActivity).openFragmentPickers()
          }*/


        if (tagName.equals(HistoricoVehiculoFragment.TAG)) {
            setToolBar("HISTORY")
            showScanner("Escanea matrícula para ver histórico", "HISTORY")
        } else {
            checkControlTimeVehicle()
            setToolBar("ALL")
        }
        setEvents()

        super.init()
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    private fun setToolBar(direction: String) {

        toolbar_title.text = getString(R.string.vehiclecontrol)
        toolbar_title.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13F)

        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconHistory: Drawable =
            resources.getDrawable(R.drawable.ic_history_black_24dp, resources.newTheme())
        val iconVehicleIn: Drawable = resources.getDrawable(R.drawable.car, resources.newTheme())
        val iconVehicleOut: Drawable =
            resources.getDrawable(R.drawable.car_off, resources.newTheme())



        if (direction.equals("OUT")) {
            listIcons.remove(iconVehicleIn)
            listIcons.add(iconVehicleOut)
            listIcons.add(iconHistory)
        }
        if (direction.equals("IN")) {
            listIcons.remove(iconVehicleOut)
            listIcons.add(iconVehicleIn)
            listIcons.add(iconHistory)
        }
        if (direction.equals("HISTORY")) {
            listIcons.remove(iconVehicleOut)
            listIcons.remove(iconVehicleIn)
            listIcons.add(iconHistory)
        }


        toolbar_icons.adapter = ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {

                if (item == iconHistory) {
                    showScanner("Escanea matrícula para ver histórico", "HISTORY")
                }

                if (item == iconVehicleIn) {
                    showScanner("Escanea matrícula del vehículo", "IN")
                }
                if (item == iconVehicleOut) {
                    showScanner("Escanea matrícula para dejar vehículo", "OUT")
                }

            }
        })
        toolbar_icons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun showScanner(description: String, action: String) {

        customDialogInput.setTitle(getString(R.string.vehiclecontrol)).setDescription(description)
            .setOkButton("Aceptar") {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    // "$description ${customDialogInput.getValue()} OK".toast(requireContext())
                    insertControlTimeVehicle(customDialogInput.getValue(), action)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
            }

            .setKoButton("Cancelar") {
                customDialogInput.dismiss()
            }.setValue("").show()

        customDialogInput.getEditText().requestFocus()

        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {

                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    // //Log.i("VERDNATURA:", "El valor es ${customDialogInput.getValue()}")
                    // "$description ${customDialogInput.getValue()} OK".toast(requireContext())
                    insertControlTimeVehicle(customDialogInput.getValue(), action)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun checkControlTimeVehicle() {

        if (!goBack) {
            splash_progress.visibility = VISIBLE
            viewModel.workerMachinery_isRegistered(user, password,"workerMachinery_isRegistered")
        }
    }

    private fun insertControlTimeVehicle(plateNumber: String, direction: String) {

        if (direction.equals("HISTORY")) {
            (activity as MainActivity).onPasillerosItemClickListener(
                PasillerosItemVO(title = "Historico Vehículo"),
                plateNumber
            )
        } else if (direction.equals("IN")) {
            splash_progress.visibility = VISIBLE
            viewModel.machineWorker_add(user, password, plateNumber,"machineWorker_add")

        } else if (direction.equals("OUT")) {
            viewModel.machineWorker_update(user, password, plateNumber,"machineWorker_update")

        } else if (direction.equals("")) {
            (activity as MainActivity).openFragmentPickers()
        }

    }

    private fun scanRequest() {
        if (scan_input != null) {
            scan_input.requestFocus()
        }
        hideKeyboards()
    }

    private fun hideKeyboards() {
        try {
            requireActivity().hideKeyboard()
        } catch (e: Exception) {
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        userFk = prefs.getString(USERFK, "0").toString()
        super.onCreate(savedInstanceState)
    }

    private fun setEvents() {

        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!scan_input.text.isNullOrEmpty()) {

                    if (tagName.equals(HistoricoVehiculoFragment.TAG)) {
                        showOptionsHistorical(scan_input.text.toString())
                    } else {
                        showOptionsVehicleControl(scan_input.text.toString())
                    }
                }
                scan_input.setText("")
                (activity as MainActivity).hideKeyboard(scan_input)
                return@setOnEditorActionListener true
            }
            false
        }
    }

    private fun showOptionsVehicleControl(plate: String) {

        customDialogThreeButtons.custom_dialog_value.visibility = GONE

        customDialogThreeButtons.setDescription("¿Qué desea hacer con el vehículo de matrícula $plate")
            .setValue(plate)
        if (isWorkedTimed.equals("UNREGISTERED")) {
            customDialogThreeButtons.setOkButtonTwo("COGER") {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    "Vuelva a escanear".toast(requireContext())
                } else {
                    splash_progress.visibility = VISIBLE
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "IN")
                }
            }
        } else {

            customDialogThreeButtons.setOkButtonThree("DEJAR") {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    "Vuelva a escanear".toast(requireContext())
                } else {
                    splash_progress.visibility = VISIBLE
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "OUT")
                }
            }
        }.setOkButtonFour("HISTÓRICO") {
            if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                "Vuelva a escanear".toast(requireContext())
            } else {
                customDialogThreeButtons.dismiss()
                insertControlTimeVehicle(customDialogThreeButtons.getValue(), "HISTORY")

            }

        }.setKoButton("Cancelar") {
            scanRequest()
            customDialogThreeButtons.dismiss()
        }.show()
    }

    private fun showOptionsHistorical(plate: String) {

        customDialogThreeButtons.custom_dialog_value.visibility = GONE

        customDialogThreeButtons.setDescription("¿Qué desea hacer con el vehículo de matrícula $plate")
            .setValue(plate)

            .setOkButtonFour("HISTÓRICO") {
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()) {
                    "Vuelva a escanear".toast(requireContext())
                } else {
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "HISTORY")

                }

            }.setKoButton("Cancelar") {
                scanRequest()
                customDialogThreeButtons.dismiss()
            }.show()
    }

    override fun observeViewModel() {

        with(viewModel) {
            splash_progress.visibility = GONE
            responseinsert.observe(viewLifecycleOwner, Observer {
                Log.i(
                    "VERDNATURA",
                    "Respuesta call vehicle_insert en el viewmodel es : ${it.response}"
                )

                if (it.isError) {

                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                    if (it.response == null || it.response == "false") {

                        customDialog.setTitle("Error")
                            .setDescription("Revisar llamada del registro de vehículos")
                            .setOkButton("Cerrar") {
                                customDialog.dismiss()
                            }.show()

                    } else {
                        if (it.response == "true") {
                            //Log.i(TD,"insertado registro")
                            //"Acción registrada correctamente".toast(context)
                            /*   customDialog.setTitle("Llamada correcta")
                                   .setDescription("Acción registrada satisfactoriamente")
                                   .setOkButton("Cerrar") {
                                       customDialog.dismiss()
                                   }.show()*/
                            (activity as MainActivity).openFragmentPickers()
                            //(activity as MainActivity).onPasillerosItemClickListener( PasillerosItemVO(title = "Sacadores"),"CONTROL")
                            //checkControlTimeVehicle()
                        }

                    }
                }
                splash_progress.visibility = GONE
            })

            responsecontrol.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = GONE
                // Log.i("VERDNATURA:","${it.response}")
                if (it.isError) {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                    if (it.response == null) {
                        customDialog.setTitle("Error")
                            .setDescription("Revisar llamada de control de vehículos")
                            .setOkButton("Cerrar") {
                                customDialog.dismiss()
                            }.show()
                    } else {
                        if (it.response.equals("1")) {
                            isWorkedTimed = "REGISTERED"
                            setToolBar("OUT")
                            /*  customDialog.setTitle("Control de vehículos")
                                  .setDescription("SI Tienes coche asignado")
                                  .setOkButton("Cerrar") {
                                      customDialog.dismiss()
                                  }.show()*/
                            if (!tagName.equals(SacadorFragment.TAG)) {
                                (activity as MainActivity).openFragmentPickers()
                            }

                        } else {

                            if (it.response.equals("0")) {
                                setToolBar("IN")
                                customDialog.setTitle("Control de vehículos")
                                    .setDescription("Escanea la matrícula del vehículo o elige sin vehículo")
                                    .setOkButton("Escanear vehículo") {
                                        customDialog.dismiss()
                                        showScanner("Escanea matrícula del vehículo", "IN")
                                    }
                                    .setKoButton("SIN VEHÍCULO") {
                                        customDialog.dismiss()

                                        (activity as MainActivity).openFragmentPickers()
                                        // getActivity()?.getFragmentManager()?.popBackStack();
                                    }.show()

                            }
                        }

                    }
                }
            })

        }

    }
}


