package es.verdnatura.presentation.view.feature.diadeventa.fragment

import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentItemdayofsaleCardBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.diadeventa.adapter.DayOfSaleAdapter
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionStateRow
import es.verdnatura.presentation.view.feature.paletizador.model.itemExpedetionState
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.buyers_fragment.*
import kotlinx.android.synthetic.main.fragment_historicovehiculo.*
import kotlinx.android.synthetic.main.fragment_item_card.*
import kotlinx.android.synthetic.main.fragment_item_card.edit_itemFk
import kotlinx.android.synthetic.main.fragment_item_card.splash_progress
import kotlinx.android.synthetic.main.fragment_itemexpeditionstate_card.*
import kotlinx.android.synthetic.main.fragment_itemshelvingdayofsale.*
import kotlinx.android.synthetic.main.fragment_parking.*
import kotlinx.android.synthetic.main.toolbar.*

class DayOfSaleFragment(
    var entryPoint: String = ""
                       ) : BaseFragment<FragmentItemdayofsaleCardBinding, DayOfSaleViewModel>(DayOfSaleViewModel::class) {

    private lateinit var customDialogList: CustomDialogList
    private var user = ""
    private var password = ""
    private var warehouseFk = ""
    private var adapter: DayOfSaleAdapter? = null
    private var listItemsRow: ArrayList<ItemExpeditionStateRow> = ArrayList()
    private var itemScan = ""

    companion object {
        fun newInstance(entryPoint: String) = DayOfSaleFragment(entryPoint)
    }

    private lateinit var customDialog: CustomDialog

    override fun getLayoutId(): Int = R.layout.fragment_itemdayofsale_card

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        setEvents()
        setToolBar(getString(R.string.dayofSale))
        super.init()

    }

    private fun setToolBar(title: String) {
        toolbar_title.text = title


        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconParking: Drawable =
            resources.getDrawable(R.drawable.ic_local_parking_black_24dp, resources.newTheme())

        listIcons.add(iconParking)

        toolbar_icons.adapter = ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {

                if (item == iconParking) {
                    showParking()

                }


            }
        })
        toolbar_icons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    private fun setEvents() {
        edit_itemFk.requestFocus()
        edit_itemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_itemFk.text.toString().isNullOrEmpty()) {
                        getItemsSaleDate(edit_itemFk.text.toString())

                    itemScan = edit_itemFk.text.toString()
                }
                edit_itemFk.setText("")
                (activity as MainActivity).hideKeyboard(edit_itemFk)
                return@setOnEditorActionListener true
            }
            false
        }


        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

    }




    private fun getItemsSaleDate(vShelvingFK: String) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        splash_progress.visibility = View.VISIBLE
        viewModel.itemShelving_getSaleDate(user, password,vShelvingFK,"itemShelving_getSaleDate")

    }


    override fun observeViewModel() {
        with(viewModel) {

         loadItemShelvingSaleDateList.observe(viewLifecycleOwner, Observer{ event ->
                    event.getContentIfNotHandled().notNull {
                        splash_progress.visibility = View.GONE
                        adapter = DayOfSaleAdapter(it.list)

                        itemexpeditionstate_recyclerview.adapter = adapter
                        itemexpeditionstate_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)


                        // navigateToToday(it)
               if  (it.list.isEmpty()){
                   customDialog.setTitle("Sin resultados")
                       .setDescription("No se ha podido obtener datos de la etiqueta escaneada. ")
                       .setOkButton("Cerrar") {
                           customDialog.dismiss()
                       }.show()
                }else{

                        if (it.list.get(0).isError){
                            customDialog.setTitle("Error")
                                .setDescription(it.list.get(0).errorMessage)
                                .setOkButton("Cerrar") {
                                    customDialog.dismiss()
                                }.show()
                        }
                }

                    }
                })
            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){

                    splash_progress.visibility = View.GONE
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()

                }else{

                    customDialog.setTitle("Información").setDescription("Parking ok").setOkButton("Cerrar") {
                        customDialog.dismiss()
                    }.show()

                }


            })


        }
    }


  /*  private fun setItemExpeditionState(itemInfo: itemExpedetionState) {
        setToolBar(getString(R.string.wagon) +":"+ itemScan)

        listItemsRow = ArrayList()
        for (i in itemInfo.list) {
            listItemsRow.add(
                ItemExpeditionStateRow(
                    title = i.key,
                    value = i.value.replace(".0",""),
                    isEditable = false
                                      )
                            )
        }



        itemexpeditionstate_recyclerview.adapter = adapter
        itemexpeditionstate_recyclerview.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)


    }*/
    private fun showParking() {

        customDialogList.setTitle("Escanea el parking")
            .setOkButton("Cerrar") {

                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

                customDialogList.dismiss()

            }.setValue("").show()


        customDialogList.getEditText().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

        customDialogList.getEditText().setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5 || actionId == 6) {
                if (customDialogList.getValue().isNotEmpty()) {
                    parking(customDialogList.getValue())
                }
                customDialogList.setValue("")
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }


    }

    fun parking(parking:String){
        viewModel.parking(
            usuario = user,
            password = password,
            parking = parking,
            scanItem = itemScan,
                         callFunction="shelvingPark")
        customDialogList.dismiss()

    }


}


