package es.verdnatura.presentation.view.feature.historico.fragment

import android.content.SharedPreferences
import android.content.res.ColorStateList
import android.graphics.Color
import android.graphics.Color.*
import android.graphics.drawable.Drawable
import android.view.Gravity
import android.view.View
import android.widget.Toast
import androidx.appcompat.widget.TooltipCompat
import androidx.core.content.ContextCompat.getColor
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.R.color.verdnatura_pumpkin_orange
import es.verdnatura.R.color.verdnatura_white
import es.verdnatura.databinding.FragmentHistoricoBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.feature.historico.adapter.HistoricoAdapter
import es.verdnatura.presentation.view.feature.historico.model.ItemHistoricoListVO
import es.verdnatura.presentation.view.feature.historico.model.ItemHistoricoVO
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_buscar_item.*
import kotlinx.android.synthetic.main.fragment_historico.*
import kotlinx.android.synthetic.main.fragment_historico.splash_progress
import kotlinx.android.synthetic.main.fragment_inventary.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*


class HistoricoFragment(
    var itemFk: String = ""
) : BaseFragment<FragmentHistoricoBinding, HistoricoViewModel>(
    HistoricoViewModel::class
) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter: HistoricoAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialog: CustomDialog
    private var listHistoric: ArrayList<ItemHistoricoVO> = ArrayList()
    private var listHistoryAux: ArrayList<ItemHistoricoVO> = ArrayList()
    private var filter: String = ""

    companion object {
        fun newInstance(entryPoint: String) = HistoricoFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_historico

    override fun init() {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        filter = "all"
        viewModel.itemDiary(user, password, itemFk, warehouseFk,"itemDiary")
        toolbar_title.text = getString(R.string.itemdiary) + ":" + itemFk
        setToolBar()
        setEvents()

        super.init()
    }

    fun generateColorStateList(
        uncheckedColor: Int = Color.WHITE,
        checkedColor: Int = verdnatura_pumpkin_orange


    ): ColorStateList {
        val states = arrayOf(

            intArrayOf(-android.R.attr.state_pressed),
            intArrayOf(android.R.attr.state_pressed)

        )
        val colors = intArrayOf(

            uncheckedColor,
            checkedColor

        )
        return ColorStateList(states, colors)
    }

    private fun setToolBar() {
        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconReload: Drawable =
            resources.getDrawable(R.drawable.ic_transaction, resources.newTheme())
        val iconGargabe: Drawable =
            resources.getDrawable(R.drawable.alpha_b_circle_outline, resources.newTheme())
        val iconFault: Drawable =
            resources.getDrawable(R.drawable.alpha_f_circle_outline, resources.newTheme())

        listIcons.add(iconGargabe)
        listIcons.add(iconFault)
        listIcons.add(iconReload)


        toolbar_icons.adapter = ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {

            override fun onOptionsItemSelected(item: Drawable) {


                if (item == iconReload) {
                    splash_progress.visibility = View.VISIBLE
                    viewModelwithFilter("all")
                    // TooltipCompat.setTooltipText(view!!, "Recarga");
                    // view?.let { TooltipCompat.setTooltipText(it, "Recarga") };
                    /* val mytoast = Toast.makeText(
                         context,
                         "Toast Message",
                         Toast.LENGTH_SHORT
                     )
                     mytoast.setGravity(Gravity.TOP or Gravity.RIGHT, 0, -100)
                     mytoast.show()*/


                }

                if (item == iconGargabe) {
                    splash_progress.visibility = View.VISIBLE
                    viewModelwithFilter("contenedor")
                    view?.performLongClick()

                }
                if (item == iconFault) {
                    splash_progress.visibility = View.VISIBLE
                    viewModelwithFilter("falta")


                }
                for (it in listIcons) {
                    if (it != item) {
                        it.setTint(getColor(context!!, verdnatura_white))
                    }
                }
                item.setTint(getColor(context!!, verdnatura_pumpkin_orange))
                view?.performLongClick()
            }

        })
        toolbar_icons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    private fun viewModelwithFilter(keyfilter: String) {
        filter = keyfilter
        viewModel.itemDiary(user, password, itemFk, warehouseFk,"itemDiary")
    }

    private fun setEvents() {
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        filter_nameordateitem.setOnKeyListener { v, keyCode, event ->
            // //Log.i("VERDNATURA:","El key a filtrar ${filter_nameordateitem.text.toString()}")
            if (filter_nameordateitem.text.toString().isNullOrEmpty()) {
                if (listHistoric.size != listHistoryAux.size) {
                    listHistoric.removeAll(listHistoryAux)
                    listHistoryAux.forEach {
                        listHistoric.add(it)
                    }
                    adapter!!.notifyDataSetChanged()
                }

            } else {
                listHistoric.removeAll(listHistoryAux)
                listHistoryAux.forEach {


                    if (it.name != null && (it.name.contains(
                            filter_nameordateitem.text.toString(),
                            true
                        ))
                    ) {
                        //Log.i("VERDNATURA:","si contiene ${filter_nameordateitem.text.toString()}")
                        listHistoric.add(it)

                    }
                }

            }


            adapter!!.notifyDataSetChanged()
            return@setOnKeyListener false
        }

    }

    override fun observeViewModel() {
        with(viewModel) {
            loadHistoricoList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {

                    splash_progress.visibility = View.GONE
                    listToAdapter(it.list, filter)
                    //  //Log.i("VERDNATURA:","el total original es ${it.list.size}")
                    /*  adapter = HistoricoAdapter(it.list)
                      historico_recyclerview.adapter = adapter
                      historico_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)*/
                    navigateToToday(it)

                }
            })
        }
    }


    private fun listToAdapter(lista: List<ItemHistoricoVO>, filter: String) {

        listHistoric.clear()
        listHistoryAux.clear()
        lista.forEach {

            if (filter.equals("all")) {
                listHistoric.add(it)
                //  listHistoryAux.add(it)
            } else {

                if ((it.name != null) && (it.name.contains(filter, true))) {
                    listHistoric.add(it)
                    //   listHistoryAux.add(it)
                }
            }

        }

        adapter = HistoricoAdapter(listHistoric)
        historico_recyclerview.adapter = adapter
        historico_recyclerview.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

        listHistoryAux.addAll(listHistoric)

    }

    private fun navigateToToday(it: ItemHistoricoListVO) {
        var positionToday = 0
        val sdf = SimpleDateFormat("yyyy-MM-dd")
        val currentDate = sdf.format(Date())

        for (item in it.list) {
            if (currentDate == item.shipped) {
                break
            }
            positionToday += 1
        }

        historico_recyclerview.scrollToPosition(positionToday)
    }

}