package es.verdnatura.presentation.view.feature.historicoshelvingparking.fragment


import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.itemShelvingLogUSerCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.diadeventa.model.ItemShelvingSaleDateList
import es.verdnatura.presentation.view.feature.historicoshelving.model.ItemShelvingLogList
import es.verdnatura.presentation.view.feature.historicoshelving.model.ItemShelvingLog
import es.verdnatura.presentation.view.feature.historicoshelvingparking.model.ParkingShelvingLog
import es.verdnatura.presentation.view.feature.historicoshelvingparking.model.ParkingShelvingLogList
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import timber.log.Timber.i


class ParkingShelvingLogViewModel(context: Context) : BaseViewModel() {
    private val itemShelvingLogUSerCase: itemShelvingLogUSerCase =
        itemShelvingLogUSerCase(context)


    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


  /*  private val _itemShelvingSaleDateList by lazy { MutableLiveData<ItemShelvingSaleDateList>() }
    val itemShelvingSaleDateList: LiveData<ItemShelvingSaleDateList>
        get() = _itemShelvingSaleDateList*/


    private val _parkingShelvingLogList by lazy { MutableLiveData<ParkingShelvingLogList>() }
    val parkingShelvingLogList: LiveData<ParkingShelvingLogList>
        get() = _parkingShelvingLogList


    val loadParkingShelvingLogList = Transformations.map(_parkingShelvingLogList) { Event(it) }



    fun parkingShelvingLog_get(usuario:String,password:String,vShelvingFK: String,callFunction:String){
        itemShelvingLogUSerCase.parkingShelvingLog_get(usuario,password,vShelvingFK).enqueue(object :
            Callback<List<ParkingShelvingLog>> {
            override fun onFailure(call: Call<List<ParkingShelvingLog>>, t: Throwable) {
                val listError:ArrayList<ParkingShelvingLog> = ArrayList()
                listError.add(ParkingShelvingLog(isError = true,errorMessage = getMessageFromAllResponse(callFunction,t.message!!)))
                _parkingShelvingLogList.value = ParkingShelvingLogList(listError)
            }

            override fun onResponse(
                call: Call<List<ParkingShelvingLog>>,
                response: Response<List<ParkingShelvingLog>>
                                   ) {

                if (!response.isSuccessful){
                    val listError:ArrayList<ParkingShelvingLog> = ArrayList()
                    listError.add(ParkingShelvingLog(isError = true,errorMessage = getMessageFromAllResponse(callFunction,response.message())))
                    _parkingShelvingLogList.value = ParkingShelvingLogList(listError)
                }else{

                    if (response.body() != null)
                    {
                        i("crea la lista " + response.body())
                        _parkingShelvingLogList.value = response.body()?.let {
                            ParkingShelvingLogList(it)
                        }


                        i("finalizada la lista ${response.body()}")

                    }

                }

            }

        })
    }



}








