package es.verdnatura.presentation.view.feature.inventario.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.View
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentInventaryBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnInvetoryNichoClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.feature.calidad.model.BuyerVO
import es.verdnatura.presentation.view.feature.inventario.adapter.InventoryAdapter
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.inventario.model.ItemInventaryVO
import ir.mirrajabi.searchdialog.SimpleSearchDialogCompat
import ir.mirrajabi.searchdialog.core.SearchResultListener
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.buyers_fragment.*
import kotlinx.android.synthetic.main.fragment_ajustes.*
import kotlinx.android.synthetic.main.fragment_buscar_item.location_recyclerview
import kotlinx.android.synthetic.main.fragment_buscar_item.splash_progress
import kotlinx.android.synthetic.main.fragment_inventary.*
import kotlinx.android.synthetic.main.fragment_inventary.filter_itemFk
import kotlinx.android.synthetic.main.toolbar.*

class InventaryFragment : BaseFragment<FragmentInventaryBinding,InventaryViewModel>(InventaryViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : InventoryAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private var listInvetory:ArrayList<ItemInventaryVO> = ArrayList()
    private var listInvetoryAux:ArrayList<ItemInventaryVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var reload = false
    private var hideLoad = true
    private var layoutManager : LinearLayoutManager? = null
    private var firstVisiblePosition = 0
    private var secondVisiblePosition = 1


    //sergio : cmbios para insertar lista compradores en inventary en el searchable
    private val items=ArrayList<SearchBuyerModel>()
    private var prefs: SharedPreferences? = null
    private lateinit var buyerId:String


    companion object {
        fun newInstance() = InventaryFragment()
    }


    //override fun getLayoutId(): Int = R.layout.fragment_inventary
    override fun getLayoutId(): Int = R.layout.fragment_inventary

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        prefs = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs!!.getString(USER,"").toString()
        password = prefs!!.getString(PASSWORD,"").toString()
        sectorFk = prefs!!.getInt(SECTORFK,1).toString()
        warehouseFk = prefs!!.getInt(WAREHOUSEFK,1).toString()

       //sergio. se quita para subir version.A la espera de como será el inventario
        viewModel.itemShelvingBuyerGet(user = user,password = password,"itemShelvingBuyerGet")
        super.onCreate(savedInstanceState)
    }


    override fun init() {

        filter_buyer.setText(prefs!!.getString(BUYER,""))
        splash_progress.visibility = View.VISIBLE
        buyerId=prefs!!.getString(BUYERID,"").toString()
        ////Log.i("VERDNATURA:","el sector es $warehouseFk")
        ////Log.i("VERDNATURA:","el sbuyerid es $buyerId")
        viewModel.getInventory(user,password,buyerId,warehouseFk,"itemShelving_filterBuyer")
        filter_buyer.setOnClickListener{
            //SimpleSearchDialogCompat(context,getString(R.string.compradores),getString(R.string.escribirparteNombre),null,initData(),
            SimpleSearchDialogCompat(context,getString(R.string.compradores),getString(R.string.escribirparteNombre),null,items,
            SearchResultListener { baseSearchDialogCompat, nombre, position ->
                //////Log.i("VERDNATURA:","nombre "+nombre.title)
                //////Log.i("VERDNATURA:","el id es "+nombre.getBuyerId())
                filter_buyer.setText((nombre.title))
                saveBuyer(nombre.getBuyerNickname())
                saveBuyerId(nombre.getBuyerId())
                buyerId=nombre.getBuyerId()
                viewModel.getInventory(user,password,nombre.getBuyerId(),warehouseFk,"itemShelving_filterBuyer")
                splash_progress.visibility = View.VISIBLE
                baseSearchDialogCompat.dismiss()
            }).show()
        }


        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        splash_progress.visibility = View.VISIBLE
        toolbar_title.text = getString(R.string.itemShelvingRadar)
        setToolBar()
        setEvents()

        if (reload){
            reload = false
            splash_progress.visibility = View.VISIBLE
            hideLoad = false
            viewModel.getInventory(user,password,buyerId,warehouseFk,"itemShelving_filterBuyer")
        }


        super.init()
    }



    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())

        listIcons.add(iconReload)

        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener{
            override fun onOptionsItemSelected(item: Drawable) {
               if (item == iconReload){
                   splash_progress.visibility = View.VISIBLE
                   firstVisiblePosition = layoutManager?.findFirstVisibleItemPosition() ?: 0
                   // sergio.
                    viewModel.getInventory(user,password,buyerId,warehouseFk,"itemShelving_filterBuyer")
                  //viewModel.getInventory(user,password,prefs!!.getInt(SECTORFK,1).toString())
               }

            }

        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        filter_itemFk.setOnKeyListener { v, keyCode, event ->
            if (filter_itemFk.text.toString().isNullOrEmpty()){
                if (listInvetory.size != listInvetoryAux.size){
                    listInvetory.removeAll(listInvetoryAux)
                    listInvetoryAux.forEach {
                        listInvetory.add(it)
                    }

                }

            }else{
                listInvetory.removeAll(listInvetoryAux)
                listInvetoryAux.forEach {
                    if ((it.itemFk.contains(filter_itemFk.text.toString(),true)) || (it.producer.contains(filter_itemFk.text.toString(),true))){
                        listInvetory.add(it)
                    }
                }


            }
            adapter!!.notifyDataSetChanged()
            return@setOnKeyListener false
        }
    }


    override fun onPause() {
        super.onPause()
        firstVisiblePosition = layoutManager?.findFirstVisibleItemPosition() ?: 0
    }

    override fun onResume() {
        super.onResume()
        if (location_recyclerview.layoutManager  != null){
            if (firstVisiblePosition <= listInvetory.size){
                location_recyclerview.layoutManager!!.scrollToPosition(firstVisiblePosition)
            }
        }
    }

    override fun observeViewModel() {
        with(viewModel){

            //sergio: para compradores
            buyersList.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                createBuyerList(it.list)
            })


            inventaryList.observe(viewLifecycleOwner, Observer {
                    if (hideLoad){
                        splash_progress.visibility = View.GONE
                    }else{
                        hideLoad = true
                    }

                    listInvetory = ArrayList()
                    listInvetoryAux = ArrayList()
                    it.list.forEach {
                        if (it.nicho != "0"){
                            listInvetory.add(it)
                            listInvetoryAux.add(it)
                        }
                    }

                    adapter = InventoryAdapter(listInvetory,object: OnInvetoryNichoClickListener{
                        override fun onInvetoryNichoClickListener(item: ItemInventaryVO) {

                            //sergio: cambiado customdialogInput pòr customdialog 6.1.4
                            customDialog.setTitle(item.itemFk+"\n"+item.longName+" "+item.size).setDescription("Cantidad real("+item.nicho+")").setOkButton("Tirar"){
                            //customDialogInput.setTitle(item.itemFk+"\n"+item.longName+" "+item.size).setDescription("Cantidad real("+item.nicho+")").setOkButton("Tirar"){
                                viewModel.itemTrash(item.itemFk,warehouseFk,user,password,item.nicho,"0","itemTrash")
                                //viewModel.itemStockUpdate(item.itemFk,warehouseFk,user,password,"0","0")
                                // viewModel.itemStockUpdate(item.itemFk,warehouseFk,user,password,customDialogInput.getValue(),"0")
                                 changeOfflineValue(item)
                                // customDialogInput.dismiss()
                                 customDialog.dismiss()
                            }.setKoButton("Cancelar"){
                                // customDialogInput.dismiss()
                              customDialog.dismiss()
                            }.show()
                       // }.setValue("0").show()
                        //sergio: ya no recoge focus porque no tiene cuadro para insertar número de artículo
                          /* customDialogInput.getEditText().requestFocus()
                            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                                    if (!customDialogInput.getValue().isNullOrEmpty()){
                                        ////Log.i("VERDNATURA:","A tirar tanto viewmodel fuera")
                                        viewModel.itemStockUpdate(item.itemFk,warehouseFk,user,password,customDialogInput.getValue(),"0")
                                        changeOfflineValue(item)
                                    }
                                    customDialogInput.setValue("")
                                    customDialogInput.dismiss()
                                    (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                    return@setOnEditorActionListener true
                                }
                                false
                            }*/

                        }
                    },onPasillerosItemClickListener = pasillerosItemClickListener!!)
                    location_recyclerview.adapter = adapter
                    layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
                    location_recyclerview.layoutManager = layoutManager

                    if (firstVisiblePosition <= listInvetory.size){
                        location_recyclerview.layoutManager!!.scrollToPosition(firstVisiblePosition)
                    }


            })


            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }
            })
        }
    }

    private fun changeOfflineValue(item:ItemInventaryVO){
        var i = 0
        var position = 0
        listInvetory.forEach {
            if (it.itemFk == item.itemFk) {
                position = i
            }
            i = i.plus(1)
        }
        listInvetory.removeAt(position)

        i = 0
        position = 0
        listInvetoryAux.forEach {
            if (it.itemFk == item.itemFk) {
                position = i
            }
            i = i.plus(1)
        }
        listInvetoryAux.removeAt(position)
        adapter?.notifyDataSetChanged()

        reload = true

    }


    //sergio:añadido para compradores
    private fun createBuyerList(list: List<BuyerVO>){
     items.clear()
        list.forEach { buyer ->
            if (!buyer.isError){
                try {
                    items.add(SearchBuyerModel(buyer.nickname,buyer.userFk));
                    ////Log.i("VERDNATURA:","Nombre comprador ${buyer.nickname} : ${buyer.userFk}")
                }catch (e:Exception){}
            }
        }}


}


