package es.verdnatura.presentation.view.feature.inventario.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetInventaryUserCase
import es.verdnatura.domain.GetItemCardUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageError
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.calidad.model.BuyerListVO
import es.verdnatura.presentation.view.feature.calidad.model.BuyerVO
import es.verdnatura.presentation.view.feature.inventario.model.InventaryListVO
import es.verdnatura.presentation.view.feature.inventario.model.ItemInventaryVO
import es.verdnatura.presentation.view.feature.login.model.errorMessage
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class InventaryViewModel(context: Context) : BaseViewModel() {


    private val getInventaryUserCase: GetInventaryUserCase = GetInventaryUserCase(context)
    private val getItemCardUserCase: GetItemCardUserCase = GetItemCardUserCase(context)

    private val _inventaryList by lazy { MutableLiveData<InventaryListVO>() }
    val inventaryList: LiveData<InventaryListVO>
        get() = _inventaryList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    private val _buyersList by lazy { MutableLiveData<BuyerListVO>() }
    val buyersList: LiveData<BuyerListVO>
        get() = _buyersList


    //sergio para obtener comprador
    fun itemShelvingBuyerGet(user:String,password:String,callFunction:String){
        getInventaryUserCase.itemShelvingBuyerGet(user,password).enqueue(object :
            Callback<List<BuyerVO>> {
            override fun onFailure(call: Call<List<BuyerVO>>, t: Throwable) {
                val listError:ArrayList<BuyerVO> = ArrayList()
                listError.add(BuyerVO(isError = true,errorMessage =getMessageFromAllResponse(callFunction,t.message!!)))
                _buyersList.value = BuyerListVO(listError)
            }

            override fun onResponse(
                call: Call<List<BuyerVO>>,
                response: Response<List<BuyerVO>>
            ) {
                if (response.body() != null){
                    _buyersList.value = response.body()?.let { BuyerListVO(it) }
                }else{
                    val listError:ArrayList<BuyerVO> = ArrayList()
                    listError.add(BuyerVO(isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())))
                    _buyersList.value = BuyerListVO(listError)
                }
            }

        })
    }


    val loadInventaryList = Transformations.map(_inventaryList) { Event(it) }

//sergio: borrar. En 6.1.3 ya no se utiliza
    /*fun getInventory(user:String,password:String,buyerFk:String){

        getInventaryUserCase.itemShelvingRadar(user,password,buyerFk).enqueue(object :
            Callback<List<ItemInventaryVO>> {
            override fun onFailure(call: Call<List<ItemInventaryVO>>, t: Throwable) {
                val listError:ArrayList<ItemInventaryVO> = ArrayList()
                listError.add(ItemInventaryVO(isError = true,errorMessage = t.message!!))
                _inventaryList.value = InventaryListVO(listError)
            }

            override fun onResponse(
                call: Call<List<ItemInventaryVO>>,
                response: Response<List<ItemInventaryVO>>
            ) {
                if (response.body() != null){
                    _inventaryList.value = response.body()?.let { InventaryListVO(it) }
                }else{
                    val listError:ArrayList<ItemInventaryVO> = ArrayList()
                    listError.add(ItemInventaryVO(isError = true,errorMessage = "Error en la llamada de itemShelvingRadar"))
                    _inventaryList.value = InventaryListVO(listError)

                }
            }

        })
    }*/

    fun getInventory(user:String,password:String,buyerFk:String,warehouseFk: String,callFunction:String){
       // getInventaryUserCase.itemShelvingRadar(user,password,buyerFk).enqueue(object :
        getInventaryUserCase.itemShelvingRadarFilterBuyer(user,password,buyerFk,warehouseFk).enqueue(object :
            Callback<List<ItemInventaryVO>> {
            override fun onFailure(call: Call<List<ItemInventaryVO>>, t: Throwable) {
                val listError:ArrayList<ItemInventaryVO> = ArrayList()
                listError.add(ItemInventaryVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,t.message!!)))
                _inventaryList.value = InventaryListVO(listError)
            }

            override fun onResponse(
                call: Call<List<ItemInventaryVO>>,
                response: Response<List<ItemInventaryVO>>
            ) {
                if (response.body() != null){
                    _inventaryList.value = response.body()?.let { InventaryListVO(it) }
                }else{
                    val listError:ArrayList<ItemInventaryVO> = ArrayList()
                    listError.add(ItemInventaryVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,response.message())))
                    _inventaryList.value = InventaryListVO(listError)

                }
            }

        })
    }



    fun itemStockUpdate(itemFk:String,warehouseFk:String,user:String,password:String,newValue:String,isTrash:String){
       //sergio: se modifica por CAU a itemTrash
        // https://cau.verdnatura.es/scp/tickets.php?id=21043 al nuevo procedimiento itemTrash
         getItemCardUserCase.itemStockUpdate(user,password,itemFk,warehouseFk,newValue,isTrash).enqueue(object : Callback<String>{
            //getItemCardUserCase.itemTrash(user,password,itemFk,warehouseFk,newValue,isTrash).enqueue(object : Callback<String>{

                override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar STOCK "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemStockUpdate")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    //sergio: se modifica por CAU a itemTrash
    fun itemTrash(itemFk:String,warehouseFk:String,user:String,password:String,newValue:String,isTrash:String,callFunction:String){


        getItemCardUserCase.itemTrash(user,password,itemFk,warehouseFk,newValue,isTrash).enqueue(object : Callback<String>{


            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,t.message!!))
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }


}
