package es.verdnatura.presentation.view.feature.login.fragment

import android.content.Context
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.Gson
import com.google.gson.JsonObject
import com.google.gson.reflect.TypeToken
import es.verdnatura.domain.GetLoginUserCase
import es.verdnatura.domain.NodeJsService
import es.verdnatura.presentation.base.*
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.login.model.*
import es.verdnatura.presentation.view.feature.paletizador.model.itemsExpeditionDynamics
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class LoginViewModel(context: Context) : BaseViewModel()
{

    private val getLoginUserCase: GetLoginUserCase = GetLoginUserCase(context)

    private val _loginitem by lazy { MutableLiveData<LoginItemVO>() }
    val loginitem: LiveData<LoginItemVO>
        get() = _loginitem

    private val _loginsalixitem by lazy { MutableLiveData<LoginSalixVO>() }
    val loginsalixitem: LiveData<LoginSalixVO>
        get() = _loginsalixitem

    private val _version by lazy { MutableLiveData<Boolean>() }
    val version: LiveData<Boolean>
        get() = _version

    private val _devicelogresponse by lazy { MutableLiveData<ResponseItemVO>() }
    val devicelogresponse: LiveData<ResponseItemVO>
        get() = _devicelogresponse

    //sergio: nueva version para actualizar
    private val _versionappitem by lazy { MutableLiveData<versionApp>() }
    val versionappitem: LiveData<versionApp>
        get() = _versionappitem

    private val _logindevice by lazy { MutableLiveData<LoginDevice>() }
    val logindevice: LiveData<LoginDevice>
        get() = _logindevice

    private val _workerId by lazy { MutableLiveData<workerId>() }
    val workerId: LiveData<workerId>
        get() = _workerId

    private val _nodejs by lazy { MutableLiveData<NodeJsService.SmartTagNode>() }
    val nodejs: LiveData<NodeJsService.SmartTagNode>
        get() = _nodejs


    //sergio:anterior login
    fun login(user: String, password: String, androiid: String)
    {
        getLoginUserCase.login(user, password, androiid).enqueue(object : Callback<String>
        {
            override fun onResponse(call: Call<String>, response: Response<String>)
            {

                var loginItemVO: LoginItemVO?
                if (response.raw().code() == 555)
                {
                    var message = response.raw().message().toString()

                    if (message.contains("autorizado"))
                    {
                        loginItemVO = LoginItemVO(
                            id = "0",
                            isError = true,
                            errorMessage = "Usuario no autorizado para este dispositivo"
                                                 )
                    } else
                    {
                        loginItemVO = LoginItemVO(
                            id = "0",
                            isError = true,
                            errorMessage = "Usuario o contraseña incorrectos"
                                                 )
                    }

                } else if (!response.body().isNullOrEmpty())
                {
                    loginItemVO = response.body()?.let { LoginItemVO(id = it) }
                } else
                {
                    loginItemVO =
                        LoginItemVO(id = "0", isError = true, errorMessage = "Error desconocido")
                }

                _loginitem.value = loginItemVO!!
            }

            override fun onFailure(call: Call<String>, t: Throwable)
            {
                val loginItemVO: LoginItemVO? =
                    LoginItemVO(id = "0", isError = true, errorMessage = t.message!!)
                _loginitem.value = loginItemVO
            }
        })
    }

    fun loginSalix(user: String, password: String,callFunction:String)
    {
        getLoginUserCase.salixLogin(user, password).enqueue(object : Callback<LoginSalixVO>
        {
            override fun onResponse(call: Call<LoginSalixVO>, response: Response<LoginSalixVO>)
            {

                var loginSalixVO: LoginSalixVO?
                if (response.body() != null)
                {
                    loginSalixVO = response.body()?.token?.let {
                        LoginSalixVO(
                            user,
                            password,
                            it,
                            isError = false,
                            errorMessage = ""
                                    )
                    }
                } else
                {
                    loginSalixVO = LoginSalixVO(
                        user,
                        password,
                        isError = true,
                        errorMessage =  getMessageFromAllResponse(callFunction,response.message())
                                               )
                }

                _loginsalixitem.value = loginSalixVO
            }

            override fun onFailure(call: Call<LoginSalixVO>, t: Throwable)
            {
                val loginSalixVO =
                    LoginSalixVO(
                        user,
                        password,
                        "",
                        isError = true,
                        errorMessage =  getMessageFromAllResponse(callFunction,t.message!!)
                                )
                _loginsalixitem.value = loginSalixVO
            }
        })
    }

    fun device_checkLogin(user: String, password: String, android_id: String,callFunction:String)
    {
        //var name=nameofFunction(object{})

        getLoginUserCase.device_checkLogin(user, password, android_id)
            .enqueue(object : Callback<LoginDevice>
            {
                override fun onResponse(call: Call<LoginDevice>, response: Response<LoginDevice>)
                {

                    var loginDevice: LoginDevice?

                /*    Log.d("VERDNATIRA",response.message())
                    Log.d("VERDNATIRA",response.errorBody().toString())
                    Log.d("VERDNATIRA",response.toString())
                    Log.d("VERDNATIRA",response.raw().toString())*/

                    if (!response.isSuccessful)
                    {
                            loginDevice = LoginDevice(
                                isError = true,
                                errorMessage =  getMessageFromAllResponse(callFunction,response.message()))


                    } else
                    {
                        if (response.body() != null)
                        {

                            loginDevice = response.body()?.let {

                                LoginDevice(
                                    it.vMessage, it.vIsAuthorized,
                                    isError = false,
                                    errorMessage = ""
                                           )

                            }

                        } else
                        {
                            loginDevice = LoginDevice(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                     )
                        }
                    }
                    _logindevice.value = loginDevice
                }

                override fun onFailure(call: Call<LoginDevice>, t: Throwable)
                {
                    val loginDevice =
                        LoginDevice(
                            "",
                            "",
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(callFunction,t.message!!)
                                   )
                    _logindevice.value = loginDevice
                }
            })
    }



    fun worker_getId(user: String, password: String,callFunction: String)
    {
        getLoginUserCase.worker_getId(user, password)
            .enqueue(object : Callback<String>
            {
                override fun onResponse(call: Call<String>, response: Response<String>)
                {

                    var workerId: workerId?


                    if (response.body() != null)
                    {

                        workerId = workerId(
                            response.body().toString(),
                            isError = false,
                            errorMessage = ""
                                           )

                    } else
                    {
                        workerId = workerId(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                           )
                    }

                    _workerId.value = workerId
                }

                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    val workerId =
                        workerId(
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(callFunction,t.message!!)
                                )
                    _workerId.value = workerId
                }

            })
    }

    fun deviceLog_add(
        user: String, password: String, app: String, versionApp: String, android_id: String,callFunction: String
                     )
    {
        getLoginUserCase.deviceLog_add(user, password, app, versionApp, android_id)
            .enqueue(object : Callback<Void>
            {
                override fun onResponse(call: Call<Void>, response: Response<Void>)
                {

                    if (!response.isSuccessful)
                    {

                        messageError(callFunction,response.message())

                    }
                }

                override fun onFailure(call: Call<Void>, t: Throwable)
                {

                    messageError(callFunction,t.message!!)
                }

            })
    }

    fun messageError(callFunction:String,message: String)
    {
        _devicelogresponse.value = ResponseItemVO(
            isError = true,
            errorMessage =  getMessageFromAllResponse(callFunction,message)
                                                 )

    }

    /*fun checkVersion_old(user:String,password:String,version:String){
         getLoginUserCase.checkVersion(user,password,version).enqueue(object : Callback<Boolean>{
             override fun onResponse(call: Call<Boolean>, response: Response<Boolean>) {
                 if (response.body() != null){
                     _version.value = response.body()
                 }else{
                     _version.value = true
                 }
             }

             override fun onFailure(call: Call<Boolean>, t: Throwable) {
                 _version.value = true
             }
         })
     }*/
    //sergio:checkVersion new
    fun checkVersion(user: String, password: String, nameApp: String,callFunction: String)
    {
        getLoginUserCase.checkVersion(user, password, nameApp)
            .enqueue(object : Callback<versionApp>
            {
                override fun onResponse(call: Call<versionApp>, response: Response<versionApp>)
                {

                    var versionApp: versionApp?

                    if (response.body() != null)
                    {

                        versionApp = response.body()?.let {

                            versionApp(
                                it.appname,
                                it.version,
                                it.IsVersionCritical,
                                isError = false,
                                errorMessage = ""
                                      )

                        }

                    } else
                    {
                        versionApp = versionApp(
                            user,
                            password,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                               )
                    }
                    _versionappitem.value = versionApp

                }

                override fun onFailure(call: Call<versionApp>, t: Throwable)
                {
                    val versionApp =
                        versionApp(
                            nameApp,
                            "",
                            isError = true,
                            errorMessage =getMessageFromAllResponse(callFunction, t.message!!)
                                  )
                    _versionappitem.value = versionApp
                }
            })
    }

    fun nodeJsTest(smarttag: String, wagon: String, shelving: String)
    {
        getLoginUserCase.nodeJsTest(smarttag, wagon, shelving)
            .enqueue(object : Callback<NodeJsService.SmartTagNode>
            {
                override fun onResponse(
                    call: Call<NodeJsService.SmartTagNode>,
                    response: Response<NodeJsService.SmartTagNode>
                                       )
                {

                    var workerId: NodeJsService.SmartTagNode?

                    /*   if (response.isSuccessful()){
                           Log.i("VERDNATURA::","OK")
                       }

                       if (!response.errorBody().toString().isEmpty()){

                       }*/

                    if (response.body() != null)
                    {
                        Log.i("VERDNATURA::", "el valor nodejs no es nulo ")
                        workerId = NodeJsService.SmartTagNode(
                            response.body().toString(),
                            isError = false,
                            errorMessage = ""
                                                             )

                    } else
                    {
                        workerId = NodeJsService.SmartTagNode(
                            isError = true,
                            errorMessage = "Error al llamar al servicio Node."
                                                             )
                        Log.i("VERDNATURA::", "el valor nodejs es NULL")
                    }

                    _nodejs.value = workerId
                }

                override fun onFailure(call: Call<NodeJsService.SmartTagNode>, t: Throwable)
                {
                    val workerId =
                        NodeJsService.SmartTagNode(isError = true, errorMessage = t.message!!)

                    //  Log.i("VERDNATURA::","el error nodejs es "+t.message+"--"+t.cause.toString()+"--"+t.stackTrace.toString())

                    _nodejs.value = workerId
                }

            })
    }

    fun getListFromJSON(json: JsonObject): MutableList<itemsExpeditionDynamics>
    {
        val gson = Gson()
        var list = mutableListOf<itemsExpeditionDynamics>()

        //  val mapType = object : TypeToken<Map<String, Any>>() {}.type

        var expeditionState: Map<String, Any> =
            gson.fromJson(json, object : TypeToken<Map<String, Any>>()
            {}.type)
        expeditionState.forEach {

            list.add(itemsExpeditionDynamics(key = it.key, value = it.value.toString()))

        }
        return list
    }

}
