package es.verdnatura.presentation.view.feature.main.activity

import android.content.SharedPreferences
import android.view.Menu
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import es.verdnatura.BuildConfig
import es.verdnatura.R
import es.verdnatura.databinding.ActivityMainBinding
import es.verdnatura.domain.ConstAndValues.MAINACTIVITY
import es.verdnatura.presentation.base.BaseActivity
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.fragment.AjustesFragment
import es.verdnatura.presentation.view.feature.articulo.fragment.ItemCardFragment
import es.verdnatura.presentation.view.feature.articulo.fragment.ItemProposalFragment
import es.verdnatura.presentation.view.feature.buffer.fragment.BufferFragment
import es.verdnatura.presentation.view.feature.buscaritem.fragment.BuscarItemFragment
import es.verdnatura.presentation.view.feature.calidad.fragment.BuyersFragment
import es.verdnatura.presentation.view.feature.calidad.fragment.QaualityFragment
import es.verdnatura.presentation.view.feature.collection.fragment.CollectionFragment
import es.verdnatura.presentation.view.feature.controlador.fragment.ControladorFragment
import es.verdnatura.presentation.view.feature.controlador.fragment.WebFragment
import es.verdnatura.presentation.view.feature.controlvehiculo.fragment.ControlVehiculoFragment
import es.verdnatura.presentation.view.feature.controlvehiculo.fragment.ControlVehiculoUsuarioFragment
import es.verdnatura.presentation.view.feature.diadeventa.fragment.DayOfSaleFragment
import es.verdnatura.presentation.view.feature.faltas.fragment.FaltasFragment
import es.verdnatura.presentation.view.feature.historico.fragment.HistoricoFragment
import es.verdnatura.presentation.view.feature.historicoshelving.fragment.itemShelvingLogFragment
import es.verdnatura.presentation.view.feature.historicovehiculo.fragment.HistoricoVehiculoFragment
import es.verdnatura.presentation.view.feature.inventario.fragment.InventaryFragment
import es.verdnatura.presentation.view.feature.main.model.ItemMenuVO
import es.verdnatura.presentation.view.feature.paletizador.fragment.*
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletVO
import es.verdnatura.presentation.view.feature.parking.fragment.ParkingFragment
import es.verdnatura.presentation.view.feature.pasillero.fragment.PasilleroFragment
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.precontrol.PreControladorFragment
import es.verdnatura.presentation.view.feature.presacador.fragment.PreSacadorFragment
import es.verdnatura.presentation.view.feature.qr.QrFragment
import es.verdnatura.presentation.view.feature.reposicion.fragment.ReposicionFragment
import es.verdnatura.presentation.view.feature.sacador.fragment.SacadorFragment
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import es.verdnatura.presentation.view.feature.shelvingparking.fragment.ShelvingParkingFragment
import es.verdnatura.presentation.view.feature.smarttag.register.RegisterSmartTagsFragment
import es.verdnatura.presentation.view.feature.smarttag.sacador.fragment.AssociateSmartTagsFragment
import es.verdnatura.presentation.view.feature.ubicador.fragment.AutomaticAddItemFragment
import es.verdnatura.presentation.view.feature.ubicador.fragment.UbicadorFragment
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import timber.log.Timber


class MainActivity : BaseActivity<ActivityMainBinding>(), OnPasillerosItemClickListener,
    OnTruckClickListener, OnPalletClickListener, OnComprobarPalletViewClickListener,
    OnCollectionSelectedListener, OnBuyerSelectedListener {

    private var lastBottomMenuItemSelected: ItemMenuVO? = null
    private lateinit var customDialog: CustomDialog
    var firstItem: ItemMenuVO? = null
    var fm = supportFragmentManager
    var TAG = "MainActivity"

    override fun getLayoutId(): Int = R.layout.activity_main

    override fun init() {
        customDialog = CustomDialog(this)
        setBottomMenu()
        if (haveSector()) addFragment(
            PasilleroFragment.newInstance("Verdnatura"),
            R.id.main_frame_layout,
            PasilleroFragment.TAG,
            false
        )
        else addFragment(
            AjustesFragment.newInstance(),
            R.id.main_frame_layout,
            AjustesFragment.TAG,
            false
        )

        // sergio: de esta manera se eliminan los logs de lanzamiento
        if (BuildConfig.DEBUG) {
            Timber.plant(Timber.DebugTree())
        }



    }

    private fun haveSector(): Boolean {
        val prefs: SharedPreferences = getSharedPreferences("es.verdnatura.user.prefs", 0)
        val sectorFk = prefs.getInt("sectorFk", -1)
        return sectorFk != -1
    }

    private fun setBottomMenu() {
        val bottomMenu = main_bottom_navigation
        bottomMenu.itemIconTintList = null
        GlobalScope.launch(Dispatchers.Main) {
            val bottomMenuItems: ArrayList<ItemMenuVO> = ArrayList()

            bottomMenuItems.add(
                ItemMenuVO(
                    0,
                    title = getString(R.string.Pasilleros),
                    defaultImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_device_hub_black_24dp
                    ),
                    selectedImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_device_hub_black_24dp_selected
                    )
                )
            )

            bottomMenuItems.add(
                ItemMenuVO(
                    1,
                    title = getString(R.string.Sacadores),
                    defaultImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_local_florist_black_24dp
                    ),
                    selectedImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_local_florist_black_24dp_selected
                    )
                )
            )

            bottomMenuItems.add(
                ItemMenuVO(
                    2,
                    title = getString(R.string.Controladores),
                    defaultImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_verified_user_black_24dp
                    ),
                    selectedImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_verified_user_black_24dp_selected
                    )
                )
            )

            bottomMenuItems.add(
                ItemMenuVO(
                    3,
                    title = getString(R.string.Paletizadores),
                    defaultImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.pallet_shipping_black
                    ),
                    selectedImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.pallet_shipping_selected
                    )
                )
            )


            bottomMenuItems.add(
                ItemMenuVO(
                    4,
                    title = getString(R.string.Ajustes),
                    defaultImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_build_black_24dp
                    ),
                    selectedImage = ContextCompat.getDrawable(
                        applicationContext, R.drawable.ic_build_black_24dp_selected
                    )
                )
            )

            // Add al items to menu view
            bottomMenuItems.forEach {
                bottomMenu.menu.add(Menu.NONE, it.id, Menu.NONE, it.title).icon = it.defaultImage
            }


            if (haveSector()) {
                // Select first item by default
                firstItem = bottomMenuItems.first()
                bottomMenu.selectedItemId = firstItem!!.id
                bottomMenu.menu.findItem(firstItem!!.id).icon = firstItem!!.selectedImage
                lastBottomMenuItemSelected = firstItem
            } else {
                // Select last item by default
                val lastItem = bottomMenuItems.last()
                bottomMenu.selectedItemId = lastItem.id
                bottomMenu.menu.findItem(lastItem.id).icon = lastItem.selectedImage
                lastBottomMenuItemSelected = lastItem
            }


            bottomMenu.setOnNavigationItemSelectedListener { menuItem ->
                // Get selected item from itemList and apply ui changes to menu view
                val selectedItemMenu = bottomMenuItems.find { menuItem.itemId == it.id }
                menuItem.icon = selectedItemMenu?.selectedImage

                // Get last selected item and disable it
                bottomMenu.menu.findItem(lastBottomMenuItemSelected?.id!!).icon =
                    lastBottomMenuItemSelected?.defaultImage
                lastBottomMenuItemSelected = selectedItemMenu


                when (selectedItemMenu?.title) {
                    getString(R.string.Pasilleros) -> {
                        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        delete_Fragments()
                        addFragment(
                            PasilleroFragment.newInstance("Verdnatura"),
                            R.id.main_frame_layout,
                            PasilleroFragment.TAG,
                            false
                        )
                        //true
                    }
                    getString(R.string.Sacadores) -> {
                        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        delete_Fragments()
                        // addFragment(SacadorFragment.newInstance(),R.id.main_frame_layout, SacadorFragment.TAG,false)
                        addFragment(
                            ControlVehiculoFragment.newInstance(MAINACTIVITY),
                            R.id.main_frame_layout,
                            MAINACTIVITY.TAG,
                            false
                        )
                        //true
                    }
                    getString(R.string.Controladores) -> {
                        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        delete_Fragments()
                        addFragment(
                            ControladorFragment.newInstance(),
                            R.id.main_frame_layout,
                            ControladorFragment.TAG,
                            false
                        )

                       // true
                    }
                    getString(R.string.Paletizadores) -> {
                        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        delete_Fragments()
                        //addFragment(ExpeditionTruckListFragment.newInstance(),R.id.main_frame_layout, ExpeditionTruckListFragment.TAG,false)
                        addFragment(
                            PasilleroFragment.newInstance("Paletizadores"),
                            R.id.main_frame_layout,
                            PasilleroFragment.TAG,
                            false
                        )

                       // true
                    }
                    getString(R.string.Ajustes) -> {
                        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        delete_Fragments()
                        addFragment(
                            AjustesFragment.newInstance(),
                            R.id.main_frame_layout,
                            AjustesFragment.TAG,
                            false
                        )
                    }

                    //else -> false
                }

                true
            }
            bottomMenu!!.setOnNavigationItemReselectedListener {

            }
        }
    }

    private fun delete_Fragments() {
        fm.getFragments().forEach {
            ///Log.i("VERDNATURA:", "si esta ${it.tag.toString()}")
            var fragment: Fragment? = supportFragmentManager.findFragmentByTag(it.tag.toString())
            if (fragment != null) supportFragmentManager
                .beginTransaction().remove(fragment)
                .commit()
        }
    }

    override fun onPasillerosItemClickListener(item: PasillerosItemVO, entryPoint: String) {

        when (item.title) {

            "Asociar Sacador SmartTags" -> {
                addFragmentOnTop(AssociateSmartTagsFragment.newInstance(getString(R.string.SmarttagAssociate)))
            }

            "Registrar SmartTags" -> {
                addFragmentOnTop(RegisterSmartTagsFragment.newInstance(entryPoint))
            }

            //BUFFERS
            "Paletizar" -> {
                addFragmentOnTop(ExpeditionTruckListFragment.newInstance())
            }
            "Consultar expedición" -> {
                // addFragment(ExpeditionStateFragment.newInstance(entryPoint),R.id.main_frame_layout, ExpeditionTruckListFragment.TAG,false)
                addFragmentOnTop(ExpeditionStateFragment.newInstance(entryPoint))
            }
            "Escanear expedición" -> {
                addFragmentOnTop(ExpeditionScanSorterFragment.newInstance(item.title))
            }

            "Gestión de buffers" -> {
                addFragmentOnTop(PasilleroFragment.newInstance("Gestión de buffer"))
            }

            "Control de usuario de vehículo" -> {
                addFragmentOnTop(ControlVehiculoUsuarioFragment.newInstance(entryPoint))
            }

            "Pre Sacador" -> {
                addFragmentOnTop(PreSacadorFragment.newInstance())
            }
            "Día de venta" -> {
                addFragmentOnTop(DayOfSaleFragment("diadeventa"))
            }

            "Ver Ticket" -> {
                addFragmentOnTop(PreControladorFragment.newInstance())
            }
            "Historial del vehículo" -> {
                addFragmentOnTop(ControlVehiculoFragment.newInstance(HistoricoVehiculoFragment.TAG))
                //   fm.popBackStack(null,FragmentManager.POP_BACK_STACK_INCLUSIVE)
                //  addFragment(HistoricoVehiculoFragment.newInstance(entryPoint),R.id.main_frame_layout, ControladorFragment.TAG,false)
            }
            "Reposición" -> {
                addFragmentOnTop(ReposicionFragment.newInstance())
            }
            "Consultar artículo" -> {
                addFragmentOnTop(ItemCardFragment.newInstance(entryPoint))
            }
            //sergio:no visible desde el menú inicio
            "Buscar artículos similares" -> {
                addFragmentOnTop(ItemProposalFragment.newInstance(entryPoint))
            }

            "Parking" -> {
                addFragmentOnTop(ParkingFragment.newInstance())
            }
            "Buscar item" -> {
                addFragmentOnTop(BuscarItemFragment.newInstance(entryPoint))
            }
            // "Buscar item full" -> {
            //   addFragmentOnTop(BuscarItemAllFragment.newInstance(entryPoint))
            // }

            "Historico" -> {
                addFragmentOnTop(HistoricoFragment.newInstance(entryPoint))
            }
            //segio: esta opción no se muestra, solo es accesible para añadir fragmentos de otras partes.
            "Historico Vehículo" -> {
                addFragmentOnTop(HistoricoVehiculoFragment.newInstance(entryPoint))
            }
            "Inventario" -> {
                addFragmentOnTop(InventaryFragment.newInstance())
            }
            "Faltas" -> {
                addFragmentOnTop(FaltasFragment.newInstance())
            }
            "Shelving Parking" -> {
                addFragmentOnTop(ShelvingParkingFragment.newInstance())
            }
            "Ubicador" -> {
                addFragmentOnTop(UbicadorFragment.newInstance(entryPoint))
            }
            "Automatic" -> {
                addFragmentOnTop(AutomaticAddItemFragment.newInstance(entryPoint))
            }
            "Calidad" -> {
                addFragmentOnTop(BuyersFragment.newInstance())
            }
            "Sacadores" -> {
                addFragmentOnTop(SacadorFragment.newInstance())
            }

            //Web fragment
            "Visor web" -> {
                addFragmentOnTop(WebFragment.newInstance("Visor web"))
            }
            //Menu qr
            "Lector Qr" -> {
                addFragmentOnTop(QrFragment.newInstance("Lector Qr"))
            }

            //Menu buffer
            "Vaciado buffer" -> {
                addFragmentOnTop(BufferFragment.newInstance("Vaciado buffer"))
            }
            "Llenado buffer" -> {
                addFragmentOnTop(BufferFragment.newInstance("Llenado buffer"))
            }
            "Acumulación buffer" -> {
                addFragmentOnTop(BufferFragment.newInstance("Acumulación buffer"))
            }
            "Desactivado buffer" -> {
                addFragmentOnTop(BufferFragment.newInstance("Desactivado buffer"))
            }

            /*sergio:pruebas*/
            "Pre Control" -> {
                addFragmentOnTop(PreControladorFragment.newInstance())
            }
            "Histórico Shelving" -> {
                addFragmentOnTop(itemShelvingLogFragment.newInstance("Histórico Shelving"))
            }

        }

    }

    fun addFragmentOnTop(fragment: Fragment?) {

        supportFragmentManager
            .beginTransaction()
            .replace(R.id.main_frame_layout, fragment!!)
            .addToBackStack(null)
            .commitAllowingStateLoss()
    }

    override fun onBackPressed() {

        try {
            fm.executePendingTransactions()
        } catch (e: Exception) {
        }

        if (fm.backStackEntryCount > 0) {

            fm.popBackStackImmediate()

        } else {
            customDialog.setTitle("Cerrar sesión")
                .setDescription("¿Estás seguro de cerrar la sesión?").setOkButton("Salir") {
                    customDialog.dismiss()
                    finish()

                }.setKoButton("Cancelar") {
                    customDialog.dismiss()
                }.show()
        }

    }

    override fun onTruckClickListener(item: ItemExpeditionTruckVO, entryPoint: String) {
        addFragmentOnTop(ExpeditionPalletFragment.newInstance(item))

    }

    override fun onPalletClickListener(itemTruck: ItemExpeditionTruckVO, itemPallet: ItemPalletVO) {
        addFragmentOnTop(ExpeditionScanFragment.newInstance(itemTruck, itemPallet))

    }

    override fun onComprobarPalletViewClickListener(
        itemTruck: ItemExpeditionTruckVO,
        itemPallet: ItemPalletVO
    ) {

        addFragmentOnTop(ExpeditionPalletDetailFragment.newInstance(itemTruck, itemPallet))

    }

    override fun onCollectionSelected(collection: CollectionVO, type: String) {
        //  fm.popBackStack()
        addFragmentOnTop(CollectionFragment.newInstance(collection, type))

    }

    override fun onBuyerSelected(userFk: String) {
        addFragmentOnTop(QaualityFragment.newInstance(userFk))
    }

    fun onVehicleSelected(tag: String) {
        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
        addFragment(
            ControlVehiculoFragment.newInstance(tag),
            R.id.main_frame_layout,
            ControlVehiculoFragment.TAG,
            false
        )
        //sergio: de esta manera la llamada del fragment no se vuelve a repetir.
        // addFragmentOnTop(ControlVehiculoFragment.newInstance(tag))
    }

    fun openFragmentPickers() {
        //delete_Fragments()
        fm.popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
        addFragment(
            SacadorFragment.newInstance(),
            R.id.main_frame_layout,
            SacadorFragment.TAG,
            false
        )
        //addFragmentOnTop(SacadorFragment.newInstance())

    }

    fun openFragmentExpeditionState() {
        //delete_Fragments()
        // fm.popBackStack(null,FragmentManager.POP_BACK_STACK_INCLUSIVE)
        // addFragment(ExpeditionStateFragment.newInstance("main"),R.id.main_frame_layout, ExpeditionStateFragment.TAG,false)

        addFragmentOnTop(ExpeditionStateFragment.newInstance("main"))

    }

}
