package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionPalletDetailBinding
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPalletClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionPalletViewAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletViewListVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_expedition_pallet_detail.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class ExpeditionPalletDetailFragment(
    var itemExpeditionTruckVO: ItemExpeditionTruckVO? = null,
    var itemPallet: ItemPalletVO? = null
) : BaseFragment<FragmentExpeditionPalletDetailBinding, ExpeditionPalletDetailViewModel>(
    ExpeditionPalletDetailViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var SectorDescription = ""
    private var adapter : ExpeditionPalletViewAdapter? = null
    private lateinit var customDialogList: CustomDialogList
    private var onPalletClickListener:OnPalletClickListener? = null

    private lateinit var customDialog: CustomDialog


    companion object {
        fun newInstance(item: ItemExpeditionTruckVO,itemPallet: ItemPalletVO) =
            ExpeditionPalletDetailFragment(item,itemPallet)
    }

    override fun onAttach(context: Context) {
        if (context is OnPalletClickListener) onPalletClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_pallet_detail

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        SectorDescription=prefs.getString(SECTORDESCRIP,"Sin Sector").toString()


        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        expedition_pallet_detail_pallet.text = "Pallet "+itemPallet!!.Pallet
        splash_progress.visibility = View.VISIBLE
        requireActivity().main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = getCURDATE(itemExpeditionTruckVO!!.ETD) + " " + itemExpeditionTruckVO!!.Destino
        setToolBar()
        setEvents()
        viewModel.expeditionPallet_List(user,password,itemPallet!!.Pallet,"expeditionPallet_View")
        super.init()
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        button_ok.setOnClickListener {
            requireActivity().onBackPressed()
            // sergio: en caso de que hay problemas habrá que cambiar aquí la llamada
            // MainActivity.addFragmentOnTop(ExpeditionPalletDetailFragment.newInstance(itemTruck,itemPallet))
            // que llame diractamente al fragment para que no se repita la ultima accion
            ////Log.i("VERDNATURA:", "pulsamos boton OK en detail fragment")

        }
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        val iconEditar : Drawable = resources.getDrawable(R.drawable.ic_mode_edit_black_24dp,resources.newTheme())
        val iconBorrar : Drawable = resources.getDrawable(R.drawable.ic_delete_forever_black_24dp,resources.newTheme())
        val iconPrint : Drawable = resources.getDrawable(R.drawable.ic_print_black_24dp,resources.newTheme())
        listIcons.add(iconEditar)
        listIcons.add(iconBorrar)
        listIcons.add(iconPrint)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.expeditionPallet_List(user,password,itemPallet!!.Pallet,"expeditionPallet_View")
                }else if(item == iconEditar){
                    onPalletClickListener!!.onPalletClickListener(itemExpeditionTruckVO!!,itemPallet!!)
                }else if(item == iconBorrar){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.expeditionPalletDel(user,password,itemPallet!!.Pallet,"expeditionPallet_Del")
                }else if(item == iconPrint){
                    splash_progress.visibility = View.VISIBLE

                    (getString(R.string.Imprimiendo)+" por " +SectorDescription).toast(requireContext())
                    viewModel.expeditionPalletPrintSet(user,password,itemPallet!!.Pallet,sectorFk,"expeditionPallet_PrintSet")

                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun getCURDATE(date:String):String{
        val c = Calendar.getInstance()
        val df = SimpleDateFormat("yyyy-mm-dd HH:mm")
        val df2 = SimpleDateFormat("HH:mm")
        c.time = df.parse(date)
        return df2.format(c.time)
    }

    override fun observeViewModel() {
        with(viewModel){
            loadExpeditionPalletList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {printExpeditionList(it) }
            })

            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    requireActivity().onBackPressed()
                }
            })
        }
    }

    private fun printExpeditionList(it: ItemPalletViewListVO){
        splash_progress.visibility = View.GONE
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = ExpeditionPalletViewAdapter(it.list)
        expedition_pallet_recyclerview.adapter = adapter
        expedition_pallet_recyclerview.layoutManager = lm
    }

}