package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionPalletBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnBarcodeRowClickListener
import es.verdnatura.presentation.common.OnComprobarPalletViewClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPalletClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionPalletAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletListVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemScanList
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_expedition_pallet.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class ExpeditionPalletFragment(
    var itemExpeditionTruckVO: ItemExpeditionTruckVO? = null
) : BaseFragment<FragmentExpeditionPalletBinding, ExpeditionPalletViewModel>(
    ExpeditionPalletViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : ExpeditionPalletAdapter? = null
    private var onPalletClickListener: OnPalletClickListener? = null
    private var onComprobarPalletViewClickListener:OnComprobarPalletViewClickListener? = null
    private lateinit var customDialogList: CustomDialogList
    private var listExpeditions:ArrayList<BarcodeVO> = ArrayList()
    private var expeditionAdapter : BarcodeAdapter? = null
    private lateinit var customDialog: CustomDialog
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null


    companion object {
        fun newInstance(item: ItemExpeditionTruckVO) =
            ExpeditionPalletFragment(item)
    }

    override fun onAttach(context: Context) {
        if (context is OnPalletClickListener) onPalletClickListener = context
        if (context is OnComprobarPalletViewClickListener) onComprobarPalletViewClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_pallet

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        mperror = MediaPlayer.create((activity as MainActivity),R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity),R.raw.ok)
        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        splash_progress.visibility = View.VISIBLE
        requireActivity().main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = getCURDATE(itemExpeditionTruckVO!!.ETD) + " " + itemExpeditionTruckVO!!.Destino
        setToolBar()
        setEvents()
        viewModel.expeditionPallet_List(user,password,itemExpeditionTruckVO!!.truckFk,"expeditionPallet_List")
        ////Log.i("VERDNATURA:","4-listamos listado de pallets para la expedicion")
        super.init()
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        val iconPlus : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())
        listIcons.add(iconReload)
        listIcons.add(iconPlus)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.expeditionPallet_List(user,password,itemExpeditionTruckVO!!.truckFk,"expeditionPallet_List")
                }else if(item == iconPlus){
                    expeditionScanAdd()
                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun getCURDATE(date:String):String{
        val c = Calendar.getInstance()
        val df = SimpleDateFormat("yyyy-mm-dd HH:mm")
        val df2 = SimpleDateFormat("HH:mm")
        c.time = df.parse(date)
        return df2.format(c.time)
    }

    override fun observeViewModel() {
        with(viewModel){
            loadExpeditionPalletList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {printExpeditionList(it) }
            })

            loadScanList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {showScanExpeditions(it) }
            })

            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }
            })


        responseCheckexpeditionScanPut.observe(viewLifecycleOwner, Observer {
            if (it.isError){
                customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                    customDialog.dismiss()
                }.show()
            }else{

                if (it.response=="0") {

                    //"Rutas iguales ${it.response}".toast(requireActivity())
                    ////Log.i("VERDNATURA:","Rutas iguales")
                }
                if (it.response=="1"){
                    mperror?.start()
                    //   "Resultado llamada $it.response".toast(requireActivity())
                    ////Log.i("VERDNATURA:","Rutas distintas")

                }}
        })

        }


    }

    private fun printExpeditionList(it: ItemPalletListVO){
        splash_progress.visibility = View.GONE
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = ExpeditionPalletAdapter(it.list,onPalletClickListener!!,onComprobarPalletViewClickListener!!,itemExpeditionTruckVO!!)
        expedition_pallet_recyclerview.adapter = adapter
        expedition_pallet_recyclerview.layoutManager = lm

    }

    private fun expeditionScanAdd(){
        splash_progress.visibility = View.VISIBLE
        viewModel.expeditionScanAdd(usuario = user,password = password,vPalletFk = "0",vTruckFk = itemExpeditionTruckVO!!.truckFk,"expeditionScan_Add")
    }

    private fun showScanExpeditions(it:ItemScanList){

        //Log.i("VERDNATURA:","VA A ESCANEAR EN PALLETFRAGMENT")
        splash_progress.visibility = View.GONE

        listExpeditions = ArrayList()

        it.list.forEach {
            if(it.expeditionFk != "0")
                listExpeditions.add(BarcodeVO(code = it.expeditionFk))
        }

        customDialogList.setTitle("Expedition ("+toolbar_title.text+")").setOkButton("Comprobar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())


            if (listExpeditions.size > 0){
                onComprobarPalletViewClickListener!!.onComprobarPalletViewClickListener(itemExpeditionTruckVO!!,
                    ItemPalletVO(Pallet = it.list.get(0).palletFk))
            }else{
                customDialog.setTitle("Atención").setDescription("No ha escaneado expediciones").setOkButton("Ok"){
                    customDialog.dismiss()
                }.show()
            }

            customDialogList.dismiss()


        }.setKoButton("Cerrar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
            splash_progress.visibility = View.VISIBLE
            viewModel.expeditionPallet_List(user,password,itemExpeditionTruckVO!!.truckFk,"expeditionPallet_List")
            listExpeditions = ArrayList()
            customDialogList.dismiss()
        }.setValue("").show()

        customDialogList.getEditText().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

        customDialogList.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!customDialogList.getValue().isNullOrEmpty()){
                        try{
                           // val numExpedition = customDialogList.getValue().toDouble()
                            if(customDialogList.getValue().length >= 7){
                                listExpeditions.add(0,BarcodeVO(code = customDialogList.getValue()))
                                customDialogList.setDescription("Total: "+listExpeditions.size)
                                viewModel.expeditionScanPut(user,password,it.list.get(0).palletFk,customDialogList.getValue(),"expeditionScan_Put")
                               ////Log.i("VERDNATURA:"," palletizando ${it.list.get(0).palletFk} expedi: ${customDialogList.getValue()}")
                                viewModel.checkRouteExpeditionScanPut(user,password,it.list.get(0).palletFk,customDialogList.getValue(),"expedition_checkRoute")
                            }else{
                                if (mperror != null) mperror!!.start()
                            }
                        }catch (e:Exception){
                            if (mperror != null) mperror!!.start()
                        }


                    expeditionAdapter!!.notifyDataSetChanged()
                }
                customDialogList.setValue("")
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        expeditionAdapter = BarcodeAdapter(listExpeditions,object: OnBarcodeRowClickListener {
            override fun onBarcodeRowClickListener(item: BarcodeVO) {

            }
        },showDelete = false)
        customDialogList.getRecyclerView().adapter = expeditionAdapter

        customDialogList.getRecyclerView().layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)


    }


}