package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.app.Activity
import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionScanBinding
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogTwoButtons
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionScanAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.*
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_expedition_scan.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class ExpeditionScanFragment (
    var itemExpeditionTruckVO: ItemExpeditionTruckVO? = null,
    var itemPalletVO: ItemPalletVO? = null
) : BaseFragment<FragmentExpeditionScanBinding, ExpeditionScanViewModel>(
    ExpeditionScanViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : ExpeditionScanAdapter? = null
    private var onPalletClickListener: OnPalletClickListener? = null
    private var onComprobarPalletViewClickListener: OnComprobarPalletViewClickListener? = null
    private lateinit var customDialogList: CustomDialogList
    private lateinit var customDialogTwoButtons: CustomDialogTwoButtons
    private var listExpeditions:ArrayList<BarcodeVO> = ArrayList()
    private var expeditionAdapter : BarcodeAdapter? = null
    private lateinit var customDialog: CustomDialog
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null

    companion object {
        fun newInstance(item: ItemExpeditionTruckVO,itemPalletVO: ItemPalletVO) =
            ExpeditionScanFragment(item,itemPalletVO)
    }

    override fun onAttach(context: Context) {
        if (context is OnPalletClickListener) onPalletClickListener = context
        if (context is OnComprobarPalletViewClickListener) onComprobarPalletViewClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_scan

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        mperror = MediaPlayer.create((activity as MainActivity),R.raw.error)

        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        customDialogTwoButtons = CustomDialogTwoButtons(requireContext())
        expedition_scan_detail_pallet.text = "Pallet "+itemPalletVO!!.Pallet
        splash_progress.visibility = View.VISIBLE
        requireActivity().main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = getCURDATE(itemExpeditionTruckVO!!.ETD) + " " + itemExpeditionTruckVO!!.Destino
        setToolBar()
        setEvents()
        viewModel.expeditionScanList(user,password,itemPalletVO!!.Pallet)
        super.init()
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        val iconPlus : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())
        listIcons.add(iconReload)
        listIcons.add(iconPlus)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.expeditionScanList(user,password,itemPalletVO!!.Pallet)
                }else if(item == iconPlus){
                    //Log.i("VERDNATURA:","ENTRAMOS A SCAN ADD")
                    expeditionScanAdd ()

                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun getCURDATE(date:String):String{
        val c = Calendar.getInstance()
        val df = SimpleDateFormat("yyyy-mm-dd HH:mm")
        val df2 = SimpleDateFormat("HH:mm")
        c.time = df.parse(date)
        return df2.format(c.time)
    }

    override fun observeViewModel() {
        with(viewModel){
            loadExpeditionScanList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {printExpeditionList(it) }
            })

            loadScanList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {showScanExpeditions(it) }
            })

            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }
            })

            responseCheckexpeditionScanPut.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                        ////Log.i("VERDNATURA:"," Resultado llamada es ${it.response.toString()}")


                 if (it.response=="0") {

                     //"Rutas iguales ${it.response}".toast(requireActivity())
                     ////Log.i("VERDNATURA:","Rutas iguales")
                    }
                    if (it.response=="1"){
                            mperror?.start()
                         //   "Resultado llamada $it.response".toast(requireActivity())
                         //   customDialog.setTitle("Rutas distintas").setDescription(it.response).setOkButton("Cerrar"){
                          //      customDialog.dismiss()
                           // }.show()
                }}
            })


            responseDeleteScan.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    viewModel.expeditionScanList(user,password,itemPalletVO!!.Pallet)
                }
            })
        }
    }

    private fun printExpeditionList(it: ItemExpeditionScanList){

            splash_progress.visibility = View.GONE
            val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
            adapter = ExpeditionScanAdapter(it.list,object: OnScanLongClickListener{
                override fun onScanLongClickListener(itemExpeditionScanVO: ItemExpeditionScanVO) {
                    customDialogTwoButtons.setTitle("Ticket: "+itemExpeditionScanVO.Ticket+" Pallet: "+itemPalletVO!!.Pallet)
                        .setDescription("Expedition: "+itemExpeditionScanVO!!.expeditionFk)
                        .setOkButton("Borrar"){
                            splash_progress.visibility = View.VISIBLE
                            viewModel.expeditionScanDel(user,password,itemExpeditionScanVO.id)
                            customDialogTwoButtons.dismiss()
                        }.setKoButton("Cancelar"){
                            customDialogTwoButtons.dismiss()
                        }.show()
                }

            })
            expedition_pallet_recyclerview.adapter = adapter
            expedition_pallet_recyclerview.layoutManager = lm


    }

    private fun expeditionScanAdd(){
        splash_progress.visibility = View.VISIBLE
        viewModel.expeditionScanAdd(usuario = user,password = password,vPalletFk = itemPalletVO!!.Pallet,vTruckFk = itemExpeditionTruckVO!!.truckFk)
        //Log.i("VERDNATURA:","entramos a escanear ticket del pallet")

    }

    private fun showScanExpeditions(it: ItemScanList){
        //sergio para smarttags:
        // Log.i("VERDNATURA:","muestra expediciones y a escanear")
        listExpeditions = ArrayList()
        splash_progress.visibility = View.GONE
        it.list.forEach {
            if(it.expeditionFk != "0")
                listExpeditions.add(BarcodeVO(code = it.expeditionFk))
        }
        //sergio:Comprueba si el numero es mayor>0
        customDialogList.setTitle("Pallet"+itemPalletVO!!.Pallet+" ("+toolbar_title.text+")").setOkButton("Comprobar"){
        //Log.i("VERDNATURA:","Añadida etiqueta")

            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())


            if (listExpeditions.size > 0){
                ////Log.i("VERDNATURA:","boton comprobar . si las expediciones >0 muetra DetailFragment")
                onComprobarPalletViewClickListener!!.onComprobarPalletViewClickListener(itemExpeditionTruckVO!!,
                    ItemPalletVO(Pallet = it.list.get(0).palletFk)
                )
            }else{
                customDialog.setTitle("Atención").setDescription("No ha escaneado expediciones").setOkButton("Ok"){
                    customDialog.dismiss()
                }.show()
            }

            customDialogList.dismiss()


        }.setKoButton("Cerrar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
            listExpeditions = ArrayList()
            customDialogList.dismiss()
        }.setValue("").show()

        customDialogList.getEditText().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())



        //sergio: seguimos escaneando id=0 -> sin especificar id=5->next
        customDialogList.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!customDialogList.getValue().isNullOrEmpty()) {

                    ////Log.i("VERDNATURA:", "scan put pallet ${it.list.get(0).palletFk}");
                    ////Log.i("VERDNATURA:", "scan put list ${customDialogList.getValue()}");

                        listExpeditions.add(BarcodeVO(code = customDialogList.getValue()))
                    viewModel.expeditionScanPut(
                        user,
                        password,
                        it.list.get(0).palletFk,
                        customDialogList.getValue()
                    )
                    ////Log.i("VERDNATURA:", "scan put pallet ${it.list.get(0).palletFk}");
                    ////Log.i("VERDNATURA:", "scan put list ${customDialogList.getValue()}");

                    // ////Log.i("VERDNATURA:","misma ruta ?")
                    //sergio:comprueba si es de la misma ruta. Check_route_expedition.
                  checkRouteExpeditionScanPut(user, password,it.list.get(0).palletFk,customDialogList.getValue())

                    ////Log.i("VERDNATURA:", "scan put pallet ${it.list.get(0).palletFk}");
                    ////Log.i("VERDNATURA:", "scan put list ${customDialogList.getValue()}");

                    expeditionAdapter!!.notifyDataSetChanged()

                }
                customDialogList.setValue("")
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        expeditionAdapter = BarcodeAdapter(listExpeditions,object: OnBarcodeRowClickListener {
            override fun onBarcodeRowClickListener(item: BarcodeVO) {

            }
        },showDelete = false)
        customDialogList.getRecyclerView().adapter = expeditionAdapter

        customDialogList.getRecyclerView().layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)


    }

    private fun checkRouteExpeditionScanPut(user: String, password: String, palletFk: String, expedition: String){

       viewModel.checkRouteExpeditionScanPut(user,password,palletFk,expedition)


    }

}