package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.SharedPreferences
import android.os.Bundle
import android.view.View.*
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionscanSorterBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.smarttag.adapter.SmartTagAdapter
import es.verdnatura.presentation.view.feature.smarttag.model.SmartTag
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.component_custom_three_dialog.*
import kotlinx.android.synthetic.main.fragment_collection.*
import kotlinx.android.synthetic.main.fragment_controlador.*
import kotlinx.android.synthetic.main.fragment_controlador.splash_progress
import kotlinx.android.synthetic.main.fragment_expedition_scan.*
import kotlinx.android.synthetic.main.fragment_login.*
import kotlinx.android.synthetic.main.fragment_sacador.*
import kotlinx.android.synthetic.main.fragment_sacador_associatesmarttag.*
import kotlinx.android.synthetic.main.toolbar.*

class ExpeditionScanSorterFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentExpeditionscanSorterBinding, ExpeditionScanSorterViewModel>(
    ExpeditionScanSorterViewModel::class
) {

    private var user = ""
    private var password = ""
    private var goBack: Boolean = false
    override fun getLayoutId(): Int = R.layout.fragment_expeditionscan_sorter
    private lateinit var customDialog: CustomDialog
    private lateinit var customDialogList: CustomDialogList
    private var listExpeditionScan: ArrayList<SmartTag> = ArrayList()
    private var expeditionScanAdapter: SmartTagAdapter? = null
    private var itemScaned: String = ""

    companion object {
        fun newInstance(entryPoint: String) = ExpeditionScanSorterFragment(entryPoint)
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())

        goBack = false
        requireActivity().main_bottom_navigation.visibility = GONE
        splash_progress.visibility = GONE

        setEvents()
        setToolBar()
        showExpeditionScan()
        super.init()
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    private fun setToolBar() {

        toolbar_title.text = entryPoint

    }

    private fun expeditionscan(ExpeditionScan: String) {

        splash_progress.visibility = VISIBLE
        viewModel.expedition_scan(user, password, ExpeditionScan,"expedition_scan")
        itemScaned = ExpeditionScan

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        super.onCreate(savedInstanceState)

    }

    private fun setEvents() {

        backButton.setOnClickListener {
            requireActivity().onBackPressed()

        }

    }


    override fun observeViewModel() {

        with(viewModel) {
            splash_progress.visibility = GONE
            responsescan.observe(viewLifecycleOwner, Observer {

                if (it.isError) {

                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {

                    listExpeditionScan.add(SmartTag(itemScaned, itemScaned))
                    customDialogList.setDescription("Total:" + listExpeditionScan.size)
                    expeditionScanAdapter!!.notifyItemInserted(listExpeditionScan.size-1)
                    customDialogList.setValue("")

                }
                splash_progress.visibility = GONE
            })

        }

    }

    private fun showExpeditionScan() {

        customDialogList.setTitle("Escanea expediciones que no han pasado por el Sorter")
            .setOkButton("Finalizar") {

                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

                /*  if (listExpeditionScan.size > 0) {
                      customDialogList.dismiss()
                      backButton.performClick()

                  } else {*/
                customDialogList.dismiss()
                backButton.performClick()

                // }

            }.setValue("").show()


        customDialogList.getEditText().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

        customDialogList.getEditText().setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5 || actionId == 6) {
                if (customDialogList.getValue().isNotEmpty()) {
                    expeditionscan(customDialogList.getValue())
                }
                customDialogList.setValue("")
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        expeditionScanAdapter =
            SmartTagAdapter(listExpeditionScan, object : OnSmartTagRowClickListener {
                override fun OnSmartTagRowClickListener(item: SmartTag) {
                }
            }, showDelete = false)
        customDialogList.getRecyclerView().adapter = expeditionScanAdapter
        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

}

