package es.verdnatura.presentation.view.feature.paletizador.fragment


import android.content.Intent
import android.content.SharedPreferences
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentItemexpeditionstateCardBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnItemExpeditionStateRowClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.imageview.activity.ImageViewActivity
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionStateAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionStateRow
import es.verdnatura.presentation.view.feature.paletizador.model.itemExpedetionState
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.buyers_fragment.*
import kotlinx.android.synthetic.main.fragment_item_card.*
import kotlinx.android.synthetic.main.fragment_item_card.edit_itemFk
import kotlinx.android.synthetic.main.fragment_item_card.itemcard_image
import kotlinx.android.synthetic.main.fragment_item_card.itemcard_layout
import kotlinx.android.synthetic.main.fragment_item_card.splash_progress
import kotlinx.android.synthetic.main.fragment_itemexpeditionstate_card.*
import kotlinx.android.synthetic.main.toolbar.*
import timber.log.Timber.i

class ExpeditionStateFragment(
    var entryPoint: String = ""
// ) : BaseFragment<FragmentExpeditionstateBinding, ExpeditionStateViewModel>(ExpeditionStateViewModel::class) {
   ) : BaseFragment<FragmentItemexpeditionstateCardBinding, ExpeditionStateViewModel>(ExpeditionStateViewModel::class) {
    private var urlLarge: String = ""
    private var titleImage: String = ""
    private var user = ""
    private var password = ""
    private var warehouseFk = ""
    private var adapter: ExpeditionStateAdapter? = null
    private var listItemsRow: ArrayList<ItemExpeditionStateRow> = ArrayList()
    private var itemScan = ""

    companion object {
        fun newInstance(entryPoint: String) = ExpeditionStateFragment(entryPoint)
    }

    private lateinit var customDialog: CustomDialog

    override fun getLayoutId(): Int = R.layout.fragment_itemexpeditionstate_card

    override fun init() {
        itemcard_layout.visibility = View.GONE
        customDialog = CustomDialog(requireContext())

        setEvents()
        setToolBar(getString(R.string.ExpeditionState))
        super.init()
    }


    private fun setToolBar(title: String) {
        toolbar_title.text = title
    }

    private fun setEvents() {
        edit_itemFk.requestFocus()
        edit_itemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_itemFk.text.toString().isNullOrEmpty()) {
                    getExpedtionState(edit_itemFk.text.toString())
                    itemScan = edit_itemFk.text.toString()
                }
                edit_itemFk.setText("")
                (activity as MainActivity).hideKeyboard(edit_itemFk)
                return@setOnEditorActionListener true
            }
            false
        }


        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        itemcard_image.setOnClickListener {
            val i: Intent = Intent(activity, ImageViewActivity::class.java)
            i.putExtra("url", urlLarge)
            i.putExtra("title", titleImage)
            startActivity(i)
        }
    }


    private fun getExpedtionState(expeditionFk: String) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        // this.itemFk = itemFk
        splash_progress.visibility = View.VISIBLE
        viewModel.expedition_getStateJSON(expeditionFk, user, password,"expedition_getState")

    }


    override fun observeViewModel() {
        with(viewModel) {
            // itemcard.observe(viewLifecycleOwner, Observer {
            itemexpedition.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError) {
                    itemcard_layout.visibility = View.GONE
                    toolbar_title.text = "Estado de la expedición"
                    customDialog.setTitle("Error")
                        .setDescription("No se ha podido obtener el estado de la expedición escaneada. ")
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {

                    if (it != null) {
                        itemcard_layout.visibility = View.VISIBLE
                        setItemExpeditionState(it)



                    } else {
                        itemcard_layout.visibility = View.GONE
                        toolbar_title.text = "Estado de la expedición"
                        customDialog.setTitle("Sin resultados")
                            .setDescription("No hemos podido encontrar el estado de la expedición.")
                            .setOkButton("Cerrar") {
                                customDialog.dismiss()
                            }.show()
                    }

                }
            })

        }
    }


    private fun setItemExpeditionState(itemInfo: itemExpedetionState) {
        setToolBar(getString(R.string.expedition) + itemScan)

        listItemsRow = ArrayList()
        for (i in itemInfo.list) {
            listItemsRow.add(
                ItemExpeditionStateRow(
                    title = i.key,
                    value = i.value.replace(".0",""),
                    isEditable = false
                )
            )
        }

        adapter = ExpeditionStateAdapter(listItemsRow, object :
            OnItemExpeditionStateRowClickListener {
            override fun OnItemExpeditionStateRowClickListener(item: ItemExpeditionStateRow) {

            }
        })

        itemexpeditionstate_recyclerview.adapter = adapter
        itemexpeditionstate_recyclerview.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)


    }


}


