package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionTruckListBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnTruckClickListener
import es.verdnatura.presentation.common.TAG
import es.verdnatura.presentation.view.component.CustomDialogHour
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionListAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckList
import es.verdnatura.presentation.view.feature.sacador.fragment.SacadorFragment
import kotlinx.android.synthetic.main.fragment_expedition_truck_list.*
import kotlinx.android.synthetic.main.toolbar.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList
import java.util.Calendar.getInstance as getInstance1


class ExpeditionTruckListFragment  : BaseFragment<FragmentExpeditionTruckListBinding, ExpeditionTruckListViewModel>(
    ExpeditionTruckListViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : ExpeditionListAdapter? = null
    private lateinit var customDialogHor: CustomDialogHour
    private var onTruckClickListener: OnTruckClickListener? = null


    companion object {//similar a static en Java
        fun newInstance() = ExpeditionTruckListFragment()
    }

    override fun onAttach(context: Context) {
        if (context is OnTruckClickListener) onTruckClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_truck_list

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialogHor = CustomDialogHour(requireContext())
        splash_progress.visibility = View.VISIBLE
        toolbar_title.text = "PScan: "+getCURDATE()
        backButton.visibility = View.GONE
        setToolBar()
        ////Log.i("VERDNATURA:","1-cargamos expediciones")
        viewModel.expeditionTruckList(user,password,"expeditionTruck_List")
        super.init()
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        val iconPlus : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())
        val iconState : Drawable = resources.getDrawable(R.drawable.ic_visibility_white_24dp,resources.newTheme())

        listIcons.add(iconState)
        listIcons.add(iconReload)
        listIcons.add(iconPlus)


        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.expeditionTruckList(user,password,"expeditionTruck_List")
                }else if(item == iconPlus){
                    addTruck()
                }
                if (item == iconState){
                    (activity as MainActivity).openFragmentExpeditionState()
                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun getCURDATE():String{
        val c: Date = getInstance1().getTime()
        val df = SimpleDateFormat("dd/MM/yyyy")
        return df.format(c);
    }

    override fun observeViewModel() {
        with(viewModel){
            loadExpeditionTruckList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {printExpeditionList(it) }
            })
            loadResponseExpeditionAdd.observe(viewLifecycleOwner, Observer { event ->
                viewModel.expeditionTruckList(user,password,"expeditionTruck_List")
            })
        }
    }

    private fun printExpeditionList(it: ItemExpeditionTruckList){
        splash_progress.visibility = View.GONE
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = ExpeditionListAdapter(it.list,onTruckClickListener!!)
        expedition_truck_recyclerview.adapter = adapter
        expedition_truck_recyclerview.layoutManager = lm
        ////Log.i("VERDNATURA:","2-Rellenamos adapter")
    }

    private fun addTruck(){
        customDialogHor.setTitle("Nuevo Camión").setOkButton("Guardar"){
            (activity as MainActivity).hideKeyboard(customDialogHor.getDestinoEditText())
            if (!customDialogHor.getDestinoValue().isNullOrEmpty() && !customDialogHor.getHoraValue().isNullOrEmpty()){
                viewModel.expeditionTruckAdd(user,password,customDialogHor.getHoraValue(),customDialogHor.getDestinoValue(),"expeditionTruck_Add")
                splash_progress.visibility = View.VISIBLE
                customDialogHor.dismiss()
            }
            (activity as MainActivity).hideKeyboard(customDialogHor.getDestinoEditText())
            customDialogHor.dismiss()
        }.setKoButton("Cancelar"){
            customDialogHor.dismiss()
        }.setHoraValue("").setDestinoValue("").show()

        customDialogHor.getDestinoEditText().requestFocus()
        customDialogHor.getDestinoEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogHor.getDestinoValue().isNullOrEmpty() && !customDialogHor.getHoraValue().isNullOrEmpty()){
                    viewModel.expeditionTruckAdd(user,password,customDialogHor.getHoraValue(),customDialogHor.getDestinoValue(),"expeditionTruck_Add")
                    splash_progress.visibility = View.VISIBLE
                    customDialogHor.dismiss()
                }

                return@setOnEditorActionListener true
            }
            (activity as MainActivity).hideKeyboard(customDialogHor.getDestinoEditText())
            false
        }
    }




}











