package es.verdnatura.presentation.view.feature.parking.fragment

import android.annotation.SuppressLint
import android.content.SharedPreferences
import android.media.MediaPlayer
import android.os.Bundle
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentParkingBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnBarcodeRowClickListener
import es.verdnatura.presentation.common.hideKeyboard
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.parking.adapter.ParkingAdapter
import kotlinx.android.synthetic.main.fragment_parking.*
import kotlinx.android.synthetic.main.toolbar.*
import timber.log.Timber

class ParkingFragment : BaseFragment<FragmentParkingBinding,ParkingViewModel>(ParkingViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private lateinit var customDialog: CustomDialog
    private var scanerList:ArrayList<BarcodeVO> = ArrayList()
    private var adapter : ParkingAdapter? = null
    private var numParking : Int = 0
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null


    override fun getLayoutId(): Int = R.layout.fragment_parking
    companion object {
        fun newInstance() = ParkingFragment()
    }


    override fun init() {
        setEvents()
        toolbar_title.text = "Parking"
        //sergio: después de las últimas actualizaciones se queda activo. Se cambia opción visibilidad.
        //viewModel.hideProgressLoading()
        splash_progress.visibility=View.GONE
        setList()
        customDialog = CustomDialog(requireContext())
        super.init()
    }

    private fun setEvents(){
        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!scan_input.text.isNullOrEmpty()) {
                    checkScan(scan_input.text.toString())
                }
                scan_input.setText("")
                requireActivity().hideKeyboard()
                return@setOnEditorActionListener true
            }
            false
        }

        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }
    }

    override fun observeViewModel() {
        with(viewModel){
            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    mperror?.start()
                    splash_progress.visibility = View.GONE
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    numParking -= 1
                    if (numParking <= 0) {
                        splash_progress.visibility = View.GONE
                        viewModel.hideProgressLoading()
                        mpok?.start()
                        setList()
                    }
                }
            })
        }
    }

    private fun setList(){
        scanerList = ArrayList()
        adapter = ParkingAdapter(scanerList,object: OnBarcodeRowClickListener{
            override fun onBarcodeRowClickListener(item: BarcodeVO) {
                scanerList.removeAt(scanerList.indexOf(item))
                adapter?.notifyDataSetChanged()
            }
        })
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

        fragment_parking_scan_list.adapter = adapter
        fragment_parking_scan_list.layoutManager = lm
    }

    private fun checkScan(txtScan:String){
        var isParking:Boolean = false
        if (txtScan.trim().length == 4 && isLetter(txtScan.substring(txtScan.length - 1)) || txtScan.contains("-")){
            isParking = true
        }

        if (isParking){
            numParking = scanerList.size
            scanerList.forEach {
                splash_progress.visibility = View.VISIBLE
                viewModel.parking(
                    usuario = user,
                    password = password,
                    parking = txtScan,
                    scanItem = it.code!!,callFunction = "shelvingPark"
                )
            }
        }else{
            scanerList.add(BarcodeVO(code = txtScan))
            adapter?.notifyDataSetChanged()
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        mperror = MediaPlayer.create((activity as MainActivity),R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity),R.raw.ok)
        super.onCreate(savedInstanceState)
    }

    @Throws(NumberFormatException::class)
    fun isLetter(text: String): Boolean {
        return text.matches("[a-zA-Z ]+".toRegex())
    }


}
