package es.verdnatura.presentation.view.feature.parking.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetUbicadorUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageError
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.common.ResponseItemVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class ParkingViewModel(context: Context) : BaseViewModel() {
    private val getUbicadorUserCase: GetUbicadorUserCase = GetUbicadorUserCase(context)

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    private val _isLoading by lazy { MutableLiveData<Boolean>() }
    val isLoading: LiveData<Boolean> = _isLoading

    private fun showProgressLoading() {
        _isLoading.value = true
    }

    fun hideProgressLoading() {
        _isLoading.value = false
    }


    fun parking(usuario: String,password: String,scanItem: String,parking:String,callFunction:String){
        showProgressLoading()
        getUbicadorUserCase.shelvingPark(usuario,password,scanItem,parking).enqueue(object :
            Callback<String> {
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,t.message!!))
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                }else{
                    if (response.body() == "Finish")
                        _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                    else
                        _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                }
            }
        })
    }


}
