package es.verdnatura.presentation.view.feature.presacador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.FragmentPreSacadorBinding
import es.verdnatura.domain.ConstAndValues.PRESACADOR
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.presacador.adapter.PreSacadorAdapter
import es.verdnatura.presentation.view.feature.presacador.model.PreSacadorItemVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyListVO
import es.verdnatura.presentation.view.feature.sacador.model.SaleVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_pre_sacador.*
import kotlinx.android.synthetic.main.fragment_pre_sacador.fragment_sacador_collections
import kotlinx.android.synthetic.main.fragment_pre_sacador.splash_progress
import kotlinx.android.synthetic.main.fragment_sacador.*
import kotlinx.android.synthetic.main.toolbar.*

class PreSacadorFragment : BaseFragment<FragmentPreSacadorBinding,PreSacadorViewModel>(PreSacadorViewModel::class) {
    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var SectorDescription = ""
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null
    private var sales:ArrayList<PreSacadorItemVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private lateinit var customDialogList: CustomDialogList
    private var saleAdapter: PreSacadorAdapter? = null
    private var ticket:String = ""
    private var lm : LinearLayoutManager? = null
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var storedPosition: Int = 0
    private var storedBackPosition : Int = 0
    private var storedShelvingPosition:Int = 0
    private var itemShelvingFkStored : String = ""
    private var goBack:Boolean = false
    private var goBack2:Boolean = false
    private var listPlacementSupply:ArrayList<BarcodeVO> = ArrayList()
    private var placementSupplyAdapter : BarcodeAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons
    private var token = ""
    private var ticketFk = ""
    //sergio: se añada para leer el articulo leido que era el buyFk
    private var originalItemScan: String = ""

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        token = prefs.getString(TOKEN,"").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        mperror = MediaPlayer.create((activity as MainActivity),R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity),R.raw.ok)
        SectorDescription=prefs.getString(SECTORDESCRIP,"Sin Sector").toString()

        super.onCreate(savedInstanceState)
    }

    override fun init() {
        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        splash_progress.visibility = View.GONE
        toolbar_title.text = getString(R.string.getticketpre)
        setToolBar()
        setEvents()


        super.init()
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()

        }
        //ESCANER =========
        hideKeyboards()
        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {

                if (!scan_input.text.toString().isNullOrEmpty()){
                    if (scan_input.text.contains("-")){
                        //es parking
                        parkingTicket(scan_input.text.toString())
                    }else{
                        if (scan_input.text.length > 6){
                            //es ticket
                            ticketFk = scan_input.text.toString()
                            searchTicket(scan_input.text.toString())
                        }else{
                            //es sale
                            findSale(scan_input.text.toString())
                        }
                    }


                }
                scan_input.setText("")
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            true
        }

        hideKeyboards()

        //LISTA =========
        collection_swipe.setOnRefreshListener {
            splash_progress.visibility = View.VISIBLE
            searchTicket(ticket)
            collection_swipe.isRefreshing = false
        }
    }

    override fun observeViewModel() {
        with(viewModel){
            salesList.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                createSaleList(it)
            })

            placementSuppleyList.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (!goBack) printShelvingResult(it)
                goBack = false
            })

            responseSalixMessage.observe(viewLifecycleOwner, Observer{
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()
                }
            })

            responseCode.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (!goBack2){
                    if (it.isError){
                        customDialog.setTitle("Error").setDescription(it.errorMessage).setKoButton("Cerrar"){
                            scanRequest()
                            customDialog.dismiss()
                        }.show()
                    }else{
                        if (checkItemScan(it.response)){
                            scanRequest()
                            customDialogList.dismiss()
                            mpok?.start()
                            onQuantityOfShelvingSelected(itemShelvingFkStored)
                        }else{
                            customDialogList.setValueTwo("")
                            //showErrorMessage("El resultado del procedimiento barcodeToItem de la etiqueta escaneada es: " +it.response)
                            showErrorMessage("La línea de compra de la etiqueta escaneada no existe.")
                            if (mperror != null) {
                                mperror?.start()
                            }
                        }
                    }
                }

                goBack2 = false

            })
        }
    }

    //CREATE LIST
    private fun searchTicket(ticketFk:String){
        Log.i(TAG,"Escaneamos ticket en previa")
        ticket = ticketFk
        splash_progress.visibility = View.VISIBLE
        viewModel.ticketToPrePrepare(
            usuario = user,
            password = password,
            ticketFk = ticketFk,
            sectorFk = sectorFk
        )
    }

    private fun createSaleList(salesList : List<PreSacadorItemVO>){
        splash_progress.visibility = View.GONE
        if (salesList.isNullOrEmpty()){
            customDialog.setTitle("Pre Sacador").setDescription("No hemos podido obtener movimientos para el ticket escaneado").setKoButton("Cancelar"){
                customDialog.dismiss()
            }.show()
        }else{
            toolbar_title.text = ticket
            sales = ArrayList()
            salesList.forEach {
                sales.add(it)
            }
            lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

            saleAdapter = PreSacadorAdapter(sales,pasillerosItemClickListener!!,object: OnQuantityClickListener{
                override fun onQuantityClick(sale: SaleVO) {
                    sales.forEachIndexed { index, saleVO ->
                        if (saleVO.idMovimiento == sale.saleFk){
                            showQuantityDialog(index)
                        }
                    }
                }

            },object :
                OnSaleClickListener {
                override fun onSaleClick(sale: SaleVO) {
                    sales.forEachIndexed { index, saleVO ->
                        if (saleVO.idMovimiento == sale.saleFk) {
                            if (saleVO.quantity != saleVO.picked) {
                                showScanner(index, saleVO)
                            } else {
                                unMarkLine(index)
                            }
                        }
                    }
                }
            })
            fragment_sacador_collections.adapter = saleAdapter
            fragment_sacador_collections.layoutManager = lm

            setTotalLines()
            if (storedPosition != 0)
                setListPosition(storedPosition)
            else if (storedBackPosition != 0)
                setListPosition(storedBackPosition)

            setScrollListener(lm!!)
        }
    }

    private fun setScrollListener(lm: LinearLayoutManager){
        fragment_sacador_collections.addOnScrollListener(object : RecyclerView.OnScrollListener(){
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                storedBackPosition = lm.findFirstVisibleItemPosition()
                super.onScrolled(recyclerView, dx, dy)
            }
        })
    }

    private fun setTotalLines(){
        var totalMark = 0
        sales.forEach {
            if (it.saldo == it.picked)
                totalMark += 1
        }
        toolbar_title.text = ticket
        toolbar_subtitle.text = "" +totalMark + "/" + sales.size
        if (totalMark == sales.size) {
            "Ticket completo".toast(this.context, Toast.LENGTH_SHORT)
        }

    }

    private fun setListPosition(position:Int){
        storedPosition = position
        fragment_sacador_collections.addViewObserver {
            lm!!.scrollToPositionWithOffset(position,0)
        }
    }

    //SEARCH AND MARK
    private fun findSale(txtscan:String){
        var index = 0
        var isBreak = false
        var isOk = false

        for (saleVO in sales) {
            if(saleVO.quantity != saleVO.picked){
                //1- Por carro
                var shelvingIndex = 0
                for (placementVO in saleVO.carros){
                    if (txtscan.uppercase() == placementVO.shelving.uppercase()){
                        if (mpok != null) mpok!!.start()
                        isOk = true
                        showShelving(index,shelvingIndex)
                        isBreak = true
                        break
                    }
                    shelvingIndex+=1
                }
                if (isBreak) break
            }
            index += 1
        }

        if (!isOk) {
            if (mperror != null) {
                mperror!!.start()
            }
            ("Elemento escaneado no encontrado: "+txtscan).toast(requireContext())
        }

    }

    private fun findSale(txtscan:String,position: Int){
        var index = 0
        var isBreak = false
        var isOk = false

        val saleVO = sales[position]

        if(saleVO.quantity != saleVO.picked){
            //1- Por carro
            var shelvingIndex = 0
            for (placementVO in saleVO.carros){
                if (txtscan.uppercase() == placementVO.shelving.uppercase()){
                    if (mpok != null) mpok!!.start()
                    isOk = true
                    showShelving(position,shelvingIndex)
                    isBreak = true
                    break
                }
                shelvingIndex+=1
            }
        }
        index += 1


        if (!isOk) {
            if (mperror != null) {
                mperror!!.start()
            }
            ("Elemento escaneado no encontrado: "+txtscan).toast(requireContext())
        }

    }

    private fun markLine(position:Int){

        saleAdapter!!.notifyDataSetChanged()
        setListPosition(position)
        saleTrackingReplace(position)
        setTotalLines()
    }

    private fun saleTrackingReplace(position: Int){
        viewModel.saleTrackingReplace(
            usuario = user,
            password = password,
            saleFk = sales[position].idMovimiento,
            vIsChecked = if (sales[position].picked == sales[position].quantity) "1" else "0",
            vOriginalQuantity = sales[position].picked.toString(),
            vStateFk = PRESACADOR,

        //sergio: se añade originalItemScan para
        vBuyFk = originalItemScan,callFunction = "saleTrackingReplace"

        )

       // ////Log.i("VERDNATURA:","El id movimiento es ${sales[position].idMovimiento}")
       // ////Log.i("VERDNATURA:","Checked is ${if (sales[position].picked == sales[position].quantity) "1" else "0"}")
      //  ////Log.i("VERDNATURA:","Position is ${position}")


    }

    private fun unMarkLine(position: Int){

        if (sales[position].quantity == sales[position].picked){
            customDialog.setTitle("Desmarcar linea").setDescription("Vas a desmarcar la linea: "+sales[position].itemFk+ " ¿Estás seguro?").setOkButton("Desmarcar"){
                sales[position].picked = 0
                saleAdapter!!.notifyDataSetChanged()
                setListPosition(position)
                viewModel.saleTrackingDel(
                    usuario = user,
                    password = password,
                    saleFk = sales[position].idMovimiento,"saleTracking_del"
                )
                setTotalLines()
                scanRequest()
                customDialog.dismiss()
            }.setKoButton("Cancelar"){
                scanRequest()
                customDialog.dismiss()
            }.show()
        }



    }

    //SHELVINGS
    private fun showShelving(position:Int,shelvingPosition:Int){
        storedShelvingPosition = shelvingPosition
        storedPosition = position
        splash_progress.visibility = View.VISIBLE
        var quantityGet = "0"
        try{
            quantityGet = (sales[position].saldo - sales[position].picked).toString()
        }catch (e:Exception){}
        viewModel.itemPlacementSupplyAiming(
            usuario = user,
            password = password,
            itemFk = sales[position].itemFk,
            quantity = quantityGet,
            shelvingFk = sales[position].carros[shelvingPosition].shelving,
            callFunction = "itemPlacementSupplyAiming"
        )
    }

    private fun printShelvingResult(placementSupplyListVO: PlacementSupplyListVO){
        var shelving = ""
        var item = ""
        var longName = ""
        var total = "0"
        var itemShelvingFk = "0"
        if (!placementSupplyListVO.list.isEmpty()){
            val placement = placementSupplyListVO.list[0]
            shelving = placement.shelving
            item = placement.itemFk
            longName = placement.longName
            total = placement.total
            itemShelvingFk = placement.itemShelvingFk
        }
        listPlacementSupply = ArrayList()
        placementSupplyListVO.list.forEach {
            listPlacementSupply.add(BarcodeVO(code = it.proposal))
        }

        customDialogList.setTitle("$shelving($item) $total del $longName").setOkButton("Coger"){



            if (customDialogList.getValueTwo().isNotEmpty()){


                if (checkItemScan(customDialogList.getValueTwo())){
                    onQuantityOfShelvingSelected(itemShelvingFk)
                    mpok?.start()
                    customDialogList.dismiss()
                }else{
                    itemShelvingFkStored = itemShelvingFk
                    splash_progress.visibility = View.VISIBLE
                    viewModel.getIdFromCode(
                        usuario = user,
                        password = password,
                        code = customDialogList.getValueTwo(),"barcodeToItem"
                    )
                    customDialogList.dismiss()
                }
                scanRequest()
                hideKeyboards()
            }else{
                "Escanea item para validar".toast(requireContext())
            }


        }.setKoButton("Cerrar"){
            scanRequest()
            hideKeyboards()
            customDialogList.dismiss()
        }.setHintValue("Cantidad que coges:").setValue(total).setHintValueTwo("Escanea item").setValueTwo("").show()


        customDialogList.getEditTextTwo().post(Runnable {
            customDialogList.getEditTextTwo().requestFocusFromTouch()
            val lManager: InputMethodManager =
                requireActivity().getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            lManager.hideSoftInputFromWindow(customDialogList.getEditTextTwo().windowToken, InputMethodManager.SHOW_FORCED)
        })

        customDialogList.getEditTextTwo().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {


                originalItemScan=customDialogList.getValueTwo()
                ////Log.i("VERDNATURA:","el item original es $originalItemScan")

                if (customDialogList.getValueTwo().isNotEmpty()){
                    if (checkItemScan(customDialogList.getValueTwo())){
                        onQuantityOfShelvingSelected(itemShelvingFk)
                        mpok?.start()
                        customDialogList.dismiss()
                    }else{
                        itemShelvingFkStored = itemShelvingFk
                        splash_progress.visibility = View.VISIBLE
                        viewModel.getIdFromCode(
                            usuario = user,
                            password = password,
                            code = customDialogList.getValueTwo(),"barcodeToItem"
                        )
                        customDialogList.dismiss()
                    }
                }else{
                    "Escanea item para validar".toast(requireContext())
                }
                scanRequest()
                hideKeyboards()
                return@setOnEditorActionListener true

            }
            false
        }

        placementSupplyAdapter = BarcodeAdapter(listPlacementSupply,object: OnBarcodeRowClickListener {
            override fun onBarcodeRowClickListener(item: BarcodeVO) {
                placementSupplyListVO.list.forEach {
                    if (it.proposal == item.code){
                        customDialogList.setValue(it.total)
                        total = it.total
                        itemShelvingFk = it.itemShelvingFk
                    }
                }

            }
        },showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    private fun onQuantityOfShelvingSelected(itemShelvingFk:String){
        //1 - MODIFICAR CANTIDAD DEL CARRO
        try {
            val shelvingVisible = sales[storedPosition].carros[storedShelvingPosition].stockTotal

            sales[storedPosition].carros[storedShelvingPosition].stockTotal = (shelvingVisible.toInt() - customDialogList.getValue().toInt()).toString()

            viewModel.itemShelvingSaleSupplyAdd(
                usuario = user,
                password = password,
                itemShelvingFk = itemShelvingFk,
                saleFk = sales[storedPosition].idMovimiento,
                quantity = customDialogList.getValue(),callFunction = "itemShelvingSaleSupplyAdd"
            )

        }catch (e:Exception){}

        //2- MODIFICAR EL PICKED DEL SALE
        try{
            sales[storedPosition].picked = sales[storedPosition].picked + customDialogList.getValue().toInt()
        }catch (e:Exception){}

        //3- MARCAR LINEA
        markLine(storedPosition)


    }

    private fun checkItemScan(valueToCheck:String):Boolean{
        val saleToCheck = sales[storedPosition]
        return saleToCheck.itemFk == valueToCheck
    }

    private fun showScanner(index:Int, sale:PreSacadorItemVO){

        customDialogInput.setTitle(""+sale.itemFk).setDescription("Escanea el carro para el item seleccionado").setOkButton("Aceptar"){
            if (!customDialogInput.getValue().isNullOrEmpty()) {
                findSale(customDialogInput.getValue(),index)
            }
            customDialogInput.setValue("")
            scanRequest()
            customDialogInput.dismiss()
            hideKeyboards()
        }.setKoButton("Cancelar"){
            customDialogInput.dismiss()
        }.setValue("").show()
        customDialogInput.getEditText().requestFocus()
        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    findSale(customDialogInput.getValue(),index)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

    }

    //OPTIONS
    private fun print(){
        viewModel.collectionStickerPrint(
            usuario = user,
            password = password,
            collectionFk = ticket,
            sectorFk = sectorFk,"collectionSticker_print"
        )
        ////Log.i("VERDNATURA:","La collection es $ticket, sectorFk $sectorFk, user $user i pass $password")
        (getString(R.string.Imprimiendo) + " por " + SectorDescription).toast(requireContext())
    }

    //OTROS
    private fun setToolBar(){
        toolbar_subtitle.visibility = View.VISIBLE

        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconPrint : Drawable = resources.getDrawable(R.drawable.ic_print_black_24dp,resources.newTheme())
        val iconParking : Drawable = resources.getDrawable(R.drawable.ic_local_parking_black_24dp,resources.newTheme())
        //sergio para hacer el presacador como sacador
        //val iconAdd : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())

        //val iconTransferir : Drawable = resources.getDrawable(R.drawable.ic_swap_horiz_black_24dp,resources.newTheme())

        listIcons.add(iconPrint)
        listIcons.add(iconParking)
        //listIcons.add(iconAdd)
        //listIcons.add(iconTransferir)

        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconPrint){
                    print()
                }else if (item == iconParking){
                    pasillerosItemClickListener?.onPasillerosItemClickListener(PasillerosItemVO(title = "Parking"),"")
                }
            }
        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun scanRequest(){
        if (scan_input != null) {
            scan_input.requestFocus()
        }
        hideKeyboards()
    }

    private fun hideKeyboards(){
        requireActivity().hideKeyboard()
    }

    private fun parkingTicket(parking:String){
        viewModel.parking(
            usuario = user,
            password = password,
            ticketFk = ticket,
            parking = parking,"shelvingPark"

        )
        ////Log.i("VERDNATURA:","El usuario $user pass $password ticket $ticketFk y parking $parking")
        if (mpok != null) mpok!!.start()
        "Ticket aparcado".toast(requireContext())
    }

    companion object {
        fun newInstance() = PreSacadorFragment()
    }

    override fun getLayoutId(): Int = R.layout.fragment_pre_sacador

    override fun onPause() {
        goBack = true
        goBack2 = true
        super.onPause()
    }

    private fun showErrorMessage(text:String){
        customDialog.setTitle("Error al marcar la linea").setDescription(text).setKoButton("Cerrar"){
            customDialog.dismiss()
        }.show()
    }

    //FALTAS / BASURA / SPLIT
    private fun showQuantityDialog(position:Int) {
        customDialogThreeButtons.setDescription(getString(R.string.txtnuevacantidad)).setValue("")
            .setOkButton(getString(R.string.Faltas)){
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()){
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                }else{
                    trash(position,customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }
            }.setOkButtonTwo(getString(R.string.BasuraRechazar)){
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()){
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                }else{
                    missing(position,customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }
            }.setOkButtonThree(getString(R.string.Reject)){
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()){
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                }else{
                    reject(position,customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }

            }.setOkButtonFour("Split"){
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()){
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                }else{
                    split(position,customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }

            }.setOkButtonAdd(getString(R.string.Agregar)){
                if (customDialogThreeButtons.getValue().trim().isNullOrEmpty()){
                    getString(R.string.Indicanuevacantidad).toast(requireContext())
                }else{
                    increaseQuantity(position,customDialogThreeButtons.getValue())
                    scanRequest()
                    customDialogThreeButtons.dismiss()
                }
            }.setKoButton("Cancelar"){
                scanRequest()
                customDialogThreeButtons.dismiss()
            }.show()
    }

    private fun split(position: Int,quantity:String){
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].saldo.toInt() - quantity.toInt()
        }catch (e:Exception){}
        viewModel.saleMove(
            usuario = user,
            password = password,
            saleFk = sales[position].idMovimiento,
            quantity = totalQuantity.toString(),
            originalQuantity = sales[position].quantity.toString(),"saleMove"
        )
        //sales[position].originalQuantity = quantity
        try{
            sales[position].saldo = quantity.toInt()
        }catch (e:Exception){
            sales[position].saldo = 0
        }
        //sales[position].startQuantity = quantity

        if (quantity == "0")
            markLine(position)
        saleAdapter!!.notifyDataSetChanged()


        //enviar mensaje a salix
        val ticket = "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
        val message = "Se ha enviado a Split el articulo "+sales[position].itemFk+" del ticket "+ticket
        viewModel.sendChekingPresence(token = token,workerId = sales[position].trabajador,message = message, "sendChekingPresence")


    }

    private fun trash(position: Int,quantity:String){
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].saldo.toInt() - quantity.toInt()
        }catch (e:Exception){}
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].idMovimiento,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "FALSE",
            originalQuantity = quantity,callFunction = "collectionMissingTrash"
        )
        //sales[position].originalQuantity = quantity
        try{
            sales[position].saldo = quantity.toInt()
        }catch (e:Exception){
            sales[position].saldo = 0
        }
        //sales[position].startQuantity = quantity
        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position)


        //enviar mensaje a salix
        val ticket = "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
        val message = "Se ha enviado a Faltas la cantidad de "+ totalQuantity +" del articulo "+sales[position].itemFk+"  ticket "+ticket
        viewModel.sendChekingPresence(token = token,workerId = sales[position].trabajador,message = message,"sendChekingPresence")
    }

    private fun missing(position: Int,quantity:String){
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].saldo.toInt() - quantity.toInt()
        }catch (e:Exception){}
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].idMovimiento,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "TRUE",
            originalQuantity = quantity,callFunction = "collectionMissingTrash"
        )
        //sales[position].originalQuantity = quantity
        try{
            sales[position].saldo = quantity.toInt()
        }catch (e:Exception){
            sales[position].saldo = 0
        }

        //sales[position].startQuantity = quantity

        if (quantity == "0")
            markLine(position)
        saleAdapter!!.notifyDataSetChanged()


        //enviar mensaje a salix
        val ticket = "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
        val message = "Se ha enviado a Basura "+ totalQuantity +" del articulo "+sales[position].itemFk+"  ticket "+ticket
        viewModel.sendChekingPresence(token = token,workerId = sales[position].trabajador,message = message,"sendChekingPresence")
    }

    private fun reject(position: Int,quantity:String){
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].saldo.toInt() - quantity.toInt()
        }catch (e:Exception){}
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].idMovimiento,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "reject",
            originalQuantity = quantity,callFunction = "collectionMissingTrash"
        )

        //enviar mensaje a salix
        val ticket = "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
        val message = "Se ha modificado la cantidad original "+sales[position].saldo+" del artículo "+sales[position].itemFk+" a nueva cantidad: "+ quantity +" del ticket "+ticket
        viewModel.sendChekingPresence(token = token,workerId = sales[position].trabajador,message = message,"sendChekingPresence")

        try{
            sales[position].saldo = quantity.toInt()
        }catch (e:Exception){
            sales[position].saldo = 0
        }
        //sales[position].startQuantity = quantity
        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position)



    }

    private fun increaseQuantity(position:Int,quantity:String){
        viewModel.collectionIncreaseQuantity(
            usuario = user,
            password = password,
            saleFk = sales[position].idMovimiento,
            quantity = quantity,
            callFunction = "collection_increaseQuantity"
        )
        try {

            //enviar mensaje a salix
            val ticket = "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
            val message = "Se ha modificado la cantidad original "+sales[position].saldo+" del artículo "+sales[position].itemFk+" a nueva cantidad: "+ quantity +" del ticket "+ticket
            viewModel.sendChekingPresence(token = token,workerId = sales[position].trabajador,message = message,"sendChekingPresence")

            try{
                sales[position].saldo = quantity.toInt()
            }catch (e:Exception){
                sales[position].saldo = 0
            }
        }catch (e:Exception){}
        saleAdapter?.notifyDataSetChanged()



    }

}
