package es.verdnatura.presentation.view.feature.presacador.fragment

import android.content.Context
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetLoginUserCase
import es.verdnatura.domain.GetPreSacadorUseCase
import es.verdnatura.domain.GetSacadorControladorUserCase
import es.verdnatura.domain.GetUbicadorUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageError
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.getMessageFromResponse
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.presacador.model.PreSacadorItemVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyListVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class PreSacadorViewModel(context: Context) : BaseViewModel()
{
    private val getPreSacadorUseCase: GetPreSacadorUseCase = GetPreSacadorUseCase(context)
    private val getSacadorControladorUserCase: GetSacadorControladorUserCase =
        GetSacadorControladorUserCase(context)
    private val getUbicadorUserCase: GetUbicadorUserCase = GetUbicadorUserCase(context)
    private val getLoginUserCase: GetLoginUserCase = GetLoginUserCase(context)

    private val _salesList by lazy { MutableLiveData<List<PreSacadorItemVO>>() }
    val salesList: LiveData<List<PreSacadorItemVO>>
        get() = _salesList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    private val _placementSuppleyList by lazy { MutableLiveData<PlacementSupplyListVO>() }
    val placementSuppleyList: LiveData<PlacementSupplyListVO>
        get() = _placementSuppleyList

    private val _responseCode by lazy { MutableLiveData<ResponseItemVO>() }
    val responseCode: LiveData<ResponseItemVO>
        get() = _responseCode

    private val _responseIncQuantity by lazy { MutableLiveData<ResponseItemVO>() }
    val responseIncQuantity: LiveData<ResponseItemVO>
        get() = _responseIncQuantity

    private val _responseSalixMessage by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSalixMessage: LiveData<ResponseItemVO>
        get() = _responseSalixMessage

    fun ticketToPrePrepare(usuario: String, password: String, ticketFk: String, sectorFk: String)
    {
        getPreSacadorUseCase.ticketToPrePrepare(usuario, password, ticketFk, sectorFk)
            .enqueue(object :
                Callback<List<PreSacadorItemVO>>
            {
                override fun onFailure(call: Call<List<PreSacadorItemVO>>, t: Throwable)
                {
                    _salesList.value = listOf()
                }

                override fun onResponse(
                    call: Call<List<PreSacadorItemVO>>,
                    response: Response<List<PreSacadorItemVO>>
                                       )
                {
                    if (response.body() != null)
                    {
                        _salesList.value = response.body()?.let { it }
                    } else
                    {
                        _salesList.value = listOf()
                    }
                }
            })
    }

    fun collectionStickerPrint(
        usuario: String,
        password: String,
        collectionFk: String,
        sectorFk: String,callFunction:String
                              )
    {
        getSacadorControladorUserCase.collectionStickerPrint(
            usuario,
            password,
            collectionFk,
            sectorFk
                                                            ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun parking(usuario: String, password: String, ticketFk: String, parking: String,callFunction:String)
    {
        getUbicadorUserCase.shelvingPark(usuario, password, ticketFk, parking)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                        )
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun itemPlacementSupplyAiming(
        usuario: String,
        password: String,
        shelvingFk: String,
        quantity: String,
        itemFk: String,callFunction:String
                                 )
    {
        getSacadorControladorUserCase.itemPlacementSupplyAiming(
            usuario,
            password,
            shelvingFk,
            quantity,
            itemFk
                                                               ).enqueue(object :
            Callback<List<PlacementSupplyVO>>
        {
            override fun onFailure(call: Call<List<PlacementSupplyVO>>, t: Throwable)
            {
                val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                listError.add(PlacementSupplyVO(isError = true, errorMessage = getMessageFromAllResponse(callFunction,t.message!!)))
                _placementSuppleyList.value = PlacementSupplyListVO(listError)
            }

            override fun onResponse(
                call: Call<List<PlacementSupplyVO>>,
                response: Response<List<PlacementSupplyVO>>
                                   )
            {
                if (response.body() != null)
                {
                    _placementSuppleyList.value = response.body()?.let { PlacementSupplyListVO(it) }
                } else
                {
                    val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                    listError.add(
                        PlacementSupplyVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                         )
                                 )
                    _placementSuppleyList.value = PlacementSupplyListVO(listError)
                }
            }
        })
    }

    fun itemShelvingSaleSupplyAdd(
        usuario: String,
        password: String,
        itemShelvingFk: String,
        saleFk: String,
        quantity: String,callFunction:String
                                 )
    {
        getSacadorControladorUserCase.itemShelvingSaleSupplyAdd(
            usuario,
            password,
            itemShelvingFk,
            saleFk,
            quantity
                                                               ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(isError = true, errorMessage = getMessageFromAllResponse(callFunction,t.message!!))
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun saleTrackingReplace(
        usuario: String,
        password: String,
        saleFk: String,
        vOriginalQuantity: String,
        vStateFk: String,
        vIsChecked: String,
        vBuyFk: String,callFunction:String
                           )
    {
        //Lo ultimo que se envia
        //sergio: se quita el 0 que habia por defecto para la previa  se añada el buyFk que sera el item escaneado.
        //getSacadorControladorUserCase.saleTrackingReplace(usuario,password,saleFk,vOriginalQuantity,vStateFk,vIsChecked,"0").enqueue(object : Callback<String>{
        getSacadorControladorUserCase.saleTrackingReplace(
            usuario,
            password,
            saleFk,
            vOriginalQuantity,
            vStateFk,
            vIsChecked,
            vBuyFk
                                                         ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun getIdFromCode(usuario: String, password: String, code: String,callFunction:String)
    {
        getSacadorControladorUserCase.getIdFromCode(usuario, password, code)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _responseCode.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                        )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _responseCode.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message()))

                    } else
                    {
                        _responseCode.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun saleTrackingDel(usuario: String, password: String, saleFk: String,callFunction:String){
        getSacadorControladorUserCase.saleTrackingDel(usuario, password, saleFk)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun saleMove(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,
        originalQuantity: String,callFunction:String)
    {
        getSacadorControladorUserCase.saleMove(
            usuario,
            password,
            saleFk,
            quantity,
            originalQuantity
                                              ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun collectionMissingTrash(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,
        type: String,
        warehouseFk: String,
        originalQuantity: String,callFunction:String)
    {
        getSacadorControladorUserCase.collectionMissingTrash(
            usuario,
            password,
            saleFk,
            quantity,
            type,
            warehouseFk,
            originalQuantity
                                                            ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun collectionIncreaseQuantity(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,callFunction:String)
    {
        getSacadorControladorUserCase.collectionIncreaseQuantity(
            usuario,
            password,
            saleFk,
            quantity
                                                                ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responseIncQuantity.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(callFunction,t.message!!)
                                                           )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _responseIncQuantity.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(callFunction,response.message()))
                } else
                {
                    _responseIncQuantity.value =
                        ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun sendChekingPresence(
        token: String,
        workerId: String,
        message: String,
        calledFunction: String
                           )
    {
        try
        {
            getLoginUserCase.sendChekingPresence(token, workerId.toInt(), message)
                .enqueue(object : Callback<Boolean>
                {
                    override fun onResponse(call: Call<Boolean>, response: Response<Boolean>)
                    {
                        if (!response.isSuccessful)
                        {
                            _responseSalixMessage.value = ResponseItemVO(
                                isError = true,
                                errorMessage = "Error en la llamada $calledFunction al enviar mensaje rocket." + getMessageFromResponse(
                                    response.message()
                                                                                                                                       )
                                                                        )
                            //Log.i("Salix","Mensaje enviado a salix")
                        }

                    }

                    override fun onFailure(call: Call<Boolean>, t: Throwable)
                    {

                        Log.i("Salix Error", "" + t.message)
                    }
                })
        } catch (e: Exception)
        {
        }

    }
}
