package es.verdnatura.presentation.view.feature.qr


import android.annotation.SuppressLint
import android.content.SharedPreferences
import android.graphics.Typeface
import android.inputmethodservice.Keyboard
import android.os.Bundle
import android.text.InputType
import android.util.Log
import android.view.KeyEvent
import android.view.KeyEvent.*
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import es.verdnatura.R
import es.verdnatura.databinding.FragmentQrBinding
import es.verdnatura.domain.ConstAndValues
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import kotlinx.android.synthetic.main.fragment_controlador.*
import kotlinx.android.synthetic.main.toolbar.*

class QrFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentQrBinding, QrFragmentViewModel>(QrFragmentViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var goBack: Boolean = false
    override fun getLayoutId(): Int = R.layout.fragment_qr
    private lateinit var customDialog: CustomDialog
    private var type = ""
    private var itemscaned = ""

    companion object {
        fun newInstance(entryPoint: String) = QrFragment(entryPoint)
    }


    override fun init() {

        splash_progress.visibility = View.GONE
        customDialog = CustomDialog(requireContext())
        setToolbar()
        setEvents()
        super.init()
    }

    private fun setToolbar() {

        toolbar_title.text = entryPoint
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        type = ConstAndValues.PRECHECKER
        super.onCreate(savedInstanceState)
    }


    private fun setEvents() {

        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        scan_input.requestFocus()
        scan_input.setOnEditorActionListener { v, actionId, event ->

            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {



                goBack = false
                if (!scan_input.text.isNullOrEmpty()) {
                    splash_progress.visibility = View.VISIBLE
                    viewModel.qr_getCall(user, password, scan_input.text.toString(),"qr_getCall")
                    itemscaned = scan_input.text.toString()
                }

            }
            scan_input.setText("")
            (activity as MainActivity).hideKeyboard(scan_input)
            return@setOnEditorActionListener true
        }
        //false



    }


    override fun observeViewModel() {

        with(viewModel) {
            qrresponse.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError) {

                   // getMessageFromJSON(it.errorMessage)
                    customDialog.setTitle("Error").setDescription( it.errorMessage+  " "+itemscaned)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()
                } else {

                    customDialog.setTitle("Mensaje")
                        .setDescription("Operación " + itemscaned + ". " + it.response)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                            backButton.performClick()
                        }.show()

                }

            })
        }

    }


}

