package es.verdnatura.presentation.view.feature.sacador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.fragment.app.setFragmentResult
import androidx.fragment.app.setFragmentResultListener
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentSacadorBinding
import es.verdnatura.domain.ConstAndValues.SACADOR
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnCollectionSelectedListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.TAG
import es.verdnatura.presentation.common.addFragment
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.login.fragment.LoginFragment
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.sacador.adapter.CollectionAdapter
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import es.verdnatura.presentation.view.feature.smarttag.sacador.fragment.AssociateSmartTagsFragment
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_sacador.*
import kotlinx.android.synthetic.main.toolbar.*

class SacadorFragment :
    BaseFragment<FragmentSacadorBinding, SacadorViewModel>(SacadorViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    private var goBack: Boolean = false
    private var goBack2: Boolean = false
    private lateinit var customDialog: CustomDialog
    private var collectionsList: ArrayList<CollectionVO> = ArrayList()
    private var adapter: CollectionAdapter? = null
    private var wagons = ""
    private var resultTagMode:String? = ""


    companion object {
        fun newInstance() = SacadorFragment()
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun getLayoutId(): Int = R.layout.fragment_sacador

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        wagons = prefs.getInt(WAGON, 2).toString()
        //sergio:listener para collectionnew
       /*  setFragmentResultListener("requestKey") { key, bundle ->
             resultTagMode = bundle.getString("tagMode")
            Log.i("VERDNATURA:", "el result es ${resultTagMode}")
             checkTagsMode()

        }*/

        super.onCreate(savedInstanceState)


    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        splash_progress.visibility = View.VISIBLE
        splash_progress_two.visibility = View.GONE
        toolbar_title.text = getString(R.string.getcollection)
        setToolBar()
        setEvents()
        viewModel.collectionGet(user, password,"collection_get")

        super.init()
    }

    override fun onPause() {
        goBack = true
        goBack2 = true
        super.onPause()
    }

    private fun setToolBar() {

        requireActivity().main_bottom_navigation.visibility = View.VISIBLE
        backButton.visibility = View.GONE
        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconAdd: Drawable =
            resources.getDrawable(R.drawable.ic_add_black_24dp, resources.newTheme())
        val iconVehicleIn: Drawable = resources.getDrawable(R.drawable.car, resources.newTheme())


        listIcons.add(iconVehicleIn)
        listIcons.add(iconAdd)

        toolbar_icons.adapter = ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconAdd) {
                    splash_progress.visibility = View.VISIBLE
                    //viewModel.collectionNew(user, password, sectorFk, wagons)
                     viewModel.collectionNew(user,password,sectorFk,wagons,getTagsType(),"collection_new_wagon")
                      //checkTagsMode()

                }

                if (item == iconVehicleIn) {
                    (activity as MainActivity).onVehicleSelected(SacadorFragment.TAG)
                }
            }
        })
        toolbar_icons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents() {
        sacador_swipe.setOnRefreshListener {
            viewModel.collectionGet(user, password,"collection_get")
            sacador_swipe.isRefreshing = false
        }
    }



    private fun checkTagsMode() {

      //  Log.i("VERDNATURA:", "Tag type: ${getTagsType()}")

        if (getTagsType().equals("Stickers")) {
            //viewModel.collectionNew(user, password, sectorFk, wagons, "Stickers")
            viewModel.collectionNew(user, password, sectorFk, wagons,"collection_new_wagon")
        } else {
         //   Log.i("VERDNATURA:", " el resultTagmode es ${resultTagMode}")
            if (resultTagMode.equals("complete")) {
                viewModel.collectionNew(user, password, sectorFk, wagons, getTagsType(),"collection_new_wagon")
            } else {
                requireActivity().addFragment(
                    AssociateSmartTagsFragment.newInstance(AssociateSmartTagsFragment.TAG),
                    R.id.main_frame_layout,
                    AssociateSmartTagsFragment.TAG
                )
            }

        }


    }



    override fun observeViewModel() {

        with(viewModel) {

            collectionList.observe(viewLifecycleOwner, Observer {
                if (!goBack2) splash_progress.visibility = View.GONE
                createCollectionList(it.list)
                splash_progress.visibility = View.GONE
                goBack2 = false
            })
            /* collectionTicketList.observe(viewLifecycleOwner, Observer {
                 splash_progress_two.visibility = View.GONE
                 if (it.isError){
                     customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Aceptar"){
                         customDialog.dismiss()
                     }.show()
                 }else{
                     if (!goBack)navigateToCollectionList(it)
                 }
                 goBack = false
             })*/
            response.observe(viewLifecycleOwner, Observer {
                splash_progress_two.visibility = View.GONE
                splash_progress.visibility = View.GONE
                if (it.isError) {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                    if (goBack) addCollectionToList(it.response)
                }
                goBack = false
                //sergio:para asegurarnos que aparece en pantalla. En mi movil personal y trabajo no hacia falta.
                viewModel.collectionGet(user,password,"collection_get")
            })
        }
    }

    private fun createCollectionList(collections: List<CollectionVO>) {
        collectionsList = ArrayList()
        collections.forEach {
            collectionsList.add(it)
        }
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = CollectionAdapter(collectionsList, onCollectionSelectedListener!!)
        fragment_sacador_collections.layoutManager = lm
        fragment_sacador_collections.adapter = adapter

    }

    private fun addCollectionToList(collection: String) {
        var isNew = true
        collectionsList.forEach {
            if (it.collectionFk == collection) {
                isNew = false
            }
        }
        if (isNew) {
            collectionsList.add(CollectionVO(collectionFk = collection, created = "Nuevo"))
            adapter?.notifyDataSetChanged()
        } else {
            customDialog.setTitle("Atención")
                .setDescription("La colección " + collection + " sigue pendiente de prepararr")
                .setOkButton("Aceptar") {
                    customDialog.dismiss()
                }.show()
        }
        //sergio:para asegurarnos que se actualiza pantalla.
        adapter?.notifyDataSetChanged()
    }

    private fun navigateToCollectionList(collection: CollectionVO) {
        collectionsList.add(collection)
        adapter?.notifyDataSetChanged()

        onCollectionSelectedListener?.onCollectionSelected(collection, SACADOR)
    }


}
