package es.verdnatura.presentation.view.feature.smarttag.sacador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.View.*
import android.view.inputmethod.EditorInfo
import androidx.activity.OnBackPressedCallback
import androidx.core.os.bundleOf
import androidx.fragment.app.setFragmentResult
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentVehiclecontrolBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.smarttag.adapter.SmartTagAdapter
import es.verdnatura.presentation.view.feature.smarttag.model.SmartTag

import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.component_custom_three_dialog.*
import kotlinx.android.synthetic.main.fragment_collection.*
import kotlinx.android.synthetic.main.fragment_controlador.*
import kotlinx.android.synthetic.main.fragment_controlador.scan_input
import kotlinx.android.synthetic.main.fragment_controlador.splash_progress
import kotlinx.android.synthetic.main.fragment_expedition_scan.*
import kotlinx.android.synthetic.main.fragment_login.*
import kotlinx.android.synthetic.main.fragment_sacador.*
import kotlinx.android.synthetic.main.fragment_sacador_associatesmarttag.*
import kotlinx.android.synthetic.main.toolbar.*
import timber.log.Timber

class AssociateSmartTagsFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentVehiclecontrolBinding, SmartTagsViewModel>(
    SmartTagsViewModel::class
) {

    private var user = ""
    private var userFk = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var goBack: Boolean = false
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    override fun getLayoutId(): Int = R.layout.fragment_sacador_associatesmarttag
    private lateinit var customDialog: CustomDialog
    private lateinit var customDialogList: CustomDialogList
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons
    private var listSmartTags: ArrayList<SmartTag> = ArrayList()
    private var smartAdapter: SmartTagAdapter? = null
    private var itemScaned: String = ""

    companion object {
        fun newInstance(entryPoint: String) = AssociateSmartTagsFragment(entryPoint)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        Timber.i("VERDNATURA:attached")
        val callback: OnBackPressedCallback =
            object : OnBackPressedCallback(true) {
                override fun handleOnBackPressed() {
                    // Leave empty do disable back press or
                    // write your code which you want
                    //Log.i("VERDNATURA:", "PULSADO")

                }
            }
        requireActivity().onBackPressedDispatcher.addCallback(
            this,
            callback
        )

        //if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogList = CustomDialogList(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        goBack = false
        requireActivity().main_bottom_navigation.visibility = GONE
        splash_progress.visibility = GONE

        setEvents()
        setToolBar()

        showScanSmartTags()
        super.init()
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    private fun setToolBar() {

        //  toolbar_title.text = getString(R.string.SmarttagAssociate)
        toolbar_title.text = entryPoint
        Timber.i("VERDNATURA:el entrypoint es ${entryPoint}")

    }

    private fun bindShelving_add(shelvingOrSmarttag: String) {

        splash_progress.visibility = VISIBLE
        //poner el bindshelving aqui

        viewModel.workerShelving_add(user, password, shelvingOrSmarttag)
        if (listSmartTags.size < 1) {

        } else {
            ("http://localhost:7777/bindShelving/${listSmartTags.first().code}" + "&" + shelvingOrSmarttag + "&${listSmartTags.size}").toast(
                requireContext()
            )
        }
        itemScaned = shelvingOrSmarttag

    }

    private fun bindShelving_remove() {

        // splash_progress.visibility = VISIBLE
        //ponser el unbindshleving aqui
        //sergio:esto se debe hacer en

        viewModel.workerShelving_delete(user, password, listSmartTags.last().code.toString())

        ("http://localhost:7777/unbindShelving/${listSmartTags.first().code}" + "&" + listSmartTags.last().code + "&${listSmartTags.size}").toast(
            requireContext()
        )

    }

    private fun scanRequest() {
        if (scan_input != null) {
            scan_input.requestFocus()
        }
        hideKeyboards()
    }

    private fun hideKeyboards() {
        try {
            requireActivity().hideKeyboard()
        } catch (e: Exception) {
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER, 0)
        user = prefs.getString(USER, "").toString()
        password = prefs.getString(PASSWORD, "").toString()
        sectorFk = prefs.getInt(SECTORFK, 1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK, 1).toString()
        userFk = prefs.getString(USERFK, "0").toString()
        super.onCreate(savedInstanceState)

    }

    private fun setEvents() {

        backButton.setOnClickListener {
            Log.i("VERDNATURA:", "PULSADO")
            requireActivity().onBackPressed()
        }

    }

    override fun observeViewModel() {

        with(viewModel) {
            splash_progress.visibility = GONE
            responseadd.observe(viewLifecycleOwner, Observer {
               // Log.i("VERDNATURA::", "Revisar llamada de las etiquetas smarttags : ${it.response}")

                if (it.isError) {

                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                   /* if (it.response != null) {

                        customDialog.setTitle("Error")
                            .setDescription("Etiqueta o carro no válido")
                            .setOkButton("Cerrar") {
                                customDialog.dismiss()
                            }.show()

                    } else {*/

                        //"Etiqueta/matrícula añadida correctamente".toast(context)
                        //var sizeSmartTags = listSmartTags.size + 1

                        var nameSmartTagwithWagon =
                            getTextScaned(0) + itemScaned
                        customDialogList.setTitle(getTextToPosition(0))

                            listSmartTags.add(SmartTag(itemScaned, nameSmartTagwithWagon))
                            smartAdapter!!.notifyDataSetChanged()

                        customDialogList.setValue("")

                  // }
                }
                splash_progress.visibility = GONE
            })
            responseremove.observe(viewLifecycleOwner, Observer {

                if (it.isError) {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                    if (it.response == null) {

                        customDialog.setTitle("Error")
                            .setDescription("Etiqueta o carro no válido")
                            .setOkButton("Cerrar") {
                                customDialog.dismiss()
                            }.show()

                    } else {
                        //var nameSmartTagwithWagon = ""

                     /*   nameSmartTagwithWagon =
                            getTextScaned(listSmartTags.size + 1) + itemScaned
                        customDialogList.setTitle(getTextToPosition(listSmartTags.size + 1))

                        listSmartTags.add(SmartTag(itemScaned, nameSmartTagwithWagon))*/
                        listSmartTags.removeAt(listSmartTags.size - 1)
                        smartAdapter!!.notifyDataSetChanged()

                        customDialogList.setValue("")

                    }
                }
                splash_progress.visibility = GONE
            })
        }

    }

    private fun getTextToPosition(TagsScaned: Int): String {
        var message = ""

        when (TagsScaned) {
            0 -> message = "Escanea etiqueta/s electrónica/s o matrícula/s"
            1 -> message = "Escanea etiqueta superior"
            2 -> message = "Escanea etiqueta del medio"
            3 -> message = "Escanea etiqueta inferior"
            4 -> message = "Carro escaneado"
        }
        return message
    }

    private fun getTextScaned(TagScaned: Int): String {
        var message = ""


        when (TagScaned) {
            0 -> message = "Escaneado:"
            1 -> message = "Carro:"
            2 -> message = "Etiqueta superior:"
            3 -> message = "Escanea del medio:"
            4 -> message = "Etiqueta inferior:"
        }

        return message

    }

    private fun showScanSmartTags() {

        customDialogList.setTitle("Escanea etiqueta/s electrónica/s o matrícula/s.")
            .setOkButton("Finalizar") {

                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

                if (listSmartTags.size > 0) {
                    //  setFragmentResult("requestkey", bundleOf("tagMode" to "SmartTag"))
                    customDialogList.dismiss()

                        val result = "complete"
                        setFragmentResult("requestKey", bundleOf("tagMode" to result))
                       (activity as MainActivity).openFragmentPickers()
                    /*listSmartTags.forEach {
                        Log.i("VERDNATURA:","El codigo es :${it.code}")
                        }
                      workerShelving_add(userFk, it.code!!)*/

                } else {
                    customDialog.setTitle("Atención").setDescription("No ha escaneado Smartags")
                        .setOkButton("Ok")
                        {
                            customDialog.dismiss()

                        }.show()
                }

            }.setKoButton("Deshacer") {
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

                if (!listSmartTags.isEmpty()) {

                    bindShelving_remove()

                } else {
                    customDialogList.dismiss()
                    requireActivity().onBackPressed()

                }

                //
            }.setValue("").show()

        customDialogList.getEditText().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())

        customDialogList.getEditText().setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5 || actionId == 6) {
                if (!customDialogList.getValue().isNullOrEmpty()) {
                    bindShelving_add(customDialogList.getValue())
                }
                customDialogList.setValue("")
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }


        smartAdapter = SmartTagAdapter(listSmartTags, object : OnSmartTagRowClickListener {
            override fun OnSmartTagRowClickListener(item: SmartTag) {
            }
        }, showDelete = false)
        customDialogList.getRecyclerView().adapter = smartAdapter
        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

}

