package es.verdnatura.presentation.view.feature.ubicador.fragment

import android.content.SharedPreferences
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentAutomaticAddItemBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnAutomaticItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogUbicador
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.ubicador.adapter.AutomaticAdapter
import es.verdnatura.presentation.view.feature.ubicador.model.ItemEscanerVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_automatic_add_item.*
import kotlinx.android.synthetic.main.toolbar.*

class AutomaticAddItemFragment(
    var shelvingFk : String = ""
) : BaseFragment<FragmentAutomaticAddItemBinding,AutomaticAddItemViewModel>(AutomaticAddItemViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : AutomaticAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogUbicador : CustomDialogUbicador
    private var listItems:ArrayList<ItemEscanerVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var deep:Int = 1
    private var contador = 0

    companion object {
        fun newInstance(entryPoint:String) = AutomaticAddItemFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_automatic_add_item

    override fun init() {

        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        customDialogUbicador = CustomDialogUbicador(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = "itemShelvingMake"
        setEvents()
        setViews()
        setToolBar()
        super.init()
    }

    private fun setToolBar(){
        toolbar_subtitle.visibility = View.VISIBLE
    }

    private fun setSubtitle(){
        toolbar_subtitle.text = "Etiquetas: "+contador
    }

    private fun setViews(){
        adapter = AutomaticAdapter(listItems,object: OnAutomaticItemClickListener{
            override fun onAutomaticItemClickListener(position: Int) {
                contador -= 1
                setSubtitle()
                if (listItems.size > position){
                    listItems.removeAt(position)
                }

                adapter!!.notifyDataSetChanged()
            }
        })
        items_recyclerview.adapter = adapter
        items_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

    private fun setEvents(){
        edit_matricula.requestFocus()
        edit_matricula.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_matricula.text.toString().isNullOrEmpty()){
                    contador += 1
                    setSubtitle()
                    listItems.add(0,ItemEscanerVO(edit_matricula.text.toString()))
                    adapter!!.notifyDataSetChanged()
                }
                edit_matricula.setText("")
                (activity as MainActivity).hideKeyboard(edit_matricula)
                return@setOnEditorActionListener true
            }

            return@setOnEditorActionListener false
        }

        button_finalizar.setOnClickListener {
            val listString:ArrayList<String> = ArrayList()
            listItems.forEach {
                listString.add(it.item)
            }
            listItems = ArrayList()
            splash_progress.visibility = View.VISIBLE
            viewModel.itemShelvingMake_multi(user,password,shelvingFk,listString,deep.toString(),warehouseFk,"1")
        }

        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }
    }

    override fun observeViewModel() {
        with(viewModel){
            response.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    requireActivity().onBackPressed()
                }
            })
        }
    }

}
