package es.verdnatura.presentation.view.feature.ubicador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.Toast
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentUbicadorBinding
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnMoreClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.OnVisibleClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogTwoButtons
import es.verdnatura.presentation.view.component.CustomDialogUbicador
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.ubicador.adapter.UbicadorAdapter
import es.verdnatura.presentation.view.feature.ubicador.model.ItemUbicadorVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_ubicador.*
import kotlinx.android.synthetic.main.toolbar.*

class UbicadorFragment(
    var shelvingFk:String = ""
) : BaseFragment<FragmentUbicadorBinding, UbicadorViewModel>(
    UbicadorViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : UbicadorAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogUbicador : CustomDialogUbicador
    private lateinit var customDialogTwoButtons:CustomDialogTwoButtons
    private var listItems:ArrayList<ItemUbicadorVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var deep:Int = 1
    private var parking: String = ""
    private var etiquetas:String = "0"
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null


    companion object {
        fun newInstance(entryPoint:String) = UbicadorFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_ubicador

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun init() {

        val prefs: SharedPreferences = requireActivity().getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        customDialogUbicador = CustomDialogUbicador(requireContext())
        customDialogTwoButtons = CustomDialogTwoButtons(requireContext())
        requireActivity().main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = getString(R.string.locator)
        setEvents()
        setToolBar()
        if (!shelvingFk.isNullOrEmpty()){
            splash_progress.visibility = View.VISIBLE
            toolbar_title.text = shelvingFk
            viewModel.itemShelvingList(user,password,shelvingFk,"itemShelvingList")
        }
        super.init()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        mperror = MediaPlayer.create((activity as MainActivity),R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity),R.raw.ok)
        super.onCreate(savedInstanceState)
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconAdd : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        listIcons.add(iconAdd)
        listIcons.add(iconReload)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconAdd){
                    showAddItem("","","","",false,ItemUbicadorVO(),"")
                }else if(item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.itemShelvingList(user,password,shelvingFk,"itemShelvingList")
                }
            }

        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        edit_matricula.requestFocus()
        edit_matricula.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_matricula.text.toString().isNullOrEmpty()){
                    toolbar_title.text = edit_matricula.text.toString()
                    shelvingFk = edit_matricula.text.toString()
                    splash_progress.visibility = View.VISIBLE
                    viewModel.itemShelvingList(user,password,edit_matricula.text.toString(),"itemShelvingList")
                }

                edit_matricula.setText("")
                (activity as MainActivity).hideKeyboard(edit_matricula)
                return@setOnEditorActionListener true
            }

            return@setOnEditorActionListener false
        }

        edit_prioridad.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_prioridad.text.toString().isNullOrEmpty()){
                    viewModel.shelvingPriorityUpdate(user,password,edit_prioridad.text.toString(),shelvingFk,"shelvingPriority_update")
                }

                (activity as MainActivity).hideKeyboard(edit_prioridad)
                return@setOnEditorActionListener true
            }

            return@setOnEditorActionListener false
        }


        automatic_img.setOnClickListener {
            if (!shelvingFk.isNullOrEmpty())
                (activity as MainActivity).onPasillerosItemClickListener(PasillerosItemVO(title = "Automatic"),entryPoint = shelvingFk)
            else
                "Escanea una matrícula".toast(requireActivity(),Toast.LENGTH_SHORT)
        }

        parking_img.setOnClickListener {
            customDialogInput.setTitle("Aparcar carro").setValue("").setOkButton("Aparcar"){
                viewModel.shelvingPark(user,password,shelvingFk,customDialogInput.getValue(),"shelvingPark")
                parking = customDialogInput.getValue()
                toolbar_title.text = shelvingFk.uppercase() + " P: "+parking + " E: "+etiquetas
                if(mpok != null) mpok!!.start()
                customDialogInput.dismiss()
            }.setKoButton("Cancelar"){
                customDialogInput.dismiss()
            }.show()

            customDialogInput.getEditText().requestFocus()
            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())

            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                    viewModel.shelvingPark(user,password,shelvingFk,customDialogInput.getValue(),"shelvingPark")
                    customDialogInput.dismiss()
                    parking = customDialogInput.getValue()
                    if (toolbar_title != null && shelvingFk != null && parking != null && etiquetas != null){
                        toolbar_title.text = shelvingFk.uppercase() + " P: "+parking+ " E: "+etiquetas
                    }else{
                        toolbar_title.text = ""
                    }
                    if (mpok != null) {
                        mpok!!.start()
                    }

                    return@setOnEditorActionListener true
                }
                false
            }

        }

        edit_img.setOnClickListener {
            customDialogInput.setTitle("Cambiar matrícula").setValue("").setOkButton("Guardar"){
                viewModel.shelvingChange(user,password,shelvingFk,customDialogInput.getValue(),"shelvingChange")
                shelvingFk = customDialogInput.getValue()
                toolbar_title.text = shelvingFk.uppercase() + " P: "+parking+ " E: "+etiquetas
                if(mpok != null) mpok!!.start()
                customDialogInput.dismiss()
            }.setKoButton("Cancelar"){
                customDialogInput.dismiss()
            }.show()

            customDialogInput.getEditText().requestFocus()
            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())

            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                    viewModel.shelvingChange(user,password,shelvingFk,customDialogInput.getValue(),"shelvingChange")
                    shelvingFk = customDialogInput.getValue()
                    toolbar_title.text = shelvingFk.uppercase() + " P: "+parking+ " E: "+etiquetas
                    if(mpok != null) mpok!!.start()
                    customDialogInput.dismiss()
                    return@setOnEditorActionListener true
                }
                false
            }

        }

        delete_img.setOnClickListener {
            customDialog.setTitle("Vaciar carro").setDescription("Esta operación no se puede rectificar.").setOkButton("Vaciar"){
                viewModel.clearShelvingList(user,password,shelvingFk,"clearShelvingList")
                listItems.forEach {
                    it.stickers = "0"
                    it.visible = "0"
                }
                if (adapter != null){
                    adapter!!.notifyDataSetChanged()
                }
                customDialog.dismiss()
            }.setKoButton("Cancelar"){
                customDialog.dismiss()
            }.show()


        }

    }

    override fun observeViewModel() {
        with(viewModel){
            loadShelvingList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {
                    deep = it.list.size
                    listItems = ArrayList()
                    var contEtiquetas:Int = 0
                    it.list.forEach { item ->
                        try{
                            contEtiquetas += item.stickers.toInt()
                        }catch (e: Exception) {}

                        parking = item.code.uppercase()
                        listItems.add(item)
                    }
                    etiquetas = contEtiquetas.toString()
                    toolbar_title.text = shelvingFk.uppercase() + " P: "+parking+ " E: "+etiquetas
                    if(it.list.size > 0)
                        edit_prioridad.setText(it.list[0].priority)
                    splash_progress.visibility = View.GONE

                    //sergio: para mostrar CustomDialogUbicator, ubicador con grouping
                    adapter = UbicadorAdapter(listItems,onPasillerosItemClickListener = pasillerosItemClickListener!!,
                        onVisibleClickListener = object: OnVisibleClickListener{
                            override fun onVisibleClickListener(item: ItemUbicadorVO) {

                               // if (item.grouping.isNullOrEmpty()){item.grouping=""}
                                showAddItem(item.item,item.packing,item.stickers,item.visible,true,item,
                                    grouping=if (item.grouping.isNullOrEmpty()) { "" } else item.grouping)
                                
                            }

                        },onMoreClickListener = object: OnMoreClickListener{
                            override fun onMoreClickListener(item: ItemUbicadorVO) {
                                showMoreOptions(item)
                            }

                        })

                    location_recyclerview.adapter = adapter
                    location_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, true)

                }
            })

            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    mperror!!.start()
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }
            })

            responsepriority.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    mperror!!.start()
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()
                }else{
                    (it.response).toast(requireContext())
                }
            })

            responseMake.observe(viewLifecycleOwner, Observer {
                if (!shelvingFk.isNullOrEmpty()){
                    splash_progress.visibility = View.VISIBLE
                    toolbar_title.text = shelvingFk
                    viewModel.itemShelvingList(user,password,shelvingFk,"itemShelvingList")
                }
            })
        }
    }

    private fun showAddItem(itemFk:String,packing: String,etiquetas: String,visible: String, delete:Boolean,itemUbicador:ItemUbicadorVO,grouping:String){
        var title = "Nuevo Item"
        if (delete) title = "Editar item "+itemFk
        customDialogUbicador.setTitle(title).setItemValue(itemFk).setPackingValue(packing).setGroupingValue(grouping).setEtiquetaValue(etiquetas).setVisibleValue(visible).setOkButton("Guardar"){
            var packingUpdate = false
            if (packing != customDialogUbicador.getPackingValue() || etiquetas != customDialogUbicador.getEtiquetaValue()){
                packingUpdate = true
            }
            saveNewItem(customDialogUbicador.getItemValue(),customDialogUbicador.getEtiquetaValue(),customDialogUbicador.getPackingValue(),customDialogUbicador.getVisibleValue(),if (delete)packingUpdate else null,itemUbicador.id,customDialogUbicador.getGroupingValue())
        }.setKoButton("Cerrar"){
            customDialogUbicador.dismiss()
        }.show()

        customDialogUbicador.getEditItem().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())

        customDialogUbicador.getVisibleItem().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                var packingUpdate = false
                if (packing != customDialogUbicador.getPackingValue() || etiquetas != customDialogUbicador.getEtiquetaValue()){
                    packingUpdate = true
                }
                saveNewItem(customDialogUbicador.getItemValue(),customDialogUbicador.getEtiquetaValue(),customDialogUbicador.getPackingValue(),customDialogUbicador.getVisibleValue(),if (delete)packingUpdate else null,itemUbicador.id,customDialogUbicador.getGroupingValue())
                return@setOnEditorActionListener true
            }
            false
        }

        customDialogUbicador.getEtiquetaItem().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                var packingUpdate = false
                if (packing != customDialogUbicador.getPackingValue() || etiquetas != customDialogUbicador.getEtiquetaValue()){
                    packingUpdate = true
                }
                saveNewItem(customDialogUbicador.getItemValue(),customDialogUbicador.getEtiquetaValue(),customDialogUbicador.getPackingValue(),customDialogUbicador.getVisibleValue(),if (delete)packingUpdate else null,itemUbicador.id,customDialogUbicador.getGroupingValue())
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun saveNewItem(itemFk:String,etiquetas:String,packing:String,visible:String,updatePacking:Boolean? = null,itemShelvingFk:String,groupingPack:String){
            if (!packing.isNullOrEmpty()){
                if (updatePacking == null){
                    if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty()){
                        viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible,packing,warehouseFk,"1","itemShelvingMake",groupingPack)
                        changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible,packing = packing))
                        customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("").setEtiquetaValue("").setVisibleValue("")
                        customDialogUbicador.getEditItem().requestFocus()
                        (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
                        customDialogUbicador.dismiss()

                    }else if(!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty()){
                        val visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                        viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible_calc,packing,warehouseFk,"1","itemShelvingMake",groupingPack)
                        changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible_calc,packing = packing,stickers = etiquetas))
                        customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("").setEtiquetaValue("").setVisibleValue("")
                        customDialogUbicador.getEditItem().requestFocus()
                        (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
                        customDialogUbicador.dismiss()

                    }else{
                        "Rellena campos obligatorios".toast((activity as MainActivity), Toast.LENGTH_LONG)
                    }
                }else{
                    if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty() && !updatePacking){
                        viewModel.itemShelvingMakeEdit(user,password,shelvingFk,itemFk,deep.toString(),visible,packing,warehouseFk,"1",itemShelvingFk,"itemShelvingMakeEdit",groupingPack)
                        changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible,packing = packing))
                        customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("").setEtiquetaValue("").setVisibleValue("")
                        customDialogUbicador.getEditItem().requestFocus()
                        (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
                        customDialogUbicador.dismiss()

                    }else if(!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty() && updatePacking){
                        var visible_calc = "0"
                        try{
                            visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                        }catch (e:Exception){}
                        viewModel.itemShelvingMakeEdit(user,password,shelvingFk,itemFk,deep.toString(),visible_calc,packing,warehouseFk,"1",itemShelvingFk,"itemShelvingMakeEdit",groupingPack)
                        changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible_calc,packing = packing,stickers = etiquetas))
                        customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("").setEtiquetaValue("").setVisibleValue("")
                        customDialogUbicador.getEditItem().requestFocus()
                        (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
                        customDialogUbicador.dismiss()

                    }else{
                        "Rellena campos obligatorios".toast((activity as MainActivity), Toast.LENGTH_LONG)
                    }
                }

            }else{
                "Packing obligatorio".toast((activity as MainActivity), Toast.LENGTH_LONG)
            }
    }

   /* private fun editItem(itemFk:String,etiquetas:String,packing:String,visible:String){
        if (!packing.isNullOrEmpty()){
            if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty()){
                viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible,packing,warehouseFk,"1","itemShelvingMake")
                changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible,packing = packing))
                customDialogUbicador.setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("")
                customDialogUbicador.getEditItem().requestFocus()
                (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
            }else if(!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty()){
                val visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible_calc,packing,warehouseFk,"1","itemShelvingMake")
                changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible_calc,packing = packing,stickers = etiquetas))
                customDialogUbicador.setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("")
                customDialogUbicador.getEditItem().requestFocus()
                (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
            }else{
                "Rellena campos obligatorios".toast((activity as MainActivity), Toast.LENGTH_LONG)
            }
        }else{
            "Packing obligatorio".toast((activity as MainActivity), Toast.LENGTH_LONG)
        }
    }*/

    private fun changeOfflineValue(item:ItemUbicadorVO){
        if (splash_progress != null)  splash_progress.visibility = View.VISIBLE
        /*listItems.add(item)
        adapter?.notifyDataSetChanged()*/
    }

    private fun showMoreOptions(item:ItemUbicadorVO){
        customDialogTwoButtons.setTitle(item.item).setDescription("Selecciona una acción").setOkButton("Transferir"){
            customDialogInput.setTitle("Nueva matrícula").setDescription("Introduce el destino para item: "+item.item).setValue("").setOkButton("Transferir"){
                if (customDialogInput.getValue().isNotEmpty()){
                    viewModel.itemShelvingTransfer(user,password,item.id,customDialogInput.getValue(),"itemShelvingTransfer")
                    listItems.remove(item)
                    adapter!!.notifyDataSetChanged()
                    customDialogInput.dismiss()
                }else{
                    "Carro incorrecto".toast(requireContext())
                }



            }.setKoButton("Cancelar"){
                customDialogInput.dismiss()
            }.show()
            customDialogInput.getEditText().requestFocus()
            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                    viewModel.itemShelvingTransfer(user,password,item.id,customDialogInput.getValue(),"itemShelvingTransfer")
                    listItems.remove(item)
                    adapter!!.notifyDataSetChanged()
                    customDialogInput.dismiss()
                    return@setOnEditorActionListener true
                }
                false
            }
            customDialogTwoButtons.dismiss()
        }.setOkButtonTwo("Eliminar"){
            customDialog.setTitle("Confirmar").setDescription("Confirmas eliminar el artículo: "+item.item+"?").setOkButton("Eliminar"){
                listItems.remove(item)
                adapter!!.notifyDataSetChanged()
                viewModel.itemShelvingDelete(user,password,item.id,"itemShelvingDelete")
                customDialogTwoButtons.dismiss()
                customDialog.dismiss()
            }.setKoButton("Cancelar"){
                customDialog.dismiss()
                customDialogTwoButtons.dismiss()
            }.show()

        }.setKoButton("Cancelar"){
            customDialogTwoButtons.dismiss()
        }.show()
    }
}